@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -6,6 +6,7 @@
 #ifndef _DEFAULT_OPTIONS_H_
 #define _DEFAULT_OPTIONS_H_
 
+#include "GmshConfig.h"
 #include "GmshDefines.h"
 #include "Options.h"
 
@@ -37,6 +38,9 @@ StringXString GeneralOptions_String[] = {
   { F|O, "AxesLabelZ" , opt_general_axes_label2 , "" , 
     "Z-axis label" },
 
+  { F|O, "BackgroundImageFileName" , opt_general_background_image_filename , "" ,
+    "Background image file in JPEG or PNG format" },
+
   { F|O, "DefaultFileName" , opt_general_default_filename , "untitled.geo" ,
     "Default project file name" },
   { F,   "Display" , opt_general_display , "" ,
@@ -81,15 +85,15 @@ StringXString GeneralOptions_String[] = {
 #endif
     "System command to launch a web browser" },
 
-  { 0, NULL , NULL , NULL , NULL }
+  { 0, 0 , 0 , "" , 0 }
 } ;
 
 StringXString GeometryOptions_String[] = {
-  { 0, NULL , NULL , NULL , NULL }
+  { 0, 0 , 0 , "" , 0 }
 } ;
 
 StringXString MeshOptions_String[] = {
-  { 0, NULL , NULL , NULL , NULL }
+  { 0, 0 , 0 , "" , 0 }
 } ;
 
 StringXString SolverOptions_String[] = {
@@ -105,7 +109,7 @@ StringXString SolverOptions_String[] = {
     "Name of solver 0" },
   { F|O, "Help0" , opt_solver_help0 , 
     "A General environment for the treatment of\nDiscrete Problems\n\n"
-    "Copyright (C) 1997-2008\nPatrick Dular and Christophe Geuzaine\n\n"
+    "Copyright (C) 1997-2009\nPatrick Dular and Christophe Geuzaine\n\n"
     "Visit http://www.geuz.org/getdp/ for more info",
     "Help string for solver 0" },
   { F|O, "Executable0" , opt_solver_executable0 , 
@@ -354,11 +358,11 @@ StringXString SolverOptions_String[] = {
   { F|O, "FifthButtonCommand4" , opt_solver_fifth_button_command4 , "" ,
     "Command associated with the fifth button for solver 4" },
 
-  { 0, NULL , NULL , NULL , NULL }
+  { 0, 0 , 0 , "" , 0 }
 } ;
 
 StringXString PostProcessingOptions_String[] = {
-  { 0, NULL , NULL , NULL , NULL }
+  { 0, 0 , 0 , "" , 0 }
 } ;
 
 StringXString ViewOptions_String[] = {
@@ -411,11 +415,11 @@ StringXString ViewOptions_String[] = {
   { F|O, "Stipple9" , opt_view_stipple9 , "1*0xFFFF" , 
     "Tenth stippling pattern" },
 
-  { 0, NULL , NULL , NULL , NULL }
+  { 0, 0 , 0 , "" , 0 }
 } ;
 
 StringXString PrintOptions_String[] = {
-  { 0, NULL , NULL , NULL , NULL }
+  { 0, 0 , 0 , "" , 0 }
 } ;
 
 // NUMBERS
@@ -458,9 +462,11 @@ StringXNumber GeneralOptions_Number[] = {
 
   { F|O, "BackgroundGradient" , opt_general_background_gradient , 1. ,
     "Draw background gradient (0=none, 1=vertical, 2=horizontal, 3=radial)" },
+  { F|O, "BackgroundImagePositionX" , opt_general_background_image_position0 , 1.e5 ,
+    "X position (in pixels) of background image (< 0: measure from right edge; >= 1e5: centered)" },
+  { F|O, "BackgroundImagePositionY" , opt_general_background_image_position1 , 1.e5 ,
+    "Y position (in pixels) of background image (< 0: measure from bottom edge; >= 1e5: centered)" },
 
-  { F,   "Clip0" , opt_general_clip0 , 0. ,
-    "Enable clipping plane 0 (Geometry=2^0, Mesh=2^1, View[i]=2^(2+i))" },
   { F,   "Clip0A" , opt_general_clip0a , 1.0 ,
     "First coefficient in equation for clipping plane 0 (`A' in `AX+BY+CZ+D=0')" },
   { F,   "Clip0B" , opt_general_clip0b , 0.0 , 
@@ -469,29 +475,23 @@ StringXNumber GeneralOptions_Number[] = {
     "Third coefficient in equation for clipping plane 0 (`C' in `AX+BY+CZ+D=0')" },
   { F,   "Clip0D" , opt_general_clip0d , 0.0 , 
     "Fourth coefficient in equation for clipping plane 0 (`D' in `AX+BY+CZ+D=0')" },
-  { F,   "Clip1" , opt_general_clip1 , 0.,
-    "Enable clipping plane 1 (Geometry=2^0, Mesh=2^1, View[i]=2^(2+i))" },
-  { F,   "Clip1A" , opt_general_clip1a , 1.0 , 
+  { F,   "Clip1A" , opt_general_clip1a , 0.0 , 
     "First coefficient in equation for clipping plane 1" },
-  { F,   "Clip1B" , opt_general_clip1b , 0.0 , 
+  { F,   "Clip1B" , opt_general_clip1b , 1.0 , 
     "Second coefficient in equation for clipping plane 1" },
   { F,   "Clip1C" , opt_general_clip1c , 0.0 , 
     "Third coefficient in equation for clipping plane 1" },
   { F,   "Clip1D" , opt_general_clip1d , 0.0 , 
     "Fourth coefficient in equation for clipping plane 1" },
-  { F,   "Clip2" , opt_general_clip2 , 0.,
-    "Enable clipping plane 2 (Geometry=2^0, Mesh=2^1, View[i]=2^(2+i))" },
-  { F,   "Clip2A" , opt_general_clip2a , 1.0 , 
+  { F,   "Clip2A" , opt_general_clip2a , 0.0 , 
     "First coefficient in equation for clipping plane 2" },
   { F,   "Clip2B" , opt_general_clip2b , 0.0 , 
     "Second coefficient in equation for clipping plane 2" },
-  { F,   "Clip2C" , opt_general_clip2c , 0.0 , 
+  { F,   "Clip2C" , opt_general_clip2c , 1.0 , 
     "Third coefficient in equation for clipping plane 2" },
   { F,   "Clip2D" , opt_general_clip2d , 0.0 , 
     "Fourth coefficient in equation for clipping plane 2" },
-  { F,   "Clip3" , opt_general_clip3 , 0.,
-    "Enable clipping plane 3 (Geometry=2^0, Mesh=2^1, View[i]=2^(2+i))" },
-  { F,   "Clip3A" , opt_general_clip3a , 1.0 , 
+  { F,   "Clip3A" , opt_general_clip3a , -1.0 , 
     "First coefficient in equation for clipping plane 3" },
   { F,   "Clip3B" , opt_general_clip3b , 0.0 , 
     "Second coefficient in equation for clipping plane 3" },
@@ -499,23 +499,19 @@ StringXNumber GeneralOptions_Number[] = {
     "Third coefficient in equation for clipping plane 3" },
   { F,   "Clip3D" , opt_general_clip3d , 0.0 , 
     "Fourth coefficient in equation for clipping plane 3" },
-  { F,   "Clip4" , opt_general_clip4 , 0.,
-    "Enable clipping plane 4 (Geometry=2^0, Mesh=2^1, View[i]=2^(2+i))" },
-  { F,   "Clip4A" , opt_general_clip4a , 1.0 , 
+  { F,   "Clip4A" , opt_general_clip4a , 0.0 , 
     "First coefficient in equation for clipping plane 4" },
-  { F,   "Clip4B" , opt_general_clip4b , 0.0 , 
+  { F,   "Clip4B" , opt_general_clip4b , -1.0 , 
     "Second coefficient in equation for clipping plane 4" },
   { F,   "Clip4C" , opt_general_clip4c , 0.0 , 
     "Third coefficient in equation for clipping plane 4" },
   { F,   "Clip4D" , opt_general_clip4d , 0.0 , 
     "Fourth coefficient in equation for clipping plane 4" },
-  { F,   "Clip5" , opt_general_clip5 , 0.,
-    "Enable clipping plane 5 (Geometry=2^0, Mesh=2^1, View[i]=2^(2+i))" },
-  { F,   "Clip5A" , opt_general_clip5a , 1.0 , 
+  { F,   "Clip5A" , opt_general_clip5a , 0.0 , 
     "First coefficient in equation for clipping plane 5" },
   { F,   "Clip5B" , opt_general_clip5b , 0.0 , 
     "Second coefficient in equation for clipping plane 5" },
-  { F,   "Clip5C" , opt_general_clip5c , 0.0 , 
+  { F,   "Clip5C" , opt_general_clip5c , -1.0 , 
     "Third coefficient in equation for clipping plane 5" },
   { F,   "Clip5D" , opt_general_clip5d , 0.0 , 
     "Fourth coefficient in equation for clipping plane 5" },
@@ -543,8 +539,6 @@ StringXNumber GeneralOptions_Number[] = {
 
   { F|O, "DoubleBuffer" , opt_general_double_buffer , 1. ,
     "Use a double buffered graphic window (on Unix, should be set to 0 when working on a remote host without GLX)" },
-  { F|O, "DrawAllModels" , opt_general_draw_all_models, 0. , 
-    "Draw all loaded models (instead of drawing only the current one)" },
   { F|O, "DrawBoundingBoxes" , opt_general_draw_bounding_box, 0. ,
     "Draw bounding boxes" },
 
@@ -570,13 +564,13 @@ StringXNumber GeneralOptions_Number[] = {
 
   { F|O, "GraphicsFontSize" , opt_general_graphics_fontsize , 17. ,
     "Size of the font in the graphic window" }, 
-  { F|S, "GraphicsHeight" , opt_general_viewport3 , 600. , 
+  { F|S, "GraphicsHeight" , opt_general_graphics_size1 , 600. , 
     "Height (in pixels) of the graphic window" }, 
   { F|S, "GraphicsPositionX" , opt_general_graphics_position0 , 50. , 
     "Horizontal position (in pixels) of the upper left corner of the graphic window" }, 
   { F|S, "GraphicsPositionY" , opt_general_graphics_position1 , 50. ,
     "Vertical position (in pixels) of the upper left corner of the graphic window" }, 
-  { F|S, "GraphicsWidth" , opt_general_viewport2 , 600. , 
+  { F|S, "GraphicsWidth" , opt_general_graphics_size0 , 600. , 
     "Width (in pixels) of the graphic window" }, 
 
   { F|O, "InitialModule", opt_general_initial_context, 0. , 
@@ -659,7 +653,7 @@ StringXNumber GeneralOptions_Number[] = {
     "Horizontal position (in pixels) of the upper left corner of the menu window" }, 
   { F|S, "MenuPositionY" , opt_general_menu_position1 , 50. ,
     "Vertical position (in pixels) of the upper left corner of the menu window" }, 
-  { F|S, "MessageAutoScroll" , opt_general_message_auto_scroll , 1. , 
+  { F|O, "MessageAutoScroll" , opt_general_message_auto_scroll , 1. , 
     "Automatically scroll message window" }, 
   { F|S, "MessagePositionX" , opt_general_message_position0 , 650. , 
     "Horizontal position (in pixels) of the upper left corner of the message window" }, 
@@ -714,7 +708,7 @@ StringXNumber GeneralOptions_Number[] = {
   { F|O, "PolygonOffsetUnits" , opt_general_polygon_offset_units , 1. , 
     "Polygon offset units (offset = factor * DZ + r * units)" },
 
-  { F|O, "QuadricSubdivisions" , opt_general_quadric_subdivisions, 8. ,
+  { F|O, "QuadricSubdivisions" , opt_general_quadric_subdivisions, 6. ,
     "Number of subdivisions used to draw points or lines as spheres or cylinders" },
 
   { F,   "RotationX" , opt_general_rotation0 , 0.0 , 
@@ -749,9 +743,9 @@ StringXNumber GeneralOptions_Number[] = {
   { F|O, "SmallAxes" , opt_general_small_axes , 1. ,
     "Display the small axes" },
   { F|O, "SmallAxesPositionX" , opt_general_small_axes_position0 , -60. ,
-    "X position of small axes (use negative values for right alignment)" },
+    "X position (in pixels) of small axes (< 0: measure from right edge; >= 1e5: centered)" },
   { F|O, "SmallAxesPositionY" , opt_general_small_axes_position1 , -40. ,
-    "Y position of small axes (use negative values for bottom alignment)" },
+    "Y position (in pixels) of small axes (< 0: measure from bottom edge; >= 1e5: centered)" },
   { F|O, "SmallAxesSize" , opt_general_small_axes_size , 30. ,
     "Size (in pixels) of small axes" },
   { F|S, "SolverPositionX" , opt_general_solver_position0 , 650. , 
@@ -798,15 +792,15 @@ StringXNumber GeneralOptions_Number[] = {
   { F|O, "ZoomFactor" , opt_general_zoom_factor , 4.0 ,
     "Middle mouse button zoom acceleration factor" },
 
-  { 0, NULL , NULL , 0. , NULL }
+  { 0, 0 , 0 , 0. , 0 }
 } ;
 
 StringXNumber GeometryOptions_Number[] = {
   { F|O, "AutoCoherence" , opt_geometry_auto_coherence , 1. , 
     "Should all duplicate entities be automatically removed?" }, 
 
-  { F|O, "CirclePoints" , opt_geometry_circle_points, 20. ,
-    "Number of points used to draw a circle/ellipse" },
+  { F,   "Clip" , opt_geometry_clip , 0.,
+    "Enable clipping planes? (Plane[i]=2^i, i=0,...,5)" },
 
   { F|O, "ExtrudeReturnLateralEntities" , opt_geometry_extrude_return_lateral, 1. ,
     "Add lateral entities in lists returned by extrusion commands?" },
@@ -833,6 +827,8 @@ StringXNumber GeometryOptions_Number[] = {
 
   { F|O, "Normals" , opt_geometry_normals , 0. , 
     "Display size of normal vectors (in pixels)" }, 
+  { F|O, "NumSubEdges" , opt_geometry_num_sub_edges , 20. ,
+    "Number of edge subdivisions between control points when displaying curves" }, 
 
   { F|O, "OCCFixSmallEdges" , opt_geometry_occ_fix_small_edges , 1. , 
     "Fix small edges in STEP, IGES and BRep models" },
@@ -840,6 +836,12 @@ StringXNumber GeometryOptions_Number[] = {
     "Fix small faces in STEP, IGES and BRep models" },
   { F|O, "OCCSewFaces" , opt_geometry_occ_sew_faces , 0. , 
     "Sew faces in STEP, IGES and BRep models" },
+  { F,   "OffsetX" , opt_geometry_offset0 , 0. , 
+    "Model diplay offset along X-axis (in model coordinates)" },
+  { F,   "OffsetY" , opt_geometry_offset1 , 0. , 
+    "Model diplay offset along Y-axis (in model coordinates)" },
+  { F,   "OffsetZ" , opt_geometry_offset2 , 0. , 
+    "Model diplay offset along Z-axis (in model coordinates)" },
   { F|O, "OldCircle" , opt_geometry_old_circle , 0. , 
     "Use old circle description (compatibility option for old Gmsh geometries)" },
   { F|O, "OldNewReg" , opt_geometry_old_newreg , 1. , 
@@ -875,13 +877,33 @@ StringXNumber GeometryOptions_Number[] = {
     "Display size of tangent vectors (in pixels)" },
   { F|O, "Tolerance" , opt_geometry_tolerance, 1.e-6 ,
     "Geometrical tolerance" },
+  { F,   "Transform" , opt_geometry_transform , 0. ,
+    "Transform model display coordinates (0=no, 1=scale)" },
+  { F,   "TransformXX" , opt_geometry_transform00 , 1. ,
+    "Element (1,1) of the 3x3 model display transformation matrix" },
+  { F,   "TransformXY" , opt_geometry_transform01 , 0. ,
+    "Element (1,2) of the 3x3 model display transformation matrix" },
+  { F,   "TransformXZ" , opt_geometry_transform02 , 0. ,
+    "Element (1,3) of the 3x3 model display transformation matrix" },
+  { F,   "TransformYX" , opt_geometry_transform10 , 0. ,
+    "Element (2,1) of the 3x3 model display transformation matrix" },
+  { F,   "TransformYY" , opt_geometry_transform11 , 1. ,
+    "Element (2,2) of the 3x3 model display transformation matrix" },
+  { F,   "TransformYZ" , opt_geometry_transform12 , 0. ,
+    "Element (2,3) of the 3x3 model display transformation matrix" },
+  { F,   "TransformZX" , opt_geometry_transform20 , 0. ,
+    "Element (3,1) of the 3x3 model display transformation matrix" },
+  { F,   "TransformZY" , opt_geometry_transform21 , 0. ,
+    "Element (3,2) of the 3x3 model display transformation matrix" },
+  { F,   "TransformZZ" , opt_geometry_transform22 , 1. ,
+    "Element (3,3) of the 3x3 model display transformation matrix" },
 
   { F|O, "Volumes" , opt_geometry_volumes , 0. , 
     "Display geometry volumes? (not implemented yet)" },
   { F|O, "VolumeNumbers" , opt_geometry_volumes_num , 0. , 
     "Display volume numbers? (not implemented yet)" },
 
-  { 0, NULL , NULL , 0. , NULL }
+  { 0, 0 , 0 , 0. , 0 }
 } ;
 
 StringXNumber MeshOptions_Number[] = {
@@ -904,8 +926,6 @@ StringXNumber MeshOptions_Number[] = {
   { F|O, "Binary" , opt_mesh_binary , 0. , 
     "Write mesh files in binary format (if possible)" },
 
-  { F|O, "C1Continuity" , opt_mesh_c1 , 0. ,
-    "Impose C1 continuity to high order meshes (only valid in 2D plane and ElemenOrder = 2)" },
   { F|O, "ChacoArchitecture" , opt_mesh_partition_chaco_architecture, 1. ,
     "(Adv. Chaco): Parallel architecture topology (0=hypercube, 1-3=mesh dimensions)" },
   { F|O, "ChacoEigensolver" , opt_mesh_partition_chaco_eigensolver, 1. ,
@@ -950,6 +970,8 @@ StringXNumber MeshOptions_Number[] = {
     "Compute characteristic lengths from curvature" },
   { F|O, "CharacteristicLengthFromPoints" , opt_mesh_lc_from_points , 1. ,
     "Compute characteristic lengths from values given at geometry points" },
+  { F,   "Clip" , opt_mesh_clip , 0.,
+    "Enable clipping planes? (Plane[i]=2^i, i=0,...,5)" },
   { F|O, "ColorCarousel" , opt_mesh_color_carousel , 1. ,
     "Mesh coloring (0=by element type, 1=by elementary entity, 2=by physical entity, 3=by partition)" },
   { F,   "CpuTime" , opt_mesh_cpu_time , 0. , 
@@ -975,6 +997,8 @@ StringXNumber MeshOptions_Number[] = {
     "Labels display frequency?" },
   { F|O, "LabelType" , opt_mesh_label_type , 0. , 
     "Type of element label (0=element number, 1=elementary entity number, 2=physical entity number, 3=partition number, 4=coordinates)" },
+  { F|O, "LcIntegrationPrecision" , opt_mesh_lc_integration_precision, 1.e-9 ,
+    "Accuracy of evaluation of the LC field for 1D mesh generation" },
   { F|O, "Light" , opt_mesh_light , 1. , 
     "Enable lighting for the mesh" },
   { F|O, "LightLines" , opt_mesh_light_lines , 1. , 
@@ -1021,6 +1045,8 @@ StringXNumber MeshOptions_Number[] = {
     "Number of triangles in the current mesh (read-only)" },
   { F|O, "Normals" , opt_mesh_normals , 0.0 ,
     "Display size of normal vectors (in pixels)" }, 
+  { F|O, "NumSubEdges" , opt_mesh_num_sub_edges , 2. ,
+    "Number of edge subdivisions when displaying high order elements" }, 
 
   { F|O, "Optimize" , opt_mesh_optimize , 0. , 
     "Optimize the mesh to improve the quality of tetrahedral elements" },
@@ -1056,11 +1082,9 @@ StringXNumber MeshOptions_Number[] = {
   { F|O, "RadiusSup" , opt_mesh_radius_sup , 0.0 , 
     "Only display elements whose longest edge is smaller than RadiusSup" },
   { F|O, "RandomFactor" , opt_mesh_rand_factor , 1.e-9 ,
-    "Random factor used in 2D and 3D meshing algorithm (test other values when the algorithm fails)" },
+    "Random factor used in the 2D meshing algorithm (use larger values if lc * rand approaches machine accuracy)" },
   { F|O, "RefineSteps" , opt_mesh_refine_steps , 10 ,
     "Number of refinement steps in the MeshAdapt-based 2D algorithms" }, 
-  { F|O, "RecombineAlgo" , opt_mesh_recombine_algo , 1 ,
-    "Recombine algorithm (1=mixed triangles-quadrangles, 2=all quadrangles)" }, 
   { F|O, "ReverseAllNormals" , opt_mesh_reverse_all_normals , 0. , 
     "Reverse all the mesh normals (for display)" },
 
@@ -1078,14 +1102,14 @@ StringXNumber MeshOptions_Number[] = {
     "Create incomplete second order elements? (8-node quads, 20-node hexas, etc.)" },
   { F|O, "SecondOrderLinear" , opt_mesh_second_order_linear , 0. ,
     "Should second order vertices simply be created by linear interpolation?" },
-  { F|O, "LcIntegrationPrecision" , opt_mesh_lc_integration_precision, 1.e-9 ,
-    "Accuracy of evaluation of the LC field for 1D mesh generation" },
   { F|O, "Smoothing" , opt_mesh_nb_smoothing , 1. ,
     "Number of smoothing steps applied to the final mesh" },
   { F|O, "SmoothInternalEdges" , opt_mesh_smooth_internal_edges , 0 ,
     "Number of smoothing steps of internal edges for high order meshes" },
   { F|O, "SmoothNormals" , opt_mesh_smooth_normals , 0. , 
     "Smooth the mesh normals?" },
+  { F|O, "SubdivisionAlgorithm" , opt_mesh_algo_subdivide , 0 ,
+    "Mesh subdivision algorithm (0=none, 1=all quadrangles, 2=all hexahedra)" }, 
   { F|O, "SurfaceEdges" , opt_mesh_surfaces_edges , 1. , 
     "Display edges of surface mesh?" },
   { F|O, "SurfaceFaces" , opt_mesh_surfaces_faces , 0. , 
@@ -1114,7 +1138,7 @@ StringXNumber MeshOptions_Number[] = {
   { F|O, "ZoneDefinition" , opt_mesh_zone_definition , 0. , 
     "Method for defining a zone (0=single zone, 1=by partition, 2=by physical)" },
 
-  { 0, NULL , NULL , 0. , NULL }
+  { 0, 0 , 0 , 0. , 0 }
 } ;
 
 StringXNumber SolverOptions_Number[] = {
@@ -1132,8 +1156,6 @@ StringXNumber SolverOptions_Number[] = {
   { F|O, "ClientServer4" , opt_solver_client_server4 , 0. ,
     "Connect solver 4 to the Gmsh server" },
 
-  { F|O, "MaximumDelay" , opt_solver_max_delay , 4.0 ,
-    "Maximum delay (in seconds) allowed for solver response" },
   { F|O, "MergeViews0" , opt_solver_merge_views0 , 1. , 
     "Automatically merge any post-processing view created by solver 0" },
   { F|O, "MergeViews1" , opt_solver_merge_views1 , 1. , 
@@ -1163,7 +1185,7 @@ StringXNumber SolverOptions_Number[] = {
   { F|O, "PopupMessages4" , opt_solver_popup_messages4 , 1. ,
     "Automatically display messages produced by solver 4" },
 
-  { 0, NULL , NULL , 0. , NULL }
+  { 0, 0 , 0 , 0. , 0 }
 } ;
 
 StringXNumber PostProcessingOptions_Number[] = {
@@ -1193,22 +1215,18 @@ StringXNumber PostProcessingOptions_Number[] = {
   { F|O, "Smoothing" , opt_post_smooth , 0. ,
     "Apply (non-reversible) smoothing to post-processing view when merged" },
 
-  { 0, NULL , NULL , 0. }
+  { 0, 0 , 0 , 0. }
 } ;
 
 StringXNumber ViewOptions_Number[] = {
+  { F|O, "AdaptVisualizationGrid" , opt_view_adapt_visualization_grid , 0. ,
+    "Use adaptive visualization grid (for high-order elements)?" }, 
   { F|O, "AngleSmoothNormals" , opt_view_angle_smooth_normals , 30.0 ,
     "Threshold angle below which normals are not smoothed" },
-  { F|O, "ArrowHeadRadius" , opt_view_arrow_head_radius , 0.12 ,
-    "Relative radius of arrow head" },
-  { F|O, "ArrowSize" , opt_view_arrow_size , 60. ,
-    "Display size of arrows (in pixels)" },
-  { F|O, "ArrowSizeProportional" , opt_view_arrow_size_proportional , 1. ,
-    "Scale the arrows according to the norm of the vector" },
-  { F|O, "ArrowStemLength" , opt_view_arrow_stem_length , 0.56 ,
-    "Relative length of arrow stem" },
-  { F|O, "ArrowStemRadius" , opt_view_arrow_stem_radius , 0.02 ,
-    "Relative radius of arrow stem" },
+  { F|O, "ArrowSizeMax" , opt_view_arrow_size_max , 60. ,
+    "Maximum display size of arrows (in pixels)" },
+  { F|O, "ArrowSizeMin" , opt_view_arrow_size_min , 0. ,
+    "Minimum display size of arrows (in pixels)" },
   { F|O, "AutoPosition" , opt_view_auto_position , 1. , 
     "Position the scale or 2D plot automatically" }, 
   { F|O, "Axes" , opt_view_axes , 0 ,
@@ -1241,6 +1259,8 @@ StringXNumber ViewOptions_Number[] = {
 
   { F|O, "CenterGlyphs" , opt_view_center_glyphs , 0, 
     "Center glyphs (arrows, numbers, etc.)" },
+  { F,   "Clip" , opt_view_clip , 0.,
+    "Enable clipping planes? (Plane[i]=2^i, i=0,...,5)" },
   { F|O, "ColormapAlpha" , opt_view_colormap_alpha , 1.0 ,
     "Colormap alpha channel value (used only if != 1)" },
   { F|O, "ColormapAlphaPower" , opt_view_colormap_alpha_power , 0.0 ,
@@ -1365,9 +1385,9 @@ StringXNumber ViewOptions_Number[] = {
   { F|O, "PointType" , opt_view_point_type , 0. , 
     "Display points as solid color dots (0), 3D spheres (1), scaled dots (2) or scaled spheres (3)" },
   { F|O, "PositionX" , opt_view_position0 , 100. , 
-    "Horizontal position (in pixels) of the upper left corner of the scale or 2D plot" }, 
+    "X position (in pixels) of the scale or 2D plot (< 0: measure from right edge; >= 1e5: centered)" }, 
   { F|O, "PositionY" , opt_view_position1 , 50. , 
-    "Vertical position (in pixels) of the upper left corner of the scale or 2D plot" }, 
+    "Y position (in pixels) of the scale or 2D plot (< 0: measure from bottom edge; >= 1e5: centered)" }, 
 
   { F,   "RaiseX" , opt_view_raise0 , 0. , 
     "Elevation of the view along X-axis (in model coordinates)" },
@@ -1401,23 +1421,23 @@ StringXNumber ViewOptions_Number[] = {
     "Tensor Visualization Type" },
   { F,   "TimeStep" , opt_view_timestep , 0. ,
     "Current time step displayed" },
-  { F,   "Transform11" , opt_view_transform00 , 1. ,
+  { F,   "TransformXX" , opt_view_transform00 , 1. ,
     "Element (1,1) of the 3x3 coordinate transformation matrix" },
-  { F,   "Transform12" , opt_view_transform01 , 0. ,
+  { F,   "TransformXY" , opt_view_transform01 , 0. ,
     "Element (1,2) of the 3x3 coordinate transformation matrix" },
-  { F,   "Transform13" , opt_view_transform02 , 0. ,
+  { F,   "TransformXZ" , opt_view_transform02 , 0. ,
     "Element (1,3) of the 3x3 coordinate transformation matrix" },
-  { F,   "Transform21" , opt_view_transform10 , 0. ,
+  { F,   "TransformYX" , opt_view_transform10 , 0. ,
     "Element (2,1) of the 3x3 coordinate transformation matrix" },
-  { F,   "Transform22" , opt_view_transform11 , 1. ,
+  { F,   "TransformYY" , opt_view_transform11 , 1. ,
     "Element (2,2) of the 3x3 coordinate transformation matrix" },
-  { F,   "Transform23" , opt_view_transform12 , 0. ,
+  { F,   "TransformYZ" , opt_view_transform12 , 0. ,
     "Element (2,3) of the 3x3 coordinate transformation matrix" },
-  { F,   "Transform31" , opt_view_transform20 , 0. ,
+  { F,   "TransformZX" , opt_view_transform20 , 0. ,
     "Element (3,1) of the 3x3 coordinate transformation matrix" },
-  { F,   "Transform32" , opt_view_transform21 , 0. ,
+  { F,   "TransformZY" , opt_view_transform21 , 0. ,
     "Element (3,2) of the 3x3 coordinate transformation matrix" },
-  { F,   "Transform33" , opt_view_transform22 , 1. ,
+  { F,   "TransformZZ" , opt_view_transform22 , 1. ,
     "Element (3,3) of the 3x3 coordinate transformation matrix" },
   { F,   "Type" , opt_view_type , 1 ,
     "Type of plot (1=3D, 2=2D space, 3=2D time)" },
@@ -1433,10 +1453,13 @@ StringXNumber ViewOptions_Number[] = {
   { F|O, "Width" , opt_view_size0 , 300. , 
     "Width (in pixels) of the scale or 2D plot" }, 
 
-  { 0, NULL , NULL , 0. , NULL }
+  { 0, 0 , 0 , 0. , 0 }
 } ;
 
 StringXNumber PrintOptions_Number[] = {
+  { F|O, "CompositeWindows" , opt_print_composite_windows , 0. ,
+    "Composite all window tiles in the same output image (for bitmap output only)" },
+
   { F|O, "EpsBackground" , opt_print_eps_background , 1. ,
     "Save image background in PostScript/PDF output" },
   { F|O, "EpsBestRoot" , opt_print_eps_best_root , 1. ,
@@ -1491,7 +1514,7 @@ StringXNumber PrintOptions_Number[] = {
   { F|O, "Text" , opt_print_text , 1. ,
     "Print text strings?" },
 
-  { 0, NULL , NULL , 0. }
+  { 0, 0 , 0 , 0. }
 } ;
 
 // COLORS
@@ -1529,7 +1552,7 @@ StringXColor GeneralOptions_Color[] = {
   { F|O, "SpecularLight" , opt_general_color_specular_light,
     {255, 255, 255, 255}, {255, 255, 255, 255}, {255, 255, 255, 255},
     "Specular light color" },
-  { 0, NULL , NULL ,  {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0} , NULL }
+  { 0, 0 , 0 ,  {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0} , 0 }
 } ;
 
 StringXColor GeometryOptions_Color[] = {
@@ -1566,7 +1589,7 @@ StringXColor GeometryOptions_Color[] = {
   { F|O, "Projection" , opt_geometry_color_projection ,
     {0, 255, 0, 255}, {0, 255, 0, 255}, {0, 0, 0, 255},
     "Projection surface color" },
-  { 0, NULL , NULL , {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0} , NULL }
+  { 0, 0 , 0 , {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0} , 0 }
 } ;
 
 #define COLW  {210, 210, 210, 255}
@@ -1650,15 +1673,15 @@ StringXColor MeshOptions_Color[] = {
   { F|O, "Seventeen", opt_mesh_color_17, COL17, COL17, COLW, "Color 17 in color carousel" },
   { F|O, "Eighteen" , opt_mesh_color_18, COL18, COL18, COLW, "Color 18 in color carousel" },
   { F|O, "Nineteen" , opt_mesh_color_19, COL19, COL19, COLW, "Color 19 in color carousel" },
-  { 0, NULL , NULL , {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0} , NULL }
+  { 0, 0 , 0 , {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0} , 0 }
 } ;
 
 StringXColor SolverOptions_Color[] = {
-  { 0, NULL , NULL , {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0} , NULL }
+  { 0, 0 , 0 , {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0} , 0 }
 } ;
 
 StringXColor PostProcessingOptions_Color[] = {
-  { 0, NULL , NULL , {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0} , NULL }
+  { 0, 0 , 0 , {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0} , 0 }
 } ;
 
 #define ELECOL  {255, 255, 255, 255}, {0, 0, 0, 255}, {0, 0, 0, 255}
@@ -1681,11 +1704,11 @@ StringXColor ViewOptions_Color[] = {
   { F|O, "Text2D" , opt_view_color_text2d , ELECOL, "2D text color" },
   { F|O, "Text3D" , opt_view_color_text3d , ELECOL, "3D text color" },
   { F|O, "Axes" , opt_view_color_axes , ELECOL, "Axes color" },
-  { 0, NULL , NULL , {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0} , NULL }
+  { 0, 0 , 0 , {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0} , 0 }
 } ;
 
 StringXColor PrintOptions_Color[] = {
-  { 0, NULL , NULL , {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0} , NULL }
+  { 0, 0 , 0 , {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0} , 0 }
 } ;
 
 #undef S
