@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -13,6 +13,7 @@
 #include <sys/stat.h>
 #include <signal.h>
 #include <time.h>
+#include <math.h>
 
 #if !defined(WIN32) || defined(__CYGWIN__)
 #include <unistd.h>
@@ -31,7 +32,7 @@
 #define RUSAGE_CHILDREN -1
 #endif
 
-#include "Message.h"
+#include "GmshMessage.h"
 
 double GetTimeInSeconds()
 {
@@ -58,7 +59,7 @@ void SleepInSeconds(double s)
 #endif
 }
 
-void GetResources(double *s, long *mem)
+static void GetResources(double *s, long *mem)
 {
 #if !defined(WIN32) || defined(__CYGWIN__)
   static struct rusage r;
@@ -75,7 +76,8 @@ void GetResources(double *s, long *mem)
 #endif
 }
 
-void CheckResources(){
+void CheckResources()
+{
 #if !defined (WIN32) || defined(__CYGWIN__)
   static struct rlimit r;
 
@@ -109,23 +111,30 @@ int GetProcessId()
 #endif
 }
 
-int UnlinkFile(const char *filename)
+std::string GetHostName()
+{
+  char host[256];
+  gethostname(host, sizeof(host));
+  return std::string(host);
+}
+
+int UnlinkFile(std::string fileName)
 {
 #if !defined(WIN32) || defined(__CYGWIN__)
-  return unlink(filename);
+  return unlink(fileName.c_str());
 #else
-  return _unlink(filename);
+  return _unlink(fileName.c_str());
 #endif
 }
 
-int StatFile(const char *filename)
+int StatFile(std::string fileName)
 {
 #if !defined(WIN32) || defined(__CYGWIN__)
   struct stat buf;
-  return stat(filename, &buf);
+  return stat(fileName.c_str(), &buf);
 #else
   struct _stat buf;
-  return _stat(filename, &buf);
+  return _stat(fileName.c_str(), &buf);
 #endif
 }
 
@@ -144,15 +153,15 @@ int KillProcess(int pid)
   return 1;
 }
 
-int SystemCall(const char *command)
+int SystemCall(std::string command)
 {
 #if defined(WIN32)
   STARTUPINFO suInfo;
   PROCESS_INFORMATION prInfo;
   memset(&suInfo, 0, sizeof(suInfo));
   suInfo.cb = sizeof(suInfo);
-  Msg::Info("Calling '%s'", command);
-  CreateProcess(NULL, (char*)command, NULL, NULL, FALSE,
+  Msg::Info("Calling '%s'", command.c_str());
+  CreateProcess(NULL, (char*)command.c_str(), NULL, NULL, FALSE,
                 NORMAL_PRIORITY_CLASS, NULL, NULL, &suInfo, &prInfo);
   return 0;
 #else
@@ -160,7 +169,21 @@ int SystemCall(const char *command)
     Msg::Error("Could not find /bin/sh: aborting system call");
     return 1;
   }
-  Msg::Info("Calling '%s'", command);
-  return system(command);
+  Msg::Info("Calling '%s'", command.c_str());
+  return system(command.c_str());
 #endif
 }
+
+//HACK: allow the use of mixed debug/non-debug code with MSVC
+#if (defined (_MSC_VER) && defined (_DEBUG))
+extern "C" {
+   _CRTIMP void __cdecl _invalid_parameter_noinfo(void) {  }
+}
+#endif
+
+//HACK: fix undefined hypot with MSVC
+#if defined (_MSC_VER)
+extern "C" {
+  double hypot(double x, double y){ return _hypot(x, y); }
+}
+#endif
