@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -6,11 +6,14 @@
 #ifndef _OPENFILE_H_
 #define _OPENFILE_H_
 
-int ParseFile(const char *filename, int close, int warn_if_missing=0);
-void ParseString(const char *str);
-void OpenProject(const char *filename);
-void OpenProjectMacFinder(const char *filename);
-int MergeFile(const char *filename, int warn_if_missing=0);
+#include <string>
+
+int ParseFile(std::string fileName, bool close, bool warnIfMissing=false);
+void ParseString(std::string str);
+void OpenProject(std::string filename);
+void OpenProjectMacFinder(const char *fileName);
+int MergeFile(std::string fileName, bool warnIfMissing=false);
+void ClearProject();
 void SetBoundingBox(double xmin, double xmax,
                     double ymin, double ymax, 
                     double zmin, double zmax);
