@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -6,6 +6,7 @@
 #ifndef _OPTIONS_H_
 #define _OPTIONS_H_
 
+#include <string>
 #include "ColorTable.h"
 
 #define GMSH_SET       (1<<0)
@@ -18,196 +19,197 @@
 
 // action is a combination of GMSH_SET, GMSH_GET, GMSH_GUI
 
-#define OPT_ARGS_STR   int num, int action, const char *val
+#define OPT_ARGS_STR   int num, int action, std::string val
 #define OPT_ARGS_NUM   int num, int action, double val
 #define OPT_ARGS_COL   int num, int action, unsigned int val
 
 // STRINGS
 
-const char *opt_general_axes_label0(OPT_ARGS_STR);
-const char *opt_general_axes_label1(OPT_ARGS_STR);
-const char *opt_general_axes_label2(OPT_ARGS_STR);
-const char *opt_general_axes_format0(OPT_ARGS_STR);
-const char *opt_general_axes_format1(OPT_ARGS_STR);
-const char *opt_general_axes_format2(OPT_ARGS_STR);
-const char *opt_general_display(OPT_ARGS_STR);
-const char *opt_general_filename(OPT_ARGS_STR);
-const char *opt_general_default_filename(OPT_ARGS_STR);
-const char *opt_general_tmp_filename(OPT_ARGS_STR);
-const char *opt_general_error_filename(OPT_ARGS_STR);
-const char *opt_general_session_filename(OPT_ARGS_STR);
-const char *opt_general_options_filename(OPT_ARGS_STR);
-const char *opt_general_editor(OPT_ARGS_STR);
-const char *opt_general_web_browser(OPT_ARGS_STR);
-const char *opt_general_gui_theme(OPT_ARGS_STR);
-const char *opt_general_graphics_font(OPT_ARGS_STR);
-const char *opt_solver_socket_name(OPT_ARGS_STR);
-const char *opt_solver_name(OPT_ARGS_STR);
-const char *opt_solver_name0(OPT_ARGS_STR);
-const char *opt_solver_name1(OPT_ARGS_STR);
-const char *opt_solver_name2(OPT_ARGS_STR);
-const char *opt_solver_name3(OPT_ARGS_STR);
-const char *opt_solver_name4(OPT_ARGS_STR);
-const char *opt_solver_executable(OPT_ARGS_STR);
-const char *opt_solver_executable0(OPT_ARGS_STR);
-const char *opt_solver_executable1(OPT_ARGS_STR);
-const char *opt_solver_executable2(OPT_ARGS_STR);
-const char *opt_solver_executable3(OPT_ARGS_STR);
-const char *opt_solver_executable4(OPT_ARGS_STR);
-const char *opt_solver_help(OPT_ARGS_STR);
-const char *opt_solver_help0(OPT_ARGS_STR);
-const char *opt_solver_help1(OPT_ARGS_STR);
-const char *opt_solver_help2(OPT_ARGS_STR);
-const char *opt_solver_help3(OPT_ARGS_STR);
-const char *opt_solver_help4(OPT_ARGS_STR);
-const char *opt_solver_extension(OPT_ARGS_STR);
-const char *opt_solver_extension0(OPT_ARGS_STR);
-const char *opt_solver_extension1(OPT_ARGS_STR);
-const char *opt_solver_extension2(OPT_ARGS_STR);
-const char *opt_solver_extension3(OPT_ARGS_STR);
-const char *opt_solver_extension4(OPT_ARGS_STR);
-const char *opt_solver_mesh_name(OPT_ARGS_STR);
-const char *opt_solver_mesh_name0(OPT_ARGS_STR);
-const char *opt_solver_mesh_name1(OPT_ARGS_STR);
-const char *opt_solver_mesh_name2(OPT_ARGS_STR);
-const char *opt_solver_mesh_name3(OPT_ARGS_STR);
-const char *opt_solver_mesh_name4(OPT_ARGS_STR);
-const char *opt_solver_mesh_command(OPT_ARGS_STR);
-const char *opt_solver_mesh_command0(OPT_ARGS_STR);
-const char *opt_solver_mesh_command1(OPT_ARGS_STR);
-const char *opt_solver_mesh_command2(OPT_ARGS_STR);
-const char *opt_solver_mesh_command3(OPT_ARGS_STR);
-const char *opt_solver_mesh_command4(OPT_ARGS_STR);
-const char *opt_solver_socket_command(OPT_ARGS_STR);
-const char *opt_solver_socket_command0(OPT_ARGS_STR);
-const char *opt_solver_socket_command1(OPT_ARGS_STR);
-const char *opt_solver_socket_command2(OPT_ARGS_STR);
-const char *opt_solver_socket_command3(OPT_ARGS_STR);
-const char *opt_solver_socket_command4(OPT_ARGS_STR);
-const char *opt_solver_name_command(OPT_ARGS_STR);
-const char *opt_solver_name_command0(OPT_ARGS_STR);
-const char *opt_solver_name_command1(OPT_ARGS_STR);
-const char *opt_solver_name_command2(OPT_ARGS_STR);
-const char *opt_solver_name_command3(OPT_ARGS_STR);
-const char *opt_solver_name_command4(OPT_ARGS_STR);
-const char *opt_solver_option_command(OPT_ARGS_STR);
-const char *opt_solver_option_command0(OPT_ARGS_STR);
-const char *opt_solver_option_command1(OPT_ARGS_STR);
-const char *opt_solver_option_command2(OPT_ARGS_STR);
-const char *opt_solver_option_command3(OPT_ARGS_STR);
-const char *opt_solver_option_command4(OPT_ARGS_STR);
-const char *opt_solver_first_option(OPT_ARGS_STR);
-const char *opt_solver_first_option0(OPT_ARGS_STR);
-const char *opt_solver_first_option1(OPT_ARGS_STR);
-const char *opt_solver_first_option2(OPT_ARGS_STR);
-const char *opt_solver_first_option3(OPT_ARGS_STR);
-const char *opt_solver_first_option4(OPT_ARGS_STR);
-const char *opt_solver_second_option(OPT_ARGS_STR);
-const char *opt_solver_second_option0(OPT_ARGS_STR);
-const char *opt_solver_second_option1(OPT_ARGS_STR);
-const char *opt_solver_second_option2(OPT_ARGS_STR);
-const char *opt_solver_second_option3(OPT_ARGS_STR);
-const char *opt_solver_second_option4(OPT_ARGS_STR);
-const char *opt_solver_third_option(OPT_ARGS_STR);
-const char *opt_solver_third_option0(OPT_ARGS_STR);
-const char *opt_solver_third_option1(OPT_ARGS_STR);
-const char *opt_solver_third_option2(OPT_ARGS_STR);
-const char *opt_solver_third_option3(OPT_ARGS_STR);
-const char *opt_solver_third_option4(OPT_ARGS_STR);
-const char *opt_solver_fourth_option(OPT_ARGS_STR);
-const char *opt_solver_fourth_option0(OPT_ARGS_STR);
-const char *opt_solver_fourth_option1(OPT_ARGS_STR);
-const char *opt_solver_fourth_option2(OPT_ARGS_STR);
-const char *opt_solver_fourth_option3(OPT_ARGS_STR);
-const char *opt_solver_fourth_option4(OPT_ARGS_STR);
-const char *opt_solver_fifth_option(OPT_ARGS_STR);
-const char *opt_solver_fifth_option0(OPT_ARGS_STR);
-const char *opt_solver_fifth_option1(OPT_ARGS_STR);
-const char *opt_solver_fifth_option2(OPT_ARGS_STR);
-const char *opt_solver_fifth_option3(OPT_ARGS_STR);
-const char *opt_solver_fifth_option4(OPT_ARGS_STR);
-const char *opt_solver_first_button(OPT_ARGS_STR);
-const char *opt_solver_first_button0(OPT_ARGS_STR);
-const char *opt_solver_first_button1(OPT_ARGS_STR);
-const char *opt_solver_first_button2(OPT_ARGS_STR);
-const char *opt_solver_first_button3(OPT_ARGS_STR);
-const char *opt_solver_first_button4(OPT_ARGS_STR);
-const char *opt_solver_first_button_command(OPT_ARGS_STR);
-const char *opt_solver_first_button_command0(OPT_ARGS_STR);
-const char *opt_solver_first_button_command1(OPT_ARGS_STR);
-const char *opt_solver_first_button_command2(OPT_ARGS_STR);
-const char *opt_solver_first_button_command3(OPT_ARGS_STR);
-const char *opt_solver_first_button_command4(OPT_ARGS_STR);
-const char *opt_solver_second_button(OPT_ARGS_STR);
-const char *opt_solver_second_button0(OPT_ARGS_STR);
-const char *opt_solver_second_button1(OPT_ARGS_STR);
-const char *opt_solver_second_button2(OPT_ARGS_STR);
-const char *opt_solver_second_button3(OPT_ARGS_STR);
-const char *opt_solver_second_button4(OPT_ARGS_STR);
-const char *opt_solver_second_button_command(OPT_ARGS_STR);
-const char *opt_solver_second_button_command0(OPT_ARGS_STR);
-const char *opt_solver_second_button_command1(OPT_ARGS_STR);
-const char *opt_solver_second_button_command2(OPT_ARGS_STR);
-const char *opt_solver_second_button_command3(OPT_ARGS_STR);
-const char *opt_solver_second_button_command4(OPT_ARGS_STR);
-const char *opt_solver_third_button(OPT_ARGS_STR);
-const char *opt_solver_third_button0(OPT_ARGS_STR);
-const char *opt_solver_third_button1(OPT_ARGS_STR);
-const char *opt_solver_third_button2(OPT_ARGS_STR);
-const char *opt_solver_third_button3(OPT_ARGS_STR);
-const char *opt_solver_third_button4(OPT_ARGS_STR);
-const char *opt_solver_third_button_command(OPT_ARGS_STR);
-const char *opt_solver_third_button_command0(OPT_ARGS_STR);
-const char *opt_solver_third_button_command1(OPT_ARGS_STR);
-const char *opt_solver_third_button_command2(OPT_ARGS_STR);
-const char *opt_solver_third_button_command3(OPT_ARGS_STR);
-const char *opt_solver_third_button_command4(OPT_ARGS_STR);
-const char *opt_solver_fourth_button(OPT_ARGS_STR);
-const char *opt_solver_fourth_button0(OPT_ARGS_STR);
-const char *opt_solver_fourth_button1(OPT_ARGS_STR);
-const char *opt_solver_fourth_button2(OPT_ARGS_STR);
-const char *opt_solver_fourth_button3(OPT_ARGS_STR);
-const char *opt_solver_fourth_button4(OPT_ARGS_STR);
-const char *opt_solver_fourth_button_command(OPT_ARGS_STR);
-const char *opt_solver_fourth_button_command0(OPT_ARGS_STR);
-const char *opt_solver_fourth_button_command1(OPT_ARGS_STR);
-const char *opt_solver_fourth_button_command2(OPT_ARGS_STR);
-const char *opt_solver_fourth_button_command3(OPT_ARGS_STR);
-const char *opt_solver_fourth_button_command4(OPT_ARGS_STR);
-const char *opt_solver_fifth_button(OPT_ARGS_STR);
-const char *opt_solver_fifth_button0(OPT_ARGS_STR);
-const char *opt_solver_fifth_button1(OPT_ARGS_STR);
-const char *opt_solver_fifth_button2(OPT_ARGS_STR);
-const char *opt_solver_fifth_button3(OPT_ARGS_STR);
-const char *opt_solver_fifth_button4(OPT_ARGS_STR);
-const char *opt_solver_fifth_button_command(OPT_ARGS_STR);
-const char *opt_solver_fifth_button_command0(OPT_ARGS_STR);
-const char *opt_solver_fifth_button_command1(OPT_ARGS_STR);
-const char *opt_solver_fifth_button_command2(OPT_ARGS_STR);
-const char *opt_solver_fifth_button_command3(OPT_ARGS_STR);
-const char *opt_solver_fifth_button_command4(OPT_ARGS_STR);
-const char *opt_view_name(OPT_ARGS_STR);
-const char *opt_view_format(OPT_ARGS_STR);
-const char *opt_view_filename(OPT_ARGS_STR);
-const char *opt_view_axes_label0(OPT_ARGS_STR);
-const char *opt_view_axes_label1(OPT_ARGS_STR);
-const char *opt_view_axes_label2(OPT_ARGS_STR);
-const char *opt_view_axes_format0(OPT_ARGS_STR);
-const char *opt_view_axes_format1(OPT_ARGS_STR);
-const char *opt_view_axes_format2(OPT_ARGS_STR);
-const char *opt_view_gen_raise0(OPT_ARGS_STR);
-const char *opt_view_gen_raise1(OPT_ARGS_STR);
-const char *opt_view_gen_raise2(OPT_ARGS_STR);
-const char *opt_view_stipple0(OPT_ARGS_STR);
-const char *opt_view_stipple1(OPT_ARGS_STR);
-const char *opt_view_stipple2(OPT_ARGS_STR);
-const char *opt_view_stipple3(OPT_ARGS_STR);
-const char *opt_view_stipple4(OPT_ARGS_STR);
-const char *opt_view_stipple5(OPT_ARGS_STR);
-const char *opt_view_stipple6(OPT_ARGS_STR);
-const char *opt_view_stipple7(OPT_ARGS_STR);
-const char *opt_view_stipple8(OPT_ARGS_STR);
-const char *opt_view_stipple9(OPT_ARGS_STR);
+std::string opt_general_axes_label0(OPT_ARGS_STR);
+std::string opt_general_axes_label1(OPT_ARGS_STR);
+std::string opt_general_axes_label2(OPT_ARGS_STR);
+std::string opt_general_axes_format0(OPT_ARGS_STR);
+std::string opt_general_axes_format1(OPT_ARGS_STR);
+std::string opt_general_axes_format2(OPT_ARGS_STR);
+std::string opt_general_background_image_filename(OPT_ARGS_STR);
+std::string opt_general_display(OPT_ARGS_STR);
+std::string opt_general_filename(OPT_ARGS_STR);
+std::string opt_general_default_filename(OPT_ARGS_STR);
+std::string opt_general_tmp_filename(OPT_ARGS_STR);
+std::string opt_general_error_filename(OPT_ARGS_STR);
+std::string opt_general_session_filename(OPT_ARGS_STR);
+std::string opt_general_options_filename(OPT_ARGS_STR);
+std::string opt_general_editor(OPT_ARGS_STR);
+std::string opt_general_web_browser(OPT_ARGS_STR);
+std::string opt_general_gui_theme(OPT_ARGS_STR);
+std::string opt_general_graphics_font(OPT_ARGS_STR);
+std::string opt_solver_socket_name(OPT_ARGS_STR);
+std::string opt_solver_name(OPT_ARGS_STR);
+std::string opt_solver_name0(OPT_ARGS_STR);
+std::string opt_solver_name1(OPT_ARGS_STR);
+std::string opt_solver_name2(OPT_ARGS_STR);
+std::string opt_solver_name3(OPT_ARGS_STR);
+std::string opt_solver_name4(OPT_ARGS_STR);
+std::string opt_solver_executable(OPT_ARGS_STR);
+std::string opt_solver_executable0(OPT_ARGS_STR);
+std::string opt_solver_executable1(OPT_ARGS_STR);
+std::string opt_solver_executable2(OPT_ARGS_STR);
+std::string opt_solver_executable3(OPT_ARGS_STR);
+std::string opt_solver_executable4(OPT_ARGS_STR);
+std::string opt_solver_help(OPT_ARGS_STR);
+std::string opt_solver_help0(OPT_ARGS_STR);
+std::string opt_solver_help1(OPT_ARGS_STR);
+std::string opt_solver_help2(OPT_ARGS_STR);
+std::string opt_solver_help3(OPT_ARGS_STR);
+std::string opt_solver_help4(OPT_ARGS_STR);
+std::string opt_solver_extension(OPT_ARGS_STR);
+std::string opt_solver_extension0(OPT_ARGS_STR);
+std::string opt_solver_extension1(OPT_ARGS_STR);
+std::string opt_solver_extension2(OPT_ARGS_STR);
+std::string opt_solver_extension3(OPT_ARGS_STR);
+std::string opt_solver_extension4(OPT_ARGS_STR);
+std::string opt_solver_mesh_name(OPT_ARGS_STR);
+std::string opt_solver_mesh_name0(OPT_ARGS_STR);
+std::string opt_solver_mesh_name1(OPT_ARGS_STR);
+std::string opt_solver_mesh_name2(OPT_ARGS_STR);
+std::string opt_solver_mesh_name3(OPT_ARGS_STR);
+std::string opt_solver_mesh_name4(OPT_ARGS_STR);
+std::string opt_solver_mesh_command(OPT_ARGS_STR);
+std::string opt_solver_mesh_command0(OPT_ARGS_STR);
+std::string opt_solver_mesh_command1(OPT_ARGS_STR);
+std::string opt_solver_mesh_command2(OPT_ARGS_STR);
+std::string opt_solver_mesh_command3(OPT_ARGS_STR);
+std::string opt_solver_mesh_command4(OPT_ARGS_STR);
+std::string opt_solver_socket_command(OPT_ARGS_STR);
+std::string opt_solver_socket_command0(OPT_ARGS_STR);
+std::string opt_solver_socket_command1(OPT_ARGS_STR);
+std::string opt_solver_socket_command2(OPT_ARGS_STR);
+std::string opt_solver_socket_command3(OPT_ARGS_STR);
+std::string opt_solver_socket_command4(OPT_ARGS_STR);
+std::string opt_solver_name_command(OPT_ARGS_STR);
+std::string opt_solver_name_command0(OPT_ARGS_STR);
+std::string opt_solver_name_command1(OPT_ARGS_STR);
+std::string opt_solver_name_command2(OPT_ARGS_STR);
+std::string opt_solver_name_command3(OPT_ARGS_STR);
+std::string opt_solver_name_command4(OPT_ARGS_STR);
+std::string opt_solver_option_command(OPT_ARGS_STR);
+std::string opt_solver_option_command0(OPT_ARGS_STR);
+std::string opt_solver_option_command1(OPT_ARGS_STR);
+std::string opt_solver_option_command2(OPT_ARGS_STR);
+std::string opt_solver_option_command3(OPT_ARGS_STR);
+std::string opt_solver_option_command4(OPT_ARGS_STR);
+std::string opt_solver_first_option(OPT_ARGS_STR);
+std::string opt_solver_first_option0(OPT_ARGS_STR);
+std::string opt_solver_first_option1(OPT_ARGS_STR);
+std::string opt_solver_first_option2(OPT_ARGS_STR);
+std::string opt_solver_first_option3(OPT_ARGS_STR);
+std::string opt_solver_first_option4(OPT_ARGS_STR);
+std::string opt_solver_second_option(OPT_ARGS_STR);
+std::string opt_solver_second_option0(OPT_ARGS_STR);
+std::string opt_solver_second_option1(OPT_ARGS_STR);
+std::string opt_solver_second_option2(OPT_ARGS_STR);
+std::string opt_solver_second_option3(OPT_ARGS_STR);
+std::string opt_solver_second_option4(OPT_ARGS_STR);
+std::string opt_solver_third_option(OPT_ARGS_STR);
+std::string opt_solver_third_option0(OPT_ARGS_STR);
+std::string opt_solver_third_option1(OPT_ARGS_STR);
+std::string opt_solver_third_option2(OPT_ARGS_STR);
+std::string opt_solver_third_option3(OPT_ARGS_STR);
+std::string opt_solver_third_option4(OPT_ARGS_STR);
+std::string opt_solver_fourth_option(OPT_ARGS_STR);
+std::string opt_solver_fourth_option0(OPT_ARGS_STR);
+std::string opt_solver_fourth_option1(OPT_ARGS_STR);
+std::string opt_solver_fourth_option2(OPT_ARGS_STR);
+std::string opt_solver_fourth_option3(OPT_ARGS_STR);
+std::string opt_solver_fourth_option4(OPT_ARGS_STR);
+std::string opt_solver_fifth_option(OPT_ARGS_STR);
+std::string opt_solver_fifth_option0(OPT_ARGS_STR);
+std::string opt_solver_fifth_option1(OPT_ARGS_STR);
+std::string opt_solver_fifth_option2(OPT_ARGS_STR);
+std::string opt_solver_fifth_option3(OPT_ARGS_STR);
+std::string opt_solver_fifth_option4(OPT_ARGS_STR);
+std::string opt_solver_first_button(OPT_ARGS_STR);
+std::string opt_solver_first_button0(OPT_ARGS_STR);
+std::string opt_solver_first_button1(OPT_ARGS_STR);
+std::string opt_solver_first_button2(OPT_ARGS_STR);
+std::string opt_solver_first_button3(OPT_ARGS_STR);
+std::string opt_solver_first_button4(OPT_ARGS_STR);
+std::string opt_solver_first_button_command(OPT_ARGS_STR);
+std::string opt_solver_first_button_command0(OPT_ARGS_STR);
+std::string opt_solver_first_button_command1(OPT_ARGS_STR);
+std::string opt_solver_first_button_command2(OPT_ARGS_STR);
+std::string opt_solver_first_button_command3(OPT_ARGS_STR);
+std::string opt_solver_first_button_command4(OPT_ARGS_STR);
+std::string opt_solver_second_button(OPT_ARGS_STR);
+std::string opt_solver_second_button0(OPT_ARGS_STR);
+std::string opt_solver_second_button1(OPT_ARGS_STR);
+std::string opt_solver_second_button2(OPT_ARGS_STR);
+std::string opt_solver_second_button3(OPT_ARGS_STR);
+std::string opt_solver_second_button4(OPT_ARGS_STR);
+std::string opt_solver_second_button_command(OPT_ARGS_STR);
+std::string opt_solver_second_button_command0(OPT_ARGS_STR);
+std::string opt_solver_second_button_command1(OPT_ARGS_STR);
+std::string opt_solver_second_button_command2(OPT_ARGS_STR);
+std::string opt_solver_second_button_command3(OPT_ARGS_STR);
+std::string opt_solver_second_button_command4(OPT_ARGS_STR);
+std::string opt_solver_third_button(OPT_ARGS_STR);
+std::string opt_solver_third_button0(OPT_ARGS_STR);
+std::string opt_solver_third_button1(OPT_ARGS_STR);
+std::string opt_solver_third_button2(OPT_ARGS_STR);
+std::string opt_solver_third_button3(OPT_ARGS_STR);
+std::string opt_solver_third_button4(OPT_ARGS_STR);
+std::string opt_solver_third_button_command(OPT_ARGS_STR);
+std::string opt_solver_third_button_command0(OPT_ARGS_STR);
+std::string opt_solver_third_button_command1(OPT_ARGS_STR);
+std::string opt_solver_third_button_command2(OPT_ARGS_STR);
+std::string opt_solver_third_button_command3(OPT_ARGS_STR);
+std::string opt_solver_third_button_command4(OPT_ARGS_STR);
+std::string opt_solver_fourth_button(OPT_ARGS_STR);
+std::string opt_solver_fourth_button0(OPT_ARGS_STR);
+std::string opt_solver_fourth_button1(OPT_ARGS_STR);
+std::string opt_solver_fourth_button2(OPT_ARGS_STR);
+std::string opt_solver_fourth_button3(OPT_ARGS_STR);
+std::string opt_solver_fourth_button4(OPT_ARGS_STR);
+std::string opt_solver_fourth_button_command(OPT_ARGS_STR);
+std::string opt_solver_fourth_button_command0(OPT_ARGS_STR);
+std::string opt_solver_fourth_button_command1(OPT_ARGS_STR);
+std::string opt_solver_fourth_button_command2(OPT_ARGS_STR);
+std::string opt_solver_fourth_button_command3(OPT_ARGS_STR);
+std::string opt_solver_fourth_button_command4(OPT_ARGS_STR);
+std::string opt_solver_fifth_button(OPT_ARGS_STR);
+std::string opt_solver_fifth_button0(OPT_ARGS_STR);
+std::string opt_solver_fifth_button1(OPT_ARGS_STR);
+std::string opt_solver_fifth_button2(OPT_ARGS_STR);
+std::string opt_solver_fifth_button3(OPT_ARGS_STR);
+std::string opt_solver_fifth_button4(OPT_ARGS_STR);
+std::string opt_solver_fifth_button_command(OPT_ARGS_STR);
+std::string opt_solver_fifth_button_command0(OPT_ARGS_STR);
+std::string opt_solver_fifth_button_command1(OPT_ARGS_STR);
+std::string opt_solver_fifth_button_command2(OPT_ARGS_STR);
+std::string opt_solver_fifth_button_command3(OPT_ARGS_STR);
+std::string opt_solver_fifth_button_command4(OPT_ARGS_STR);
+std::string opt_view_name(OPT_ARGS_STR);
+std::string opt_view_format(OPT_ARGS_STR);
+std::string opt_view_filename(OPT_ARGS_STR);
+std::string opt_view_axes_label0(OPT_ARGS_STR);
+std::string opt_view_axes_label1(OPT_ARGS_STR);
+std::string opt_view_axes_label2(OPT_ARGS_STR);
+std::string opt_view_axes_format0(OPT_ARGS_STR);
+std::string opt_view_axes_format1(OPT_ARGS_STR);
+std::string opt_view_axes_format2(OPT_ARGS_STR);
+std::string opt_view_gen_raise0(OPT_ARGS_STR);
+std::string opt_view_gen_raise1(OPT_ARGS_STR);
+std::string opt_view_gen_raise2(OPT_ARGS_STR);
+std::string opt_view_stipple0(OPT_ARGS_STR);
+std::string opt_view_stipple1(OPT_ARGS_STR);
+std::string opt_view_stipple2(OPT_ARGS_STR);
+std::string opt_view_stipple3(OPT_ARGS_STR);
+std::string opt_view_stipple4(OPT_ARGS_STR);
+std::string opt_view_stipple5(OPT_ARGS_STR);
+std::string opt_view_stipple6(OPT_ARGS_STR);
+std::string opt_view_stipple7(OPT_ARGS_STR);
+std::string opt_view_stipple8(OPT_ARGS_STR);
+std::string opt_view_stipple9(OPT_ARGS_STR);
 
 // NUMBERS
 
@@ -216,14 +218,14 @@ double opt_general_fontsize(OPT_ARGS_NUM);
 double opt_general_graphics_fontsize(OPT_ARGS_NUM);
 double opt_general_graphics_position0(OPT_ARGS_NUM);
 double opt_general_graphics_position1(OPT_ARGS_NUM);
+double opt_general_graphics_size0(OPT_ARGS_NUM);
+double opt_general_graphics_size1(OPT_ARGS_NUM);
 double opt_general_solver_position0(OPT_ARGS_NUM);
 double opt_general_solver_position1(OPT_ARGS_NUM);
 double opt_general_context_position0(OPT_ARGS_NUM);
 double opt_general_context_position1(OPT_ARGS_NUM);
 double opt_general_file_chooser_position0(OPT_ARGS_NUM);
 double opt_general_file_chooser_position1(OPT_ARGS_NUM);
-double opt_general_viewport2(OPT_ARGS_NUM);
-double opt_general_viewport3(OPT_ARGS_NUM);
 double opt_general_polygon_offset_always(OPT_ARGS_NUM);
 double opt_general_polygon_offset_factor(OPT_ARGS_NUM);
 double opt_general_polygon_offset_units(OPT_ARGS_NUM);
@@ -278,6 +280,8 @@ double opt_general_shine(OPT_ARGS_NUM);
 double opt_general_shine_exponent(OPT_ARGS_NUM);
 double opt_general_color_scheme(OPT_ARGS_NUM);
 double opt_general_background_gradient(OPT_ARGS_NUM);
+double opt_general_background_image_position0(OPT_ARGS_NUM);
+double opt_general_background_image_position1(OPT_ARGS_NUM);
 double opt_general_verbosity(OPT_ARGS_NUM);
 double opt_general_nopopup(OPT_ARGS_NUM);
 double opt_general_non_modal_windows(OPT_ARGS_NUM);
@@ -288,7 +292,6 @@ double opt_general_orthographic(OPT_ARGS_NUM);
 double opt_general_mouse_selection(OPT_ARGS_NUM);
 double opt_general_mouse_hover_meshes(OPT_ARGS_NUM);
 double opt_general_draw_bounding_box(OPT_ARGS_NUM);
-double opt_general_draw_all_models(OPT_ARGS_NUM);
 double opt_general_fast_redraw(OPT_ARGS_NUM);
 double opt_general_xmin(OPT_ARGS_NUM);
 double opt_general_xmax(OPT_ARGS_NUM);
@@ -324,32 +327,26 @@ double opt_general_trackball(OPT_ARGS_NUM);
 double opt_general_rotation_center_cg(OPT_ARGS_NUM);
 double opt_general_zoom_factor(OPT_ARGS_NUM);
 double opt_general_expert_mode(OPT_ARGS_NUM);
-double opt_general_clip0(OPT_ARGS_NUM);
 double opt_general_clip0a(OPT_ARGS_NUM);
 double opt_general_clip0b(OPT_ARGS_NUM);
 double opt_general_clip0c(OPT_ARGS_NUM);
 double opt_general_clip0d(OPT_ARGS_NUM);
-double opt_general_clip1(OPT_ARGS_NUM);
 double opt_general_clip1a(OPT_ARGS_NUM);
 double opt_general_clip1b(OPT_ARGS_NUM);
 double opt_general_clip1c(OPT_ARGS_NUM);
 double opt_general_clip1d(OPT_ARGS_NUM);
-double opt_general_clip2(OPT_ARGS_NUM);
 double opt_general_clip2a(OPT_ARGS_NUM);
 double opt_general_clip2b(OPT_ARGS_NUM);
 double opt_general_clip2c(OPT_ARGS_NUM);
 double opt_general_clip2d(OPT_ARGS_NUM);
-double opt_general_clip3(OPT_ARGS_NUM);
 double opt_general_clip3a(OPT_ARGS_NUM);
 double opt_general_clip3b(OPT_ARGS_NUM);
 double opt_general_clip3c(OPT_ARGS_NUM);
 double opt_general_clip3d(OPT_ARGS_NUM);
-double opt_general_clip4(OPT_ARGS_NUM);
 double opt_general_clip4a(OPT_ARGS_NUM);
 double opt_general_clip4b(OPT_ARGS_NUM);
 double opt_general_clip4c(OPT_ARGS_NUM);
 double opt_general_clip4d(OPT_ARGS_NUM);
-double opt_general_clip5(OPT_ARGS_NUM);
 double opt_general_clip5a(OPT_ARGS_NUM);
 double opt_general_clip5b(OPT_ARGS_NUM);
 double opt_general_clip5c(OPT_ARGS_NUM);
@@ -387,6 +384,19 @@ double opt_general_light50(OPT_ARGS_NUM);
 double opt_general_light51(OPT_ARGS_NUM);
 double opt_general_light52(OPT_ARGS_NUM);
 double opt_general_light53(OPT_ARGS_NUM);
+double opt_geometry_transform(OPT_ARGS_NUM);
+double opt_geometry_transform00(OPT_ARGS_NUM);
+double opt_geometry_transform01(OPT_ARGS_NUM);
+double opt_geometry_transform02(OPT_ARGS_NUM);
+double opt_geometry_transform10(OPT_ARGS_NUM);
+double opt_geometry_transform11(OPT_ARGS_NUM);
+double opt_geometry_transform12(OPT_ARGS_NUM);
+double opt_geometry_transform20(OPT_ARGS_NUM);
+double opt_geometry_transform21(OPT_ARGS_NUM);
+double opt_geometry_transform22(OPT_ARGS_NUM);
+double opt_geometry_offset0(OPT_ARGS_NUM);
+double opt_geometry_offset1(OPT_ARGS_NUM);
+double opt_geometry_offset2(OPT_ARGS_NUM);
 double opt_geometry_auto_coherence(OPT_ARGS_NUM);
 double opt_geometry_highlight_orphans(OPT_ARGS_NUM);
 double opt_geometry_tolerance(OPT_ARGS_NUM);
@@ -414,18 +424,20 @@ double opt_geometry_occ_fix_small_faces(OPT_ARGS_NUM);
 double opt_geometry_occ_sew_faces(OPT_ARGS_NUM);
 double opt_geometry_old_circle(OPT_ARGS_NUM);
 double opt_geometry_old_newreg(OPT_ARGS_NUM);
-double opt_geometry_circle_points(OPT_ARGS_NUM);
+double opt_geometry_num_sub_edges(OPT_ARGS_NUM);
 double opt_geometry_extrude_spline_points(OPT_ARGS_NUM);
 double opt_geometry_extrude_return_lateral(OPT_ARGS_NUM);
 double opt_geometry_scaling_factor(OPT_ARGS_NUM);
 double opt_geometry_snap0(OPT_ARGS_NUM);
 double opt_geometry_snap1(OPT_ARGS_NUM);
 double opt_geometry_snap2(OPT_ARGS_NUM);
+double opt_geometry_clip(OPT_ARGS_NUM);
 double opt_mesh_label_frequency(OPT_ARGS_NUM);
 double opt_mesh_optimize(OPT_ARGS_NUM);
 double opt_mesh_optimize_netgen(OPT_ARGS_NUM);
 double opt_mesh_refine_steps(OPT_ARGS_NUM);
 double opt_mesh_normals(OPT_ARGS_NUM);
+double opt_mesh_num_sub_edges(OPT_ARGS_NUM);
 double opt_mesh_tangents(OPT_ARGS_NUM);
 double opt_mesh_explode(OPT_ARGS_NUM);
 double opt_mesh_scaling_factor(OPT_ARGS_NUM);
@@ -476,13 +488,12 @@ double opt_mesh_bdf_field_format(OPT_ARGS_NUM);
 double opt_mesh_nb_smoothing(OPT_ARGS_NUM);
 double opt_mesh_algo2d(OPT_ARGS_NUM);
 double opt_mesh_algo3d(OPT_ARGS_NUM);
-double opt_mesh_recombine_algo(OPT_ARGS_NUM);
+double opt_mesh_algo_subdivide(OPT_ARGS_NUM);
 double opt_mesh_mesh_only_visible(OPT_ARGS_NUM);
 double opt_mesh_min_circ_points(OPT_ARGS_NUM);
 double opt_mesh_allow_swap_edge_angle(OPT_ARGS_NUM);
 double opt_mesh_min_curv_points(OPT_ARGS_NUM);
 double opt_mesh_order(OPT_ARGS_NUM);
-double opt_mesh_c1(OPT_ARGS_NUM);
 double opt_mesh_smooth_internal_edges(OPT_ARGS_NUM);
 double opt_mesh_second_order_experimental(OPT_ARGS_NUM);
 double opt_mesh_second_order_linear(OPT_ARGS_NUM);
@@ -524,7 +535,7 @@ double opt_mesh_partition_chaco_terminal_propogation(OPT_ARGS_NUM);
 double opt_mesh_partition_metis_algorithm(OPT_ARGS_NUM);
 double opt_mesh_partition_metis_edge_matching(OPT_ARGS_NUM);
 double opt_mesh_partition_metis_refine_algorithm(OPT_ARGS_NUM);
-double opt_solver_max_delay(OPT_ARGS_NUM);
+double opt_mesh_clip(OPT_ARGS_NUM);
 double opt_solver_listen(OPT_ARGS_NUM);
 double opt_solver_plugins(OPT_ARGS_NUM);
 double opt_solver_client_server(OPT_ARGS_NUM);
@@ -582,11 +593,8 @@ double opt_view_transform12(OPT_ARGS_NUM);
 double opt_view_transform20(OPT_ARGS_NUM);
 double opt_view_transform21(OPT_ARGS_NUM);
 double opt_view_transform22(OPT_ARGS_NUM);
-double opt_view_arrow_size(OPT_ARGS_NUM);
-double opt_view_arrow_size_proportional(OPT_ARGS_NUM);
-double opt_view_arrow_head_radius(OPT_ARGS_NUM);
-double opt_view_arrow_stem_length(OPT_ARGS_NUM);
-double opt_view_arrow_stem_radius(OPT_ARGS_NUM);
+double opt_view_arrow_size_min(OPT_ARGS_NUM);
+double opt_view_arrow_size_max(OPT_ARGS_NUM);
 double opt_view_normals(OPT_ARGS_NUM);
 double opt_view_tangents(OPT_ARGS_NUM);
 double opt_view_displacement_factor(OPT_ARGS_NUM);
@@ -597,6 +605,7 @@ double opt_view_visible(OPT_ARGS_NUM);
 double opt_view_intervals_type(OPT_ARGS_NUM);
 double opt_view_saturate_values(OPT_ARGS_NUM);
 double opt_view_max_recursion_level(OPT_ARGS_NUM);
+double opt_view_adapt_visualization_grid(OPT_ARGS_NUM);
 double opt_view_target_error(OPT_ARGS_NUM);
 double opt_view_colormap_alpha(OPT_ARGS_NUM);
 double opt_view_colormap_alpha_power(OPT_ARGS_NUM);
@@ -662,6 +671,7 @@ double opt_view_point_size(OPT_ARGS_NUM);
 double opt_view_line_width(OPT_ARGS_NUM);
 double opt_view_point_type(OPT_ARGS_NUM);
 double opt_view_line_type(OPT_ARGS_NUM);
+double opt_view_clip(OPT_ARGS_NUM);
 double opt_print_format(OPT_ARGS_NUM);
 double opt_print_eps_compress(OPT_ARGS_NUM);
 double opt_print_eps_ps3shading(OPT_ARGS_NUM);
@@ -686,6 +696,7 @@ double opt_print_gif_interlace(OPT_ARGS_NUM);
 double opt_print_gif_transparent(OPT_ARGS_NUM);
 double opt_print_text(OPT_ARGS_NUM);
 double opt_print_tex_as_equation(OPT_ARGS_NUM);
+double opt_print_composite_windows(OPT_ARGS_NUM);
 
 // COLORS
 
@@ -764,8 +775,8 @@ typedef struct {
 typedef struct {
   int level;
   const char *str;
-  const char * (*function)(int num, int action, const char *val);
-  const char *def;
+  std::string (*function)(int num, int action, std::string val);
+  std::string def;
   const char *help;
 } StringXString;
 
@@ -788,19 +799,19 @@ typedef struct {
   const char *help;
 } StringXColor;
 
-void Init_Options(int num);
-void Init_Options_GUI(int num);
-void ReInit_Options(int num);
-void Print_Options(int num, int level, int diff, int help, const char *filename);
-void Print_OptionsDoc();
+void InitOptions(int num);
+void InitOptionsGUI(int num);
+void ReInitOptions(int num);
+void PrintOptions(int num, int level, int diff, int help, const char *filename);
+void PrintOptionsDoc();
 
 bool StringOption(int action, const char *category, int num, 
-		  const char *name, const char *val);
+		  const char *name, std::string &val);
 bool NumberOption(int action, const char *category, int num, 
 		  const char *name, double &val);
 bool ColorOption(int action, const char *category, int num, 
 		 const char *name, unsigned int &val);
 
-GmshColorTable *Get_ColorTable(int num);
+GmshColorTable *GetColorTable(int num);
 
 #endif
