@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -6,6 +6,7 @@
 #ifndef _GEDGE_H_
 #define _GEDGE_H_
 
+#include <stdio.h>
 #include "GEntity.h"
 #include "GVertex.h"
 #include "SVector3.h"
@@ -16,6 +17,8 @@ class MElement;
 class MLine;
 class ExtrudeParams;
 
+#include <set>
+
 // A model edge.
 class GEdge : public GEntity {
  private:
@@ -26,6 +29,8 @@ class GEdge : public GEntity {
   GVertex *v0, *v1;
   std::list<GFace *> l_faces;
 
+  std::set<GFace *> bl_faces;
+
  public:
   GEdge(GModel *model, int tag, GVertex *_v0, GVertex *_v1);
   virtual ~GEdge();
@@ -48,7 +53,7 @@ class GEdge : public GEntity {
   virtual void setVisibility(char val, bool recursive=false);
 
   // true if the edge is a seam for the given face.
-  virtual bool isSeam(GFace *face) const { return false; }
+  virtual bool isSeam(const GFace *face) const { return false; }
 
   // get the bounding box
   virtual SBoundingBox3d bounds() const;
@@ -56,19 +61,19 @@ class GEdge : public GEntity {
   // faces that this entity bounds
   virtual std::list<GFace*> faces() const { return l_faces; }
 
-  // get the parameter location for a point in space on the edge
-  // (returns std::numeric_limits<double>::max() if failed)
-  virtual double parFromPoint(const SPoint3 &) const = 0;
-
   // get the point for the given parameter location
   virtual GPoint point(double p) const = 0;
-
-  // get the closest point on the edge to the given point
-  virtual GPoint closestPoint(const SPoint3 & queryPoint) const;
-
+  
   // true if the edge contains the given parameter
   virtual bool containsParam(double pt) const;
 
+  // get the position for the given parameter location
+  virtual SVector3 position(double p) const
+  {
+    GPoint gp = point(p);
+    return SVector3(gp.x(), gp.y(), gp.z());
+  }
+  
   // get first derivative of edge at the given parameter
   virtual SVector3 firstDer(double par) const = 0;
 
@@ -80,7 +85,7 @@ class GEdge : public GEntity {
   virtual double curvature(double par) const;
 
   // reparmaterize the point onto the given face
-  virtual SPoint2 reparamOnFace(GFace *face, double epar, int dir) const;
+  virtual SPoint2 reparamOnFace(const GFace *face, double epar, int dir) const;
 
   // return the minimum number of segments used for meshing the edge
   virtual int minimumMeshSegments() const { return 1; }
@@ -91,6 +96,9 @@ class GEdge : public GEntity {
   // return a type-specific additional information string
   virtual std::string getAdditionalInfoString();
 
+  // export in GEO format
+  virtual void writeGEO(FILE *fp);
+
   // tell if the edge is a 3D edge (in opposition with a trimmed curve on a surface)
   virtual bool is3D() const { return true; }
 
@@ -126,6 +134,25 @@ class GEdge : public GEntity {
   // true if entity is periodic in the "dim" direction.
   virtual bool periodic(int dim) const { return v0 == v1; }
 
+  // true if edge is used in hyperbolic layer on face gf
+  virtual bool inHyperbolicLayer(GFace* gf)
+  {
+    return bl_faces.find(gf) != bl_faces.end();
+  }
+
+  virtual void flagInHyperbolicLayer(GFace* gf) {bl_faces.insert(gf);}
+
+  // get bounds of parametric coordinate 
+  virtual Range<double> parBounds(int i) const = 0;
+  
+  // return the point on the face closest to the given point
+  virtual GPoint closestPoint(const SPoint3 & queryPoint,double& param) const;
+
+  // try to reparametrise the point on the edge
+  virtual double parFromPoint(const SVector3& Q) const;
+  
+  virtual bool XYZToU(const SVector3& Q,double& t,const double relax=0.5) const;
+
   struct {
     char Method;
     double coeffTransfinite;
