@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -77,6 +77,7 @@ class GEntity {
     Bezier,
     ParametricCurve,
     BoundaryLayerCurve,
+    CompoundCurve,
     DiscreteCurve,
     Plane,
     Nurb,
@@ -116,6 +117,7 @@ class GEntity {
       "Bezier",
       "Parametric curve",
       "Boundary layer curve",
+      "Compound curve",
       "Discrete curve",
       "Plane",
       "Nurb",
