@@ -1,16 +1,17 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
 
 #include <sstream>
+#include "GmshConfig.h"
+#include "GmshMessage.h"
 #include "GModel.h"
 #include "MElement.h"
 #include "discreteRegion.h"
 #include "discreteFace.h"
 #include "discreteEdge.h"
 #include "discreteVertex.h"
-#include "GmshMessage.h"
 
 #if defined(HAVE_GMSH_EMBEDDED)
 #include "GmshEmbedded.h"
@@ -23,14 +24,13 @@
 #include "Context.h"
 #endif
 
-extern Context_T CTX;
-
 std::vector<GModel*> GModel::list;
 int GModel::_current = -1;
 
 GModel::GModel(std::string name)
-  : _octree(0), _geo_internals(0), _occ_internals(0), _fm_internals(0), 
-    _fields(0), _currentMeshEntity(0), modelName(name), normals(0)
+  : _name(name), _visible(1), _octree(0), 
+    _geo_internals(0), _occ_internals(0), _fm_internals(0), 
+    _fields(0), _currentMeshEntity(0), normals(0)
 {
   partitionSize[0] = 0; partitionSize[1] = 0;
   list.push_back(this);
@@ -58,9 +58,11 @@ GModel::~GModel()
 
 GModel *GModel::current(int index)
 {
+  if(list.empty()){
+    Msg::Error("No current model available: creating one");
+    new GModel();
+  }
   if(index >= 0) _current = index;
-  if(list.empty()) return 0; // not an error
-
   if(_current < 0 || _current >= (int)list.size()) return list.back();
   return list[_current];
 }
@@ -75,6 +77,8 @@ GModel *GModel::findByName(std::string name)
 
 void GModel::destroy()
 {
+  _name.clear();
+
   for(riter it = firstRegion(); it != lastRegion(); ++it)
     delete *it;
   regions.clear();
@@ -196,7 +200,7 @@ void GModel::snapVertices()
 {
   viter vit = firstVertex();
 
-  double tol = CTX.geom.tolerance;
+  double tol = CTX::instance()->geom.tolerance;
 
   while (vit != lastVertex()){
     std::list<GEdge*> edges = (*vit)->edges();
@@ -311,7 +315,9 @@ int GModel::setPhysicalName(std::string name, int number)
 
 std::string GModel::getPhysicalName(int number)
 {
-  return physicalNames[number];
+  if(physicalNames.count(number))
+    return physicalNames[number];
+  return "";
 }
 
 SBoundingBox3d GModel::bounds()
@@ -868,14 +874,14 @@ int GModel::removeDuplicateMeshVertices(double tolerance)
       MElement *e2 = factory.create(e->getTypeForMSH(), verts, e->getNum(), 
                                     e->getPartition());
       switch(e2->getNumEdges()){
-      case 0: elements[0][entities[i]->tag()].push_back(e2); break;
-      case 1: elements[1][entities[i]->tag()].push_back(e2); break;
-      case 3: elements[2][entities[i]->tag()].push_back(e2); break;
-      case 4: elements[3][entities[i]->tag()].push_back(e2); break;
-      case 6: elements[4][entities[i]->tag()].push_back(e2); break;
+      case 0:  elements[0][entities[i]->tag()].push_back(e2); break;
+      case 1:  elements[1][entities[i]->tag()].push_back(e2); break;
+      case 3:  elements[2][entities[i]->tag()].push_back(e2); break;
+      case 4:  elements[3][entities[i]->tag()].push_back(e2); break;
+      case 6:  elements[4][entities[i]->tag()].push_back(e2); break;
       case 12: elements[5][entities[i]->tag()].push_back(e2); break;
-      case 9: elements[6][entities[i]->tag()].push_back(e2); break;
-      case 8: elements[7][entities[i]->tag()].push_back(e2); break;
+      case 9:  elements[6][entities[i]->tag()].push_back(e2); break;
+      case 8:  elements[7][entities[i]->tag()].push_back(e2); break;
       }
     }
   }
@@ -899,3 +905,20 @@ int GModel::removeDuplicateMeshVertices(double tolerance)
     
   return diff;
 }
+
+void GModel::setSelection(int val)
+{
+  std::vector<GEntity*> entities;
+  getEntities(entities);
+
+  for(unsigned int i = 0; i < entities.size(); i++){
+    entities[i]->setSelection(val);
+    // reset selection in elements (stored in the visibility flag to
+    // save space)
+    if(val == 0){
+      for(int j = 0; j < entities[i]->getNumMeshElements(); j++)
+        if(entities[i]->getMeshElement(j)->getVisibility() == 2)
+          entities[i]->getMeshElement(j)->setVisibility(1);
+    }
+  }  
+}
