@@ -1,10 +1,12 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
+
 #include <string>
-#include "GModel.h"
+#include "GmshConfig.h"
 #include "GmshMessage.h"
+#include "GModel.h"
 #include "fourierVertex.h"
 #include "fourierEdge.h"
 #include "fourierFace.h"
@@ -142,14 +144,14 @@ void GModel::_deleteFMInternals()
 
 int GModel::readFourier(const std::string &fn)
 {
-  Msg::Error("Gmsh has to be compiled with Fourier Model support to load '%s'",
+  Msg::Error("Gmsh must be compiled with Fourier Model support to load '%s'",
       fn.c_str());
   return 0;
 }
 
 int GModel::writeFourier(const std::string &fn)
 {
-  Msg::Error("Gmsh has to be compiled with Fourier Model support to load '%s'",
+  Msg::Error("Gmsh must be compiled with Fourier Model support to load '%s'",
 	     fn.c_str());
   return 0;
 }
