@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -169,7 +169,7 @@ int GModel::readMSH(const std::string &name)
       vertexMap.clear();
       int minVertex = numVertices + 1, maxVertex = -1;
       for(int i = 0; i < numVertices; i++) {
-        int num, iClasDim, iClasTag;	
+        int num;	
         double xyz[3], uv[2];
 	MVertex *newVertex = 0;
 	if (!parametric){
@@ -186,6 +186,7 @@ int GModel::readMSH(const std::string &name)
 	  newVertex = new MVertex(xyz[0], xyz[1], xyz[2], 0, num);
 	}
 	else{
+          int iClasDim, iClasTag;
 	  if(!binary){	    
 	    if (fscanf(fp, "%d %lf %lf %lf %d %d", &num, &xyz[0], &xyz[1], &xyz[2],
                        &iClasDim, &iClasTag) != 6)
@@ -330,7 +331,7 @@ int GModel::readMSH(const std::string &name)
             }
             createElementMSH(this, num, type, physical, elementary, partition, 
                              vertices, elements, physicals);
-            if(numElements > 100000) 
+            if(numElements > 100000)
               Msg::ProgressMeter(numElementsPartial + i + 1, numElements, 
                                  "Reading elements");
           }
@@ -386,6 +387,7 @@ int GModel::readMSH(const std::string &name)
     storePhysicalTagsInEntities(this, i, physicals[i]);
 
   fclose(fp);
+
   return postpro ? 2 : 1;
 }
 
@@ -676,7 +678,11 @@ int GModel::readSTL(const std::string &name, double tolerance)
   // "solid", or binary data header
   char buffer[256];
   fgets(buffer, sizeof(buffer), fp);
-  
+
+  // workaround for stupid 3D-DOCTOR software, which uses "solid" to
+  // start its binary files
+  if(!strncmp(buffer, "solid 3D-DOCTOR", 15)) buffer[0] = 'z';
+
   if(!strncmp(buffer, "solid", 5)){
     // ASCII STL
     points.resize(1);
@@ -720,7 +726,7 @@ int GModel::readSTL(const std::string &name, double tolerance)
       unsigned int nfacets = 0;
       size_t ret = fread(&nfacets, sizeof(unsigned int), 1, fp);
       bool swap = false;
-      if(nfacets > 10000000){
+      if(nfacets > 100000000){
         Msg::Info("Swapping bytes from binary file");
         swap = true;
         SwapBytes((char*)&nfacets, sizeof(unsigned int), 1);
