@@ -1,10 +1,11 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
 
-#include "GModelIO_OCC.h"
+#include "GmshConfig.h"
 #include "GmshMessage.h"
+#include "GModelIO_OCC.h"
 #include "Context.h"
 #include "OCCVertex.h"
 #include "OCCEdge.h"
@@ -19,8 +20,6 @@
 #include "MeshGmsh_EdgeConstrain.hxx"
 #endif
 
-extern Context_T CTX;
-
 #if defined(HAVE_OCC)
 
 void OCC_Internals::buildLists()
@@ -347,10 +346,10 @@ void OCC_Internals::loadBREP(const char *fn)
   BRep_Builder aBuilder;
   BRepTools::Read(shape, (char*)fn, aBuilder);
   BRepTools::Clean(shape);
-  healGeometry(CTX.geom.tolerance, 
-               CTX.geom.occ_fix_small_edges,
-               CTX.geom.occ_fix_small_faces,
-               CTX.geom.occ_sew_faces);
+  healGeometry(CTX::instance()->geom.tolerance, 
+               CTX::instance()->geom.occFixSmallEdges,
+               CTX::instance()->geom.occFixSmallFaces,
+               CTX::instance()->geom.occSewFaces);
   BRepTools::Clean(shape);
   buildLists();
 }
@@ -363,10 +362,10 @@ void OCC_Internals::loadSTEP(const char *fn)
   reader.TransferRoots(); 
   shape = reader.OneShape();  
   BRepTools::Clean(shape);
-  healGeometry(CTX.geom.tolerance, 
-               CTX.geom.occ_fix_small_edges,
-               CTX.geom.occ_fix_small_faces,
-               CTX.geom.occ_sew_faces);
+  healGeometry(CTX::instance()->geom.tolerance, 
+               CTX::instance()->geom.occFixSmallEdges,
+               CTX::instance()->geom.occFixSmallFaces,
+               CTX::instance()->geom.occSewFaces);
   BRepTools::Clean(shape);
   buildLists();
 }
@@ -379,10 +378,10 @@ void OCC_Internals::loadIGES(const char *fn)
   reader.TransferRoots(); 
   shape = reader.OneShape();  
   BRepTools::Clean(shape);
-  healGeometry(CTX.geom.tolerance, 
-               CTX.geom.occ_fix_small_edges,
-               CTX.geom.occ_fix_small_faces,
-               CTX.geom.occ_sew_faces);
+  healGeometry(CTX::instance()->geom.tolerance, 
+               CTX::instance()->geom.occFixSmallEdges,
+               CTX::instance()->geom.occFixSmallFaces,
+               CTX::instance()->geom.occSewFaces);
   BRepTools::Clean(shape);
   buildLists();
 }
@@ -700,28 +699,28 @@ void GModel::_deleteOCCInternals()
 
 int GModel::readOCCSTEP(const std::string &fn)
 {
-  Msg::Error("Gmsh has to be compiled with OpenCascade support to load '%s'",
+  Msg::Error("Gmsh must be compiled with OpenCascade support to load '%s'",
 	     fn.c_str());
   return 0;
 }
 
 int GModel::readOCCIGES(const std::string &fn)
 {
-  Msg::Error("Gmsh has to be compiled with OpenCascade support to load '%s'",
+  Msg::Error("Gmsh must be compiled with OpenCascade support to load '%s'",
 	     fn.c_str());
   return 0;
 }
 
 int GModel::readOCCBREP(const std::string &fn)
 {
-  Msg::Error("Gmsh has to be compiled with OpenCascade support to load '%s'",
+  Msg::Error("Gmsh must be compiled with OpenCascade support to load '%s'",
 	     fn.c_str());
   return 0;
 }
 
 int GModel::importOCCShape(const void *shape, const void *options)
 {
-  Msg::Error("Gmsh has to be compiled with OpenCascade support to import "
+  Msg::Error("Gmsh must be compiled with OpenCascade support to import "
 	     "a TopoDS_Shape");
   return 0;
 }
