@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -42,7 +42,7 @@ void GVertex::delEdge(GEdge *e)
   l_edges.erase(std::find(l_edges.begin(), l_edges.end(), e));
 }
 
-SPoint2 GVertex::reparamOnFace(GFace *gf, int) const
+SPoint2 GVertex::reparamOnFace(const GFace *gf, int) const
 {
   return gf->parFromPoint(SPoint3(x(), y(), z()));
 }
@@ -56,6 +56,12 @@ std::string GVertex::getAdditionalInfoString()
   return sstream.str();
 }
 
+void GVertex::writeGEO(FILE *fp)
+{
+  fprintf(fp, "Point(%d) = {%.16g, %.16g, %.16g, %.16g};\n",
+          tag(), x(), y(), z(), prescribedMeshSizeAtVertex());
+}
+
 unsigned int GVertex::getNumMeshElements()
 {
   return points.size(); 
