@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -15,8 +15,6 @@
 
 #define SQU(a)      ((a)*(a))
 
-extern Context_T CTX;
-
 static List_T *ListOfTransformedPoints = NULL;
 
 // Comparison routines
@@ -33,8 +31,8 @@ static int comparePosition(const void *a, const void *b)
   Vertex *q = *(Vertex **)a;
   Vertex *w = *(Vertex **)b;
 
-  // Warning: tolerance! (before 1.61, it was set to 1.e-10 * CTX.lc)
-  double eps = CTX.geom.tolerance * CTX.lc; 
+  // Warning: tolerance! (before 1.61, it was set to 1.e-10 * CTX::instance()->lc)
+  double eps = CTX::instance()->geom.tolerance * CTX::instance()->lc; 
 
   if(q->Pos.X - w->Pos.X > eps) return 1;
   if(q->Pos.X - w->Pos.X < -eps) return -1;
@@ -126,6 +124,7 @@ Vertex *Create_Vertex(int Num, double u, double v, gmshSurface *surf, double lc)
   pV->u = u;
   pV->geometry = surf;
   pV->pntOnGeometry = SPoint2(u,v);
+  surf->vertex_defined_on_surface=true;
   return pV;
 }
 
@@ -330,7 +329,7 @@ void End_Curve(Curve *c)
     mat[0][2] = c->Circle.invmat[2][0] = dir12[2];
 
     // assume circle in z=0 plane
-    if(CTX.geom.old_circle) {
+    if(CTX::instance()->geom.oldCircle) {
       if(n[0] == 0.0 && n[1] == 0.0) {
         mat[2][0] = c->Circle.invmat[0][2] = 0;
         mat[2][1] = c->Circle.invmat[1][2] = 0;
@@ -424,7 +423,7 @@ void End_Curve(Curve *c)
     for(int i = 0; i < 4; i++)
       c->Circle.v[i] = v[i];
 
-    if(!CTX.expert_mode && c->Num > 0 && A3 - A1 > 1.01 * M_PI){
+    if(!CTX::instance()->expertMode && c->Num > 0 && A3 - A1 > 1.01 * M_PI){
       Msg::Error("Circle or ellipse arc %d greater than Pi (angle=%g)", c->Num, A3-A1);
       Msg::Error("(If you understand what this implies, you can disable this error");
       Msg::Error("message by selecting `Enable expert mode' in the option dialog.");
@@ -657,7 +656,7 @@ int NEWPOINT(void)
 
 int NEWLINE(void)
 {
-  if(CTX.geom.old_newreg)
+  if(CTX::instance()->geom.oldNewreg)
     return NEWREG();
   else
     return (GModel::current()->getGEOInternals()->MaxLineNum + 1);
@@ -665,7 +664,7 @@ int NEWLINE(void)
 
 int NEWLINELOOP(void)
 {
-  if(CTX.geom.old_newreg)
+  if(CTX::instance()->geom.oldNewreg)
     return NEWREG();
   else
     return (GModel::current()->getGEOInternals()->MaxLineLoopNum + 1);
@@ -673,7 +672,7 @@ int NEWLINELOOP(void)
 
 int NEWSURFACE(void)
 {
-  if(CTX.geom.old_newreg)
+  if(CTX::instance()->geom.oldNewreg)
     return NEWREG();
   else
     return (GModel::current()->getGEOInternals()->MaxSurfaceNum + 1);
@@ -681,7 +680,7 @@ int NEWSURFACE(void)
 
 int NEWSURFACELOOP(void)
 {
-  if(CTX.geom.old_newreg)
+  if(CTX::instance()->geom.oldNewreg)
     return NEWREG();
   else
     return (GModel::current()->getGEOInternals()->MaxSurfaceLoopNum + 1);
@@ -689,7 +688,7 @@ int NEWSURFACELOOP(void)
 
 int NEWVOLUME(void)
 {
-  if(CTX.geom.old_newreg)
+  if(CTX::instance()->geom.oldNewreg)
     return NEWREG();
   else
     return (GModel::current()->getGEOInternals()->MaxVolumeNum + 1);
@@ -697,12 +696,12 @@ int NEWVOLUME(void)
 
 int NEWFIELD(void)
 {
-  return (GModel::current()->getFields()->max_id() + 1);
+  return (GModel::current()->getFields()->maxId() + 1);
 }
 
 int NEWPHYSICAL(void)
 {
-  if(CTX.geom.old_newreg)
+  if(CTX::instance()->geom.oldNewreg)
     return NEWREG();
   else
     return (GModel::current()->getGEOInternals()->MaxPhysicalNum + 1);
@@ -914,13 +913,6 @@ static void CopySurface(Surface *s, Surface *ss)
   //ss->Method = s->Method;
   //ss->Recombine = s->Recombine;
   //ss->RecombineAngle = s->RecombineAngle;
-  ss->a = s->a;
-  ss->b = s->b;
-  ss->c = s->c;
-  ss->d = s->d;
-  for(i = 0; i < 3; i++)
-    for(j = 0; j < 3; j++)
-      ss->plan[i][j] = s->plan[i][j];
   ss->Generatrices = List_Create(List_Nbr(s->Generatrices), 1, sizeof(Curve *));
   List_Copy(s->Generatrices, ss->Generatrices);
   End_Surface(ss);
@@ -1602,7 +1594,7 @@ static void ApplyTransformationToPoint(double matrix[4][4], Vertex *v,
   // OK if the guy who builds the geometry knowns what he's
   // doing). Instead of adding one more option, let's just bypass all
   // the checks if auto_coherence==0...
-  if(CTX.geom.auto_coherence && end_curve_surface){
+  if(CTX::instance()->geom.autoCoherence && end_curve_surface){
     List_T *All = Tree2List(GModel::current()->getGEOInternals()->Curves);
     for(int i = 0; i < List_Nbr(All); i++) {
       Curve *c;
@@ -1729,7 +1721,7 @@ void TranslateShapes(double X, double Y, double Z, List_T *shapes)
   SetTranslationMatrix(matrix, T);
   ApplicationOnShapes(matrix, shapes);
 
-  if(CTX.geom.auto_coherence)
+  if(CTX::instance()->geom.autoCoherence)
     ReplaceAllDuplicates();
 }
 
@@ -1743,7 +1735,7 @@ void DilatShapes(double X, double Y, double Z, double A, List_T *shapes)
   SetDilatationMatrix(matrix, T, A);
   ApplicationOnShapes(matrix, shapes);
 
-  if(CTX.geom.auto_coherence)
+  if(CTX::instance()->geom.autoCoherence)
     ReplaceAllDuplicates();
 }
 
@@ -1771,7 +1763,7 @@ void RotateShapes(double Ax, double Ay, double Az,
   SetTranslationMatrix(matrix, T);
   ApplicationOnShapes(matrix, shapes);
 
-  if(CTX.geom.auto_coherence)
+  if(CTX::instance()->geom.autoCoherence)
     ReplaceAllDuplicates();
 }
 
@@ -1782,7 +1774,7 @@ void SymmetryShapes(double A, double B, double C, double D, List_T *shapes)
   SetSymmetryMatrix(matrix, A, B, C, D);
   ApplicationOnShapes(matrix, shapes);
 
-  if(CTX.geom.auto_coherence)
+  if(CTX::instance()->geom.autoCoherence)
     ReplaceAllDuplicates();
 }
 
@@ -2008,18 +2000,18 @@ static int Extrude_ProtudePoint(int type, int ip,
     c->end = chapeau;
     break;
   case TRANSLATE_ROTATE:
-    d = CTX.geom.extrude_spline_points;
+    d = CTX::instance()->geom.extrudeSplinePoints;
     d = d ? d : 1;
     c = Create_Curve(NEWLINE(), MSH_SEGM_SPLN, 1, NULL, NULL, -1, -1, 0., 1.);
     c->Control_Points =
-      List_Create(CTX.geom.extrude_spline_points + 1, 1, sizeof(Vertex *));
+      List_Create(CTX::instance()->geom.extrudeSplinePoints + 1, 1, sizeof(Vertex *));
     c->Extrude = new ExtrudeParams;
     c->Extrude->fill(type, T0, T1, T2, A0, A1, A2, X0, X1, X2, alpha);
     if(e)
       c->Extrude->mesh = e->mesh;
     List_Add(c->Control_Points, &pv);
     c->beg = pv;
-    for(i = 0; i < CTX.geom.extrude_spline_points; i++) {
+    for(i = 0; i < CTX::instance()->geom.extrudeSplinePoints; i++) {
       if(i)
         chapeau = DuplicateVertex(chapeau);
       T[0] = -X0;
@@ -2063,7 +2055,7 @@ static int Extrude_ProtudePoint(int type, int ip,
 
   List_Reset(ListOfTransformedPoints);
 
-  if(CTX.geom.auto_coherence && final)
+  if(CTX::instance()->geom.autoCoherence && final)
     ReplaceAllDuplicates();
 
   return chapeau->Num;
@@ -2223,7 +2215,7 @@ static int Extrude_ProtudeCurve(int type, int ic,
 
   *ps = s;
 
-  if(CTX.geom.auto_coherence && final)
+  if(CTX::instance()->geom.autoCoherence && final)
     ReplaceAllDuplicates();
 
   return chapeau->Num;
@@ -2278,16 +2270,16 @@ static int Extrude_ProtudeSurface(int type, int is,
   // FIXME: this is a really ugly hack for backward compatibility, so
   // that we don't screw up the old .geo files too much. (Before
   // version 1.54, we didn't always create new volumes during "Extrude
-  // Surface". Now we do, but with "CTX.geom.old_newreg==1", this
+  // Surface". Now we do, but with "CTX::instance()->geom.oldNewreg==1", this
   // bumps the NEWREG() counter, and thus changes the whole automatic
-  // numbering sequence.) So we locally force old_newreg to 0: in most
+  // numbering sequence.) So we locally force oldNewreg to 0: in most
   // cases, since we define points, curves, etc., before defining
   // volumes, the NEWVOLUME() call below will return a fairly low
   // number, that will not interfere with the other numbers...
-  int tmp = CTX.geom.old_newreg;
-  CTX.geom.old_newreg = 0;
+  int tmp = CTX::instance()->geom.oldNewreg;
+  CTX::instance()->geom.oldNewreg = 0;
   Volume *v = Create_Volume(NEWVOLUME(), MSH_VOLUME);
-  CTX.geom.old_newreg = tmp;
+  CTX::instance()->geom.oldNewreg = tmp;
 
   v->Extrude = new ExtrudeParams;
   v->Extrude->fill(type, T0, T1, T2, A0, A1, A2, X0, X1, X2, alpha);
@@ -2397,7 +2389,7 @@ static int Extrude_ProtudeSurface(int type, int is,
 
   *pv = v;
 
-  if(CTX.geom.auto_coherence)
+  if(CTX::instance()->geom.autoCoherence)
     ReplaceAllDuplicates();
 
   List_Reset(ListOfTransformedPoints);
@@ -2478,7 +2470,7 @@ void ExtrudeShapes(int type, List_T *list_in,
           body.Num = ps->Num;
           body.Type = ps->Typ;
           List_Add(list_out, &body);
-          if(CTX.geom.extrude_return_lateral){
+          if(CTX::instance()->geom.extrudeReturnLateral){
             for(int j = 0; j < List_Nbr(ps->Generatrices); j++){
               Curve *c;
               List_Read(ps->Generatrices, j, &c);
@@ -2511,7 +2503,7 @@ void ExtrudeShapes(int type, List_T *list_in,
           body.Num = pv->Num;
           body.Type = pv->Typ;
           List_Add(list_out, &body);
-          if(CTX.geom.extrude_return_lateral){
+          if(CTX::instance()->geom.extrudeReturnLateral){
             for(int j = 0; j < List_Nbr(pv->Surfaces); j++){
               Surface *s;
               List_Read(pv->Surfaces, j, &s);
@@ -2673,7 +2665,7 @@ static void ReplaceDuplicatePoints()
 
   Msg::Debug("Removed %d duplicate points", start - end);
 
-  if(CTX.geom.old_newreg) {
+  if(CTX::instance()->geom.oldNewreg) {
     GModel::current()->getGEOInternals()->MaxPointNum = 0;
     Tree_Action(GModel::current()->getGEOInternals()->Points, MaxNumPoint);
   }
@@ -2784,7 +2776,7 @@ static void ReplaceDuplicateCurves()
 
   Msg::Debug("Removed %d duplicate curves", start - end);
 
-  if(CTX.geom.old_newreg) {
+  if(CTX::instance()->geom.oldNewreg) {
     GModel::current()->getGEOInternals()->MaxLineNum = 0;
     Tree_Action(GModel::current()->getGEOInternals()->Curves, MaxNumCurve);
   }
@@ -2846,7 +2838,7 @@ static void ReplaceDuplicateSurfaces()
 
   Msg::Debug("Removed %d duplicate surfaces", start - end);
 
-  if(CTX.geom.old_newreg) {
+  if(CTX::instance()->geom.oldNewreg) {
     GModel::current()->getGEOInternals()->MaxSurfaceNum = 0;
     Tree_Action(GModel::current()->getGEOInternals()->Surfaces, MaxNumSurface);
   } 
@@ -2876,87 +2868,92 @@ void ReplaceAllDuplicates()
   ReplaceDuplicateSurfaces();
 }
 
+// Projection of a point on a surface
 
-// Projection of a point on a curve or a surface
+struct PointSurface{
+  Vertex *p;
+  Surface *s;
+};
 
-static Curve *CURVE;
-static Surface *SURFACE;
-static Vertex *VERTEX;
-
-static double min1d(double (*funct) (double), double *xmin)
+static void projectPS(gmshVector<double> &x, gmshVector<double> &res, void *data)
 {
-  // 0. for tolerance allows for maximum as code in gsl_brent
-  return (brent(CURVE->ubeg, 0.5*(CURVE->ubeg + CURVE->uend), CURVE->uend,
-                funct, 0., xmin));
+  PointSurface *ps = (PointSurface*)data;
+  Vertex c = InterpolateSurface(ps->s, x(0), x(1), 0, 0);
+  Vertex du = InterpolateSurface(ps->s, x(0), x(1), 1, 1);
+  Vertex dv = InterpolateSurface(ps->s, x(0), x(1), 1, 2);
+  res(0) =
+    (c.Pos.X - ps->p->Pos.X) * du.Pos.X +
+    (c.Pos.Y - ps->p->Pos.Y) * du.Pos.Y +
+    (c.Pos.Z - ps->p->Pos.Z) * du.Pos.Z;
+  res(1) =
+    (c.Pos.X - ps->p->Pos.X) * dv.Pos.X +
+    (c.Pos.Y - ps->p->Pos.Y) * dv.Pos.Y +
+    (c.Pos.Z - ps->p->Pos.Z) * dv.Pos.Z;
 }
 
-static void projectPS(int N, double x[], double res[])
+bool ProjectPointOnSurface(Surface *s, Vertex &p, double uv[2])
 {
-  //x[1] = u x[2] = v
-  Vertex du, dv, c;
-  c = InterpolateSurface(SURFACE, x[1], x[2], 0, 0);
-  du = InterpolateSurface(SURFACE, x[1], x[2], 1, 1);
-  dv = InterpolateSurface(SURFACE, x[1], x[2], 1, 2);
-  res[1] =
-    (c.Pos.X - VERTEX->Pos.X) * du.Pos.X +
-    (c.Pos.Y - VERTEX->Pos.Y) * du.Pos.Y +
-    (c.Pos.Z - VERTEX->Pos.Z) * du.Pos.Z;
-  res[2] =
-    (c.Pos.X - VERTEX->Pos.X) * dv.Pos.X +
-    (c.Pos.Y - VERTEX->Pos.Y) * dv.Pos.Y +
-    (c.Pos.Z - VERTEX->Pos.Z) * dv.Pos.Z;
-}
+  gmshVector<double> x(2);
+  x(0) = uv[0];
+  x(1) = uv[1];
+  PointSurface ps = {&p, s};
 
-static double projectPC(double u)
-{
-  Vertex c = InterpolateCurve(CURVE, u, 0);
-  return sqrt(SQU(c.Pos.X - VERTEX->Pos.X) +
-              SQU(c.Pos.Y - VERTEX->Pos.Y) + 
-              SQU(c.Pos.Z - VERTEX->Pos.Z));
-}
+  Vertex pp = InterpolateSurface(s, uv[0], uv[1], 0, 0);
+  double d2 = 
+    (pp.Pos.X - p.Pos.X)*(pp.Pos.X - p.Pos.X) + 
+    (pp.Pos.Y - p.Pos.Y)*(pp.Pos.Y - p.Pos.Y) + 
+    (pp.Pos.Z - p.Pos.Z)*(pp.Pos.Z - p.Pos.Z) ;
+  if(d2 < 1.e-12) return true;
 
-bool ProjectPointOnCurve(Curve *c, Vertex *v, Vertex *RES, Vertex *DER)
-{
-  double xmin;
-  CURVE = c;
-  VERTEX = v;
-  min1d(projectPC, &xmin);
-  *RES = InterpolateCurve(CURVE, xmin, 0);
-  *DER = InterpolateCurve(CURVE, xmin, 1);
-  if(xmin > c->uend) {
-    xmin = c->uend;
-    *RES = InterpolateCurve(CURVE, c->uend, 0);
-    *DER = InterpolateCurve(CURVE, c->uend, 1);
+  double UMIN = 0.;
+  double UMAX = 1.;
+  double VMIN = 0.;
+  double VMAX = 1.;
+  int ITER = 0;
+  while(1) {
+    bool success = newton_fd(projectPS, x, &ps);
+    if(success && x(0) >= UMIN && x(0) <= UMAX && x(1) >= VMIN && x(1) <= VMAX){
+      p = InterpolateSurface(s, x(0), x(1), 0, 0);
+      uv[0] = x(0);
+      uv[1] = x(1);
+      if (ITER > 0)
+        Msg::Info("ProjectPoint (%g,%g,%g) On Surface %d converged after %d iterations",
+                  p.Pos.X, p.Pos.Y, p.Pos.Z, s->Num, ITER);
+      return true;
+    }
+    x(0) = UMIN + (UMAX - UMIN) * ((rand() % 10000) / 10000.);
+    x(1) = VMIN + (VMAX - VMIN) * ((rand() % 10000) / 10000.);
+    if (ITER++ > 100)break;
   }
-  else if(xmin < c->ubeg) {
-    xmin = c->ubeg;
-    *RES = InterpolateCurve(CURVE, c->ubeg, 0);
-    *DER = InterpolateCurve(CURVE, c->ubeg, 1);
-  }  
-  return true;
-}
-
-bool ProjectPointOnSurface(Surface *s, Vertex &p, double u[2])
-{
-  double x[3] = { 0.5, u[0], u[1] };
-  int check;
-  SURFACE = s;
-  VERTEX = &p;
-
-  newt(x, 2, &check, projectPS);
-  Vertex vv = InterpolateSurface(s, x[1], x[2], 0, 0);
-  double res[3];
-  projectPS(2, x, res);
-  double resid = sqrt(res[1] * res[1] + res[2] * res[2]);
-
-  p.Pos.X = vv.Pos.X;
-  p.Pos.Y = vv.Pos.Y;
-  p.Pos.Z = vv.Pos.Z;
-  u[0] = x[1];
-  u[1] = x[2];
-  if(resid > 1.e-6)
-    return false;  
-  return true;
+  {
+    int NSAMPLES = 500;
+    double uok, vok;
+    double dmin = 1.e22;
+    for (int i = 0; i < NSAMPLES; i++){
+      const double U = i / (double)(NSAMPLES - 1);
+      for (int j = 0; j < NSAMPLES; j++){
+	const double V = j / (double)(NSAMPLES - 1);
+	Vertex pp = InterpolateSurface(s, U, V, 0, 0);
+	double d2 =
+          (pp.Pos.X - p.Pos.X) * (pp.Pos.X - p.Pos.X) + 
+	  (pp.Pos.Y - p.Pos.Y) * (pp.Pos.Y - p.Pos.Y) + 
+	  (pp.Pos.Z - p.Pos.Z) * (pp.Pos.Z - p.Pos.Z);
+	if (d2 < dmin) {
+	  dmin = d2;
+	  uok = U;
+	  vok = V;
+	}
+      }
+    }
+    p = InterpolateSurface(s, uok, vok, 0, 0);
+    uv[0] = uok;
+    uv[1] = vok;
+    if (ITER > 0)
+      Msg::Info("Brute force method used for projection of point (%g %g %g) on surface %d",
+                p.Pos.X, p.Pos.Y, p.Pos.Z, s->Num);
+    return true;
+  }
+  return false;
 }
 
 // Split line
@@ -3057,9 +3054,11 @@ bool SplitCurve(int line_id, List_T *vertices_id, List_T *shapes)
       if(surface_curve->Num == c->Num){
         List_Remove(s->Generatrices, j);
         List_Insert_In_List(shapes, j, s->Generatrices);
+        j+=List_Nbr(shapes)-1;
       }else if(surface_curve->Num == -c->Num){
         List_Remove(s->Generatrices, j);
         List_Insert_In_List(rshapes, j, s->Generatrices);
+        j+=List_Nbr(shapes)-1;
       }
     }
   }
@@ -3072,24 +3071,19 @@ bool SplitCurve(int line_id, List_T *vertices_id, List_T *shapes)
 
 // Intersect a curve with a surface
 
-static void intersectCS(int N, double x[], double res[])
-{
-  // (x[1], x[2]) = surface params, x[3] = curve param
-  Vertex s = InterpolateSurface(SURFACE, x[1], x[2], 0, 0);
-  Vertex c = InterpolateCurve(CURVE, x[3], 0);
-  res[1] = s.Pos.X - c.Pos.X;
-  res[2] = s.Pos.Y - c.Pos.Y;
-  res[3] = s.Pos.Z - c.Pos.Z;
-}
+struct CurveSurface{
+  Curve *c;
+  Surface *s;
+};
 
-static bool IntersectCurveSurface(Curve *c, Surface *s, double x[4])
+static void intersectCS(gmshVector<double> &uvt, gmshVector<double> &res, void *data)
 {
-  int check;
-  SURFACE = s;
-  CURVE = c;
-  newt(x, 3, &check, intersectCS);
-  if(check) return false;
-  return true;
+  CurveSurface *cs = (CurveSurface*)data;
+  Vertex vs = InterpolateSurface(cs->s, uvt(0), uvt(1), 0, 0);
+  Vertex vc = InterpolateCurve(cs->c, uvt(2), 0);
+  res(0) = vs.Pos.X - vc.Pos.X;
+  res(1) = vs.Pos.Y - vc.Pos.Y;
+  res(2) = vs.Pos.Z - vc.Pos.Z;
 }
 
 bool IntersectCurvesWithSurface(List_T *curve_ids, int surface_id, List_T *shapes)
@@ -3104,15 +3098,19 @@ bool IntersectCurvesWithSurface(List_T *curve_ids, int surface_id, List_T *shape
     List_Read(curve_ids, i, &curve_id);
     Curve *c = FindCurve((int)curve_id);
     if(c){
-      double x[4] = {0., 0.5, 0.5, 0.5};
-      if(IntersectCurveSurface(c, s, x)){
-        Vertex p = InterpolateCurve(c, x[3], 0);
+      CurveSurface cs = {c, s};
+      gmshVector<double> uvt(3);
+      uvt(0) = 0.5;
+      uvt(1) = 0.5;
+      uvt(2) = 0.5;
+      if(newton_fd(intersectCS, uvt, &cs)){
+        Vertex p = InterpolateCurve(c, uvt(2), 0);
         Vertex *v = Create_Vertex(NEWPOINT(), p.Pos.X, p.Pos.Y, p.Pos.Z, p.lc, p.u);
         Tree_Insert(GModel::current()->getGEOInternals()->Points, &v);
-        Shape s;
-        s.Type = MSH_POINT;
-        s.Num = v->Num;
-        List_Add(shapes, &s);
+        Shape sh;
+        sh.Type = MSH_POINT;
+        sh.Num = v->Num;
+        List_Add(shapes, &sh);
       }
     }
     else{
@@ -3140,8 +3138,10 @@ void sortEdgesInLoop(int num, List_T *edges)
     List_Read(edges, i, &j);
     if((c = FindCurve(j)))
       List_Add(temp, &c);
-    else
+    else{
+      return;
       Msg::Error("Unknown curve %d in line loop %d", j, num);
+    }
   }
   List_Reset(edges);
 
