@@ -1,15 +1,15 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
 
 #include <limits>
-#include "GModel.h"
+#include "GmshConfig.h"
 #include "GmshMessage.h"
+#include "GModel.h"
 #include "OCCEdge.h"
 #include "OCCFace.h"
 #include "Context.h"
-extern Context_T CTX;
 
 #if defined(HAVE_OCC)
 #include "Geom2dLProp_CLProps2d.hxx"
@@ -30,23 +30,6 @@ OCCEdge::OCCEdge(GModel *model, TopoDS_Edge edge, int num, GVertex *v1, GVertex
   // build the reverse curve
   c_rev = c;
   c_rev.Reverse();
-
-
-//   if (v0 == v1){
-//     const int JJ = 52;
-//     for (int i=1;i<JJ;i++){
-//       const double t = i/((double) JJ);
-//       const double xi = s0 + (s1-s0) * t; 
-//       GPoint p = point(xi);
-//       MEdgeVertex *v = new MEdgeVertex (p.x(),p.y(),p.z(),this,xi);
-//       mesh_vertices.push_back(v);
-//       meshAttributes.Method = MESH_NONE;
-//       if (i == 1)lines.push_back(new MLine(v1->mesh_vertices[0],v));
-//       else if (i == JJ-1)lines.push_back(new MLine(v,v2->mesh_vertices[0]));
-//       else lines.push_back(new MLine(mesh_vertices[i-1],v));
-//     }
-//   }
-
 }
 
 Range<double> OCCEdge::parBounds(int i) const
@@ -54,7 +37,7 @@ Range<double> OCCEdge::parBounds(int i) const
   return Range<double>(s0, s1);
 }
 
-void OCCEdge::setTrimmed (OCCFace *f)
+void OCCEdge::setTrimmed(OCCFace *f)
 {
   if (!trimmed){
     trimmed = f;
@@ -64,7 +47,7 @@ void OCCEdge::setTrimmed (OCCFace *f)
   }
 }
 
-SPoint2 OCCEdge::reparamOnFace(GFace *face, double epar, int dir) const
+SPoint2 OCCEdge::reparamOnFace(const GFace *face, double epar, int dir) const
 {
   const TopoDS_Face *s = (TopoDS_Face*) face->getNativePtr();
   double t0, t1;
@@ -92,7 +75,7 @@ SPoint2 OCCEdge::reparamOnFace(GFace *face, double epar, int dir) const
   const double dx = p1.x()-p2.x();
   const double dy = p1.y()-p2.y();
   const double dz = p1.z()-p2.z();
-  if(sqrt(dx * dx + dy * dy + dz * dz) > 1.e-4 * CTX.lc){
+  if(sqrt(dx * dx + dy * dy + dz * dz) > 1.e-4 * CTX::instance()->lc){
     // return reparamOnFace(face, epar,-1);      
     Msg::Warning("Reparam on face partially failed for curve %d surface %d at point %g",
 		 tag(), face->tag(), epar);
@@ -107,7 +90,7 @@ SPoint2 OCCEdge::reparamOnFace(GFace *face, double epar, int dir) const
 }
 
 // True if the edge is a seam for the given face
-bool OCCEdge::isSeam(GFace *face) const
+bool OCCEdge::isSeam(const GFace *face) const
 {
   if (face->geomType() == GEntity::CompoundSurface)return false; 
   const TopoDS_Face *s = (TopoDS_Face*) face->getNativePtr();
@@ -128,7 +111,7 @@ GPoint OCCEdge::point(double par) const
     return trimmed->point(u, v);
   }
   else if(!curve.IsNull()){
-    gp_Pnt pnt = curve->Value (par);
+    gp_Pnt pnt = curve->Value(par);
     return GPoint(pnt.X(), pnt.Y(), pnt.Z());
   }
   else{
@@ -141,17 +124,11 @@ SVector3 OCCEdge::firstDer(double par) const
 {  
   BRepAdaptor_Curve brepc(c);
   BRepLProp_CLProps prop(brepc, 1, 1e-5);
-  prop.SetParameter (par);
+  prop.SetParameter(par);
   gp_Vec d1 = prop.D1();
   return SVector3(d1.X(), d1.Y(), d1.Z());
 }
 
-double OCCEdge::parFromPoint(const SPoint3 &pt) const
-{
-  Msg::Error("parFromPoint not implemented for OCCEdge");
-  return std::numeric_limits<double>::max();
-}
-
 GEntity::GeomType OCCEdge::geomType() const
 {
   if(curve.IsNull()){
@@ -208,7 +185,7 @@ int OCCEdge::minimumMeshSegments() const
   if(geomType() == Line)
     np = GEdge::minimumMeshSegments();
   else 
-    np = CTX.mesh.min_curv_points - 1;
+    np = CTX::instance()->mesh.minCurvPoints - 1;
   
   // if the edge is closed, ensure that at least 3 points are
   // generated in the 1D mesh (4 segments, one of which is
@@ -222,10 +199,8 @@ int OCCEdge::minimumDrawSegments() const
 {
   if(geomType() == Line)
     return GEdge::minimumDrawSegments();
-  else if(geomType() == Circle || geomType() == Ellipse)
-    return CTX.geom.circle_points;
   else
-    return 20 * GEdge::minimumDrawSegments();
+    return CTX::instance()->geom.numSubEdges * GEdge::minimumDrawSegments();
 }
 
 double OCCEdge::curvature(double par) const 
@@ -251,19 +226,32 @@ double OCCEdge::curvature(double par) const
       Crv = prop.Curvature();
   }
   if(Crv <= eps) Crv = eps;
-  
-  // std::list<GFace*> ff = faces();
-  // std::list<GFace *>::iterator it =  ff.begin();
-  // while (it != ff.end()){
-  //   SPoint2 par2 = reparamOnFace((*it),par,1);
-  //   const double cc = (*it)->curvature ( par2 );
-  //   if (cc > 0)
-  //     Crv = std::max( Crv, cc);  
-  //   ++it;
-  // }  
-  // printf("curvature = %12.5E\n",Crv); 
-
   return Crv;
 }
 
+void OCCEdge::writeGEO(FILE *fp)
+{
+  if(geomType() == Circle){
+    gp_Pnt center;
+    if(curve.IsNull()){
+      center = Handle(Geom_Circle)::DownCast(curve2d)->Location();
+    }
+    else{
+      center = Handle(Geom_Circle)::DownCast(curve)->Location();
+    }
+    // GEO supports only circle arcs < Pi
+    if(s1 - s0 < M_PI){
+      fprintf(fp, "p%d = newp;\n", tag());
+      fprintf(fp, "Point(p%d + 1) = {%.16g, %.16g, %.16g};\n", 
+              tag(), center.X(), center.Y(), center.Z());
+      fprintf(fp, "Circle(%d) = {%d, p%d + 1, %d};\n", 
+              tag(), getBeginVertex()->tag(), tag(), getEndVertex()->tag());
+    }
+    else
+      GEdge::writeGEO(fp);
+  }
+  else
+    GEdge::writeGEO(fp);
+}
+
 #endif
