@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -12,8 +12,6 @@
 #include "GmshMessage.h"
 #include "Context.h"
 
-extern Context_T CTX;
-
 gmshFace::gmshFace(GModel *m, Surface *face)
   : GFace(m, face->Num), s(face)
 {
@@ -37,19 +35,10 @@ gmshFace::gmshFace(GModel *m, Surface *face)
       Msg::Error("Unknown curve %d", c->Num);
   }
 
-  // always compute and store the mean plane for plane surfaces
-  // (simply using the bounding vertices)
-  if(s->Typ == MSH_SURF_PLAN){
-    computeMeanPlane();
-    for(int i = 0; i < 3; i++)
-      for(int j = 0; j < 3; j++)
-        s->plan[i][j] = meanPlane.plan[i][j];
-    s->a = meanPlane.a;
-    s->b = meanPlane.b;
-    s->c = meanPlane.c;
-    s->d = meanPlane.d;
-  }
-  
+  // always compute and store the mean plane for plane surfaces (using
+  // the bounding vertices)
+  if(s->Typ == MSH_SURF_PLAN) computeMeanPlane();
+
   if(s->EmbeddedCurves){
     for(int i = 0; i < List_Nbr(s->EmbeddedCurves); i++){
       Curve *c;
@@ -162,10 +151,18 @@ SVector3 gmshFace::normal(const SPoint2 &param) const
 
 Pair<SVector3,SVector3> gmshFace::firstDer(const SPoint2 &param) const
 {
-  Vertex vu = InterpolateSurface(s, param[0], param[1], 1, 1);
-  Vertex vv = InterpolateSurface(s, param[0], param[1], 1, 2);
-  return Pair<SVector3, SVector3>(SVector3(vu.Pos.X, vu.Pos.Y, vu.Pos.Z),
-                                  SVector3(vv.Pos.X, vv.Pos.Y, vv.Pos.Z));
+  if(s->Typ == MSH_SURF_PLAN && !s->geometry){
+    double x, y, z, VX[3], VY[3];
+    getMeanPlaneData(VX, VY, x, y, z);
+    return Pair<SVector3, SVector3>(SVector3(VX[0], VX[1], VX[2]),
+                                    SVector3(VY[0], VY[1], VY[2]));
+  }
+  else{
+    Vertex vu = InterpolateSurface(s, param[0], param[1], 1, 1);
+    Vertex vv = InterpolateSurface(s, param[0], param[1], 1, 2);
+    return Pair<SVector3, SVector3>(SVector3(vu.Pos.X, vu.Pos.Y, vu.Pos.Z),
+                                    SVector3(vv.Pos.X, vv.Pos.Y, vv.Pos.Z));
+  }
 }
 
 GPoint gmshFace::point(double par1, double par2) const
@@ -186,57 +183,39 @@ GPoint gmshFace::point(double par1, double par2) const
 
 GPoint gmshFace::closestPoint(const SPoint3 & qp, const double initialGuess[2]) const
 {
-  if (s->Typ ==  MSH_SURF_PLAN && !s->geometry){
+  if (s->Typ == MSH_SURF_PLAN && !s->geometry){
     double XP = qp.x();
     double YP = qp.y();
     double ZP = qp.z();
-    //double a = meanPlane.a;
-    //double b = meanPlane.b;
-    //double c = meanPlane.c;
-    //double d = meanPlane.d;
-    // X = P +   H N, find y such that
-    // a X_1 + b X_2 + c X_3 + d = 0
-    // a ( XP + y XN) + b ( YP + y YN) + c ( ZP + y ZN) + d = 0
-    // H ( a XN + b YN + c ZN ) = - (a XP + b YP + c ZP + d)    
-    //const double H = -(a*XP + b*YP + c *ZP + d)/(a*a + b*b + c*c);
-    //const double X = XP + H * a;
-    //const double Y = YP + H * b;
-    //const double Z = ZP + H * c;
-    // now compute parametric coordinates
-    double x,y,z,VX[3], VY[3];
+    double VX[3], VY[3], x, y, z;
     getMeanPlaneData(VX, VY, x, y, z);
-    // XP = X + u VX + v VY
-    // We are sure to be on the plane
-    double M[3][2] = {{VX[0],VY[0]},{VX[1],VY[1]},{VX[2],VY[2]}};
+    double M[3][2] = {{VX[0], VY[0]}, {VX[1], VY[1]}, {VX[2], VY[2]}};
     double MN[2][2];
-    double B[3] = {XP-x,YP-y,ZP-z};
-    double BN[2],UV[2];
-    for (int i=0;i<2;i++){
+    double B[3] = {XP - x, YP - y, ZP - z};
+    double BN[2], UV[2];
+    for (int i = 0; i < 2; i++){
       BN[i] = 0;
-      for (int k=0;k<3;k++){
+      for (int k = 0; k < 3; k++){
 	BN[i] += B[k] * M[k][i];
       }
     }
-    for (int i=0;i<2;i++){
-      for (int j=0;j<2;j++){
+    for (int i = 0; i < 2; i++){
+      for (int j = 0; j < 2; j++){
 	MN[i][j] = 0;
-	for (int k=0;k<3;k++){
+	for (int k = 0; k < 3; k++){
 	  MN[i][j] += M[k][i] * M[k][j];
 	}
       }
     }
-    sys2x2(MN,BN,UV);    
-
-    //    GPoint test = point (UV[0],UV[1]);
-    //    printf("%g %g %g vs %g %g %g\n",XP,YP,ZP,test.x(),test.y(),test.z());
+    sys2x2(MN, BN, UV);
     return GPoint(XP, YP, ZP, this, UV);    
   }
 
   Vertex v;
-  double u[2] = {initialGuess[0],initialGuess[1]};
   v.Pos.X = qp.x();
   v.Pos.Y = qp.y();
   v.Pos.Z = qp.z();
+  double u[2] = {initialGuess[0], initialGuess[1]};
   bool result = ProjectPointOnSurface(s, v, u);
   if (!result)
     return GPoint(-1.e22, -1.e22, -1.e22, 0, u);
