@@ -1,19 +1,17 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
 
 #include <string.h>
-#include "ReadImg.h"
-#include "GmshMessage.h"
-#include "GmshUI.h"
-#include "PView.h"
-#include "PViewDataList.h"
-  
 #include <FL/Fl_JPEG_Image.H>
 #include <FL/Fl_PNM_Image.H>
 #include <FL/Fl_PNG_Image.H>
 #include <FL/Fl_BMP_Image.H>
+#include "ReadImg.h"
+#include "GmshMessage.h"
+#include "PView.h"
+#include "PViewDataList.h"
 
 // from an image, we create a post-procession view
 
@@ -23,13 +21,13 @@ static PViewDataList *Img2Data(Fl_RGB_Image &img_init, int quads=1,
   img_init.desaturate(); // convert to grayscale
 
   // resize if necessary
-  Fl_RGB_Image * img;
+  Fl_RGB_Image *img;
   if(!resizex || !resizey)
-    img = (Fl_RGB_Image *) img_init.copy();
+    img = (Fl_RGB_Image*)img_init.copy();
   else
-    img = (Fl_RGB_Image *) img_init.copy(resizex, resizey);
+    img = (Fl_RGB_Image*)img_init.copy(resizex, resizey);
 
-  const uchar *data = img->array;
+  const unsigned char *data = img->array;
   int height = img->h();
   int width = img->w();
   int dim = img->d();
@@ -43,8 +41,8 @@ static PViewDataList *Img2Data(Fl_RGB_Image &img_init, int quads=1,
 
   double z = 0.;
   for(int i = 0; i < height - 1; i++) {
-    const uchar *a = data + i * width * dim;
-    const uchar *a1 = data + (i + 1) * width * dim;
+    const unsigned char *a = data + i * width * dim;
+    const unsigned char *a1 = data + (i + 1) * width * dim;
     double y = height - i - 1;
     double y1 = height - i - 2;
     for(int j = 0; j < width - 1; j++) {
@@ -110,26 +108,26 @@ static int EndPos(const char *name, PViewData *d)
   }
 }
 
-int read_pnm(const char *name) 
+int read_pnm(std::string fileName) 
 {
-  Fl_PNM_Image img(name);
-  return EndPos(name, Img2Data(img));
+  Fl_PNM_Image img(fileName.c_str());
+  return EndPos(fileName.c_str(), Img2Data(img));
 }
 
-int read_jpeg(const char *name) 
+int read_jpeg(std::string fileName) 
 {
-  Fl_JPEG_Image img(name);
-  return EndPos(name, Img2Data(img));
+  Fl_JPEG_Image img(fileName.c_str());
+  return EndPos(fileName.c_str(), Img2Data(img));
 }
 
-int read_png(const char *name) 
+int read_png(std::string fileName) 
 {
-  Fl_PNG_Image img(name);
-  return EndPos(name, Img2Data(img));
+  Fl_PNG_Image img(fileName.c_str());
+  return EndPos(fileName.c_str(), Img2Data(img));
 }
 
-int read_bmp(const char *name) 
+int read_bmp(std::string fileName) 
 {
-  Fl_BMP_Image img(name);
-  return EndPos(name, Img2Data(img));
+  Fl_BMP_Image img(fileName.c_str());
+  return EndPos(fileName.c_str(), Img2Data(img));
 }
