@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -13,11 +13,9 @@
 #include "GModel.h"
 #include "Field.h"
 
-extern Context_T CTX;
-
 // computes the characteristic length of the mesh at a vertex in order
 // to have the geometry captured with accuracy. A parameter called
-// CTX.mesh.min_circ_points tells the minimum number of points per
+// CTX::instance()->mesh.minCircPoints tells the minimum number of points per
 // radius of curvature
 
 static double max_edge_curvature(const GVertex *gv)
@@ -77,16 +75,16 @@ static double LC_MVertex_CURV(GEntity *ge, double U, double V)
   double Crv = 0;
   switch(ge->dim()){
   case 0:        
-    // Crv = max_edge_curvature((const GVertex *)ge);
-    // Crv = std::max(max_surf_curvature((const GVertex *)ge), Crv);
-    Crv = max_surf_curvature((const GVertex *)ge);
+    Crv = max_edge_curvature((const GVertex *)ge);
+    Crv = std::max(max_surf_curvature((const GVertex *)ge), Crv);
+    //Crv = max_surf_curvature((const GVertex *)ge);
     break;
   case 1:
     {
       GEdge *ged = (GEdge *)ge;
-      //Crv = ged->curvature(U);
-      //Crv = std::max(Crv, max_surf_curvature(ged, U));
-      Crv = max_surf_curvature(ged, U);      
+      Crv = ged->curvature(U);
+      Crv = std::max(Crv, max_surf_curvature(ged, U));
+      //Crv = max_surf_curvature(ged, U);      
     }
     break;
   case 2:
@@ -97,7 +95,7 @@ static double LC_MVertex_CURV(GEntity *ge, double U, double V)
     break;
   }
  
-  double lc = Crv > 0 ? 2 * M_PI / Crv / CTX.mesh.min_circ_points : MAX_LC;
+  double lc = Crv > 0 ? 2 * M_PI / Crv / CTX::instance()->mesh.minCircPoints : MAX_LC;
   return lc;
 }
 
@@ -111,7 +109,7 @@ static double LC_MVertex_PNTS(GEntity *ge, double U, double V)
       GVertex *gv = (GVertex *)ge;
       double lc = gv->prescribedMeshSizeAtVertex();
       // FIXME we might want to remove this to make all lc treatment consistent
-      if(lc >= MAX_LC) return CTX.lc / 10.;
+      if(lc >= MAX_LC) return CTX::instance()->lc / 10.;
       return lc;
     }
   case 1:
@@ -119,13 +117,17 @@ static double LC_MVertex_PNTS(GEntity *ge, double U, double V)
       GEdge *ged = (GEdge *)ge;
       GVertex *v1 = ged->getBeginVertex();
       GVertex *v2 = ged->getEndVertex();
-      Range<double> range = ged->parBounds(0);      
-      double a = (U - range.low()) / (range.high() - range.low()); 
-      double lc = (1 - a) * v1->prescribedMeshSizeAtVertex() +
-        (a) * v2->prescribedMeshSizeAtVertex() ;
-      // FIXME we might want to remove this to make all lc treatment consistent
-      if(lc >= MAX_LC) return CTX.lc / 10.;
-      return lc;
+      if (v1 && v2){
+	Range<double> range = ged->parBounds(0);      
+	double a = (U - range.low()) / (range.high() - range.low()); 
+	double lc = (1 - a) * v1->prescribedMeshSizeAtVertex() +
+	  (a) * v2->prescribedMeshSizeAtVertex() ;
+	// FIXME we might want to remove this to make all lc treatment consistent
+	if(lc >= MAX_LC) return CTX::instance()->lc / 10.;
+	return lc;
+      }
+      else 
+	return MAX_LC; 
     }
   default:
     return MAX_LC;
@@ -136,16 +138,16 @@ static double LC_MVertex_PNTS(GEntity *ge, double U, double V)
 double BGM_MeshSize(GEntity *ge, double U, double V, double X, double Y, double Z)
 {
   // default lc (mesh size == size of the model)
-  double l1 = CTX.lc;
+  double l1 = CTX::instance()->lc;
 
   // lc from points
   double l2 = MAX_LC;
-  if(CTX.mesh.lc_from_points && ge->dim() < 2) 
+  if(CTX::instance()->mesh.lcFromPoints && ge->dim() < 2) 
     l2 = LC_MVertex_PNTS(ge, U, V);
 
   // lc from curvature
   double l3 = MAX_LC;
-  if(CTX.mesh.lc_from_curvature && ge->dim() < 3)
+  if(CTX::instance()->mesh.lcFromCurvature && ge->dim() < 3)
     l3 = LC_MVertex_CURV(ge, U, V);
 
   // lc from fields
@@ -156,26 +158,26 @@ double BGM_MeshSize(GEntity *ge, double U, double V, double X, double Y, double
     if(f) l4 = (*f)(X, Y, Z, ge);
   }
 
-  // take the minimum, then contrain by lc_min and lc_max
+  // take the minimum, then contrain by lcMin and lcMax
   double lc = std::min(std::min(std::min(l1, l2), l3), l4);
-  lc = std::max(lc, CTX.mesh.lc_min);
-  lc = std::min(lc, CTX.mesh.lc_max);
+  lc = std::max(lc, CTX::instance()->mesh.lcMin);
+  lc = std::min(lc, CTX::instance()->mesh.lcMax);
 
   if(lc <= 0.){
     Msg::Error("Wrong characteristic length lc = %g (lcmin = %g, lcmax = %g)",
-	       lc, CTX.mesh.lc_min, CTX.mesh.lc_max);
+	       lc, CTX::instance()->mesh.lcMin, CTX::instance()->mesh.lcMax);
     lc = l1;
   }
 
-  return lc * CTX.mesh.lc_factor;
+  return lc * CTX::instance()->mesh.lcFactor;
 }
 
 bool Extend1dMeshIn2dSurfaces()
 {
-  return CTX.mesh.lc_extend_from_boundary ? true : false;
+  return CTX::instance()->mesh.lcExtendFromBoundary ? true : false;
 }
 
 bool Extend2dMeshIn3dVolumes()
 {
-  return CTX.mesh.lc_extend_from_boundary ? true : false;
+  return CTX::instance()->mesh.lcExtendFromBoundary ? true : false;
 }
