@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -9,6 +9,7 @@
 #include <string>
 #include <map>
 #include <list>
+#include "GmshConfig.h"
 
 #if !defined(HAVE_NO_POST)
 #include "PView.h"
@@ -35,11 +36,11 @@ class FieldOption {
  public:
   FieldOption(std::string help, bool *_status) : _help(help), status(_status) {}
   virtual ~FieldOption() {}
-  virtual FieldOptionType get_type() = 0;
-  virtual void get_text_representation(std::string & v_str) = 0;
-  virtual std::string get_description(){ return _help; }
-  std::string get_type_name(){
-    switch(get_type()){
+  virtual FieldOptionType getType() = 0;
+  virtual void getTextRepresentation(std::string & v_str) = 0;
+  virtual std::string getDescription(){ return _help; }
+  std::string getTypeName(){
+    switch(getType()){
     case FIELD_OPTION_INT: return "integer"; break;
     case FIELD_OPTION_DOUBLE: return "float"; break;
     case FIELD_OPTION_BOOL: return "boolean"; break;
@@ -49,8 +50,8 @@ class FieldOption {
     default: return "unknown";
     }
   }
-  virtual void numerical_value(double val) { throw(1); }
-  virtual double numerical_value() const { throw(1); }
+  virtual void numericalValue(double val) { throw(1); }
+  virtual double numericalValue() const { throw(1); }
   virtual const std::list<int> &list() const { throw(1); }
   virtual std::list<int> &list() { throw(1); }
   virtual const std::string &string() const { throw(1); }
@@ -65,12 +66,12 @@ class Field {
   virtual ~Field() {}
   bool update_needed;
   Field();
-  virtual const char *get_name() = 0;
+  virtual const char *getName() = 0;
 #if !defined(HAVE_NO_POST)
-  void put_on_view(PView * view, int comp = -1);
-  void put_on_new_view();
+  void putOnView(PView * view, int comp = -1);
+  void putOnNewView();
 #endif
-  virtual std::string get_description(){ return ""; }
+  virtual std::string getDescription(){ return ""; }
 };
 
 class FieldFactory {
@@ -84,14 +85,14 @@ class FieldManager : public std::map<int, Field*> {
   std::map<std::string, FieldFactory*> map_type_name;
   void reset();
   Field *get(int id);
-  Field *new_field(int id, std::string type_name);
-  void delete_field(int id);
-  int new_id();
-  int max_id();
+  Field *newField(int id, std::string type_name);
+  void deleteField(int id);
+  int newId();
+  int maxId();
   FieldManager();
   int background_field;
   // compatibility with -bgm
-  void set_background_mesh(int iView);
+  void setBackgroundMesh(int iView);
 };
 
 #endif
