@@ -1,9 +1,10 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
 
 #include <stdlib.h>
+#include "GmshConfig.h"
 #include "GmshMessage.h"
 #include "Numeric.h"
 #include "Context.h"
@@ -17,14 +18,13 @@
 #include "BackgroundMesh.h"
 #include "BoundaryLayers.h"
 #include "HighOrder.h"
+#include "Generator.h"
 
 #if !defined(HAVE_NO_POST)
 #include "PView.h"
 #include "PViewData.h"
 #endif
 
-extern Context_T CTX;
-
 static MVertex* isEquivalentTo(std::multimap<MVertex*, MVertex*> &m, MVertex *v)
 {
   std::multimap<MVertex*, MVertex*>::iterator it = m.lower_bound(v);
@@ -196,9 +196,9 @@ void GetStatistics(double stat[50], double quality[4][100])
     stat[12] += (*it)->pyramids.size();
   }
   
-  stat[13] = CTX.mesh_timer[0];
-  stat[14] = CTX.mesh_timer[1];
-  stat[15] = CTX.mesh_timer[2];
+  stat[13] = CTX::instance()->meshTimer[0];
+  stat[14] = CTX::instance()->meshTimer[1];
+  stat[15] = CTX::instance()->meshTimer[2];
   
   if(quality){
     for(int i = 0; i < 3; i++)
@@ -212,26 +212,26 @@ void GetStatistics(double stat[50], double quality[4][100])
       for(GModel::fiter it = m->firstFace(); it != m->lastFace(); ++it){
 	GetQualityMeasure((*it)->quadrangles, gamma, gammaMin, gammaMax,
 			  eta, etaMin, etaMax, rho, rhoMin, rhoMax, 
-			  disto, distoMin,distoMax,quality);
+			  disto, distoMin, distoMax, quality);
 	GetQualityMeasure((*it)->triangles, gamma, gammaMin, gammaMax,
 			  eta, etaMin, etaMax, rho, rhoMin, rhoMax,
-			  disto, distoMin,distoMax,quality);
+			  disto, distoMin, distoMax, quality);
       }
     }
     else{
       for(GModel::riter it = m->firstRegion(); it != m->lastRegion(); ++it){
 	GetQualityMeasure((*it)->tetrahedra, gamma, gammaMin, gammaMax,
 			  eta, etaMin, etaMax, rho, rhoMin, rhoMax, 
-			  disto, distoMin,distoMax,quality);
+			  disto, distoMin, distoMax, quality);
 	GetQualityMeasure((*it)->hexahedra, gamma, gammaMin, gammaMax,
 			  eta, etaMin, etaMax, rho, rhoMin, rhoMax,
-			  disto, distoMin,distoMax,quality);
+			  disto, distoMin, distoMax, quality);
 	GetQualityMeasure((*it)->prisms, gamma, gammaMin, gammaMax,
 			  eta, etaMin, etaMax, rho, rhoMin, rhoMax,
-			  disto, distoMin,distoMax,quality);
+			  disto, distoMin, distoMax,quality);
 	GetQualityMeasure((*it)->pyramids, gamma, gammaMin, gammaMax,
 			  eta, etaMin, etaMax, rho, rhoMin, rhoMax,
-			  disto, distoMin,distoMax,quality);
+			  disto, distoMin, distoMax, quality);
       }
     }
     double N = stat[9] + stat[10] + stat[11] + stat[12];
@@ -252,7 +252,7 @@ void GetStatistics(double stat[50], double quality[4][100])
 #if !defined(HAVE_NO_POST)
   stat[26] = PView::list.size();
   for(unsigned int i = 0; i < PView::list.size(); i++) {
-    PViewData *data = PView::list[i]->getData();
+    PViewData *data = PView::list[i]->getData(true);
     stat[27] += data->getNumPoints();
     stat[28] += data->getNumLines();
     stat[29] += data->getNumTriangles();
@@ -268,16 +268,16 @@ void GetStatistics(double stat[50], double quality[4][100])
 
 static bool TooManyElements(GModel *m, int dim)
 {
-  if(CTX.expert_mode || !m->getNumVertices()) return false;
+  if(CTX::instance()->expertMode || !m->getNumVertices()) return false;
 
   // try to detect obvious mistakes in characteristic lenghts (one of
   // the most common cause for erroneous bug reports on the mailing
   // list)
   double sumAllLc = 0.;
   for(GModel::viter it = m->firstVertex(); it != m->lastVertex(); ++it)
-    sumAllLc += (*it)->prescribedMeshSizeAtVertex() * CTX.mesh.lc_factor;
+    sumAllLc += (*it)->prescribedMeshSizeAtVertex() * CTX::instance()->mesh.lcFactor;
   sumAllLc /= (double)m->getNumVertices();
-  if(!sumAllLc || pow(CTX.lc / sumAllLc, dim) > 1.e10) 
+  if(!sumAllLc || pow(CTX::instance()->lc / sumAllLc, dim) > 1.e10) 
     return !Msg::GetBinaryAnswer
       ("Your choice of characteristic lengths will likely produce a very\n"
        "large mesh. Do you really want to continue?\n\n"
@@ -289,7 +289,7 @@ static bool TooManyElements(GModel *m, int dim)
 
 static bool CancelDelaunayHybrid(GModel *m)
 {
-  if(CTX.expert_mode) return false;
+  if(CTX::instance()->expertMode) return false;
   int n = 0;
   for(GModel::riter it = m->firstRegion(); it != m->lastRegion(); ++it)
     n += (*it)->getNumMeshElements();
@@ -314,18 +314,18 @@ static void Mesh1D(GModel *m)
   std::for_each(m->firstEdge(), m->lastEdge(), meshGEdge());
 
   double t2 = Cpu();
-  CTX.mesh_timer[0] = t2 - t1;
-  Msg::Info("Mesh 1D complete (%g s)", CTX.mesh_timer[0]);
+  CTX::instance()->meshTimer[0] = t2 - t1;
+  Msg::Info("Mesh 1D complete (%g s)", CTX::instance()->meshTimer[0]);
   Msg::StatusBar(1, false, "Mesh");
 }
 
 static void PrintMesh2dStatistics(GModel *m)
 {
   FILE *statreport = 0;
-  if(CTX.create_append_statreport == 1)
-    statreport = fopen(CTX.statreport, "w");
-  else if(CTX.create_append_statreport == 2)
-    statreport = fopen(CTX.statreport, "a");
+  if(CTX::instance()->createAppendMeshStatReport == 1)
+    statreport = fopen(CTX::instance()->meshStatReportFileName.c_str(), "w");
+  else if(CTX::instance()->createAppendMeshStatReport == 2)
+    statreport = fopen(CTX::instance()->meshStatReportFileName.c_str(), "a");
   else return;
 
   double worst = 1, best = 0, avg = 0;
@@ -350,37 +350,25 @@ static void PrintMesh2dStatistics(GModel *m)
     numFaces++;
   }
 
-  if(CTX.create_append_statreport == 1){
+  if(CTX::instance()->createAppendMeshStatReport == 1){
     fprintf(statreport, "2D stats\tname\t\t#faces\t\t#fail\t\t"
             "#t\t\tQavg\t\tQbest\t\tQworst\t\t#Q>90\t\t#Q>90/#t\t"
             "#e\t\ttau\t\t#Egood\t\t#Egood/#e\tCPU\n");
   }
 
-  fprintf(statreport,"\t%16s\t%d\t\t%d\t\t", CTX.base_filename, numFaces, nUnmeshed);
+  fprintf(statreport,"\t%16s\t%d\t\t%d\t\t", m->getName().c_str(), numFaces, nUnmeshed);
   fprintf(statreport,"%d\t\t%8.7f\t%8.7f\t%8.7f\t%d\t\t%8.7f\t",
           nTotT, avg / (double)nTotT, best, worst, nTotGoodQuality,
           (double)nTotGoodQuality / nTotT);
   fprintf(statreport,"%d\t\t%8.7f\t%d\t\t%8.7f\t%8.1f\n",
           nTotE, exp(e_avg / (double)nTotE), nTotGoodLength,
-          (double)nTotGoodLength / nTotE, CTX.mesh_timer[1]);
+          (double)nTotGoodLength / nTotE, CTX::instance()->meshTimer[1]);
   fclose(statreport);
 }
 
 static void Mesh2D(GModel *m)
 {
   if(TooManyElements(m, 2)) return;
-
-  if(!CTX.expert_mode && (CTX.mesh.algo2d == ALGO_2D_DELAUNAY ||
-			  CTX.mesh.algo2d == ALGO_2D_FRONTAL)){
-    if(!Msg::GetBinaryAnswer
-       ("The 2D Delaunay and Frontal algorithms are still experimental\n"
-	"and produce triangles with random orientations. Do you really\n"
-	"want to continue?\n\n"
-	"(To disable this warning in the future, select `Enable expert\n"
-	"mode' in the option dialog.)",
-	"Continue", "Cancel")) return;
-  }
-  
   Msg::StatusBar(1, true, "Meshing 2D...");
   double t1 = Cpu();
 
@@ -406,12 +394,12 @@ static void Mesh2D(GModel *m)
       if(nIter++ > 10) break;
     }
   }
-
-  gmshCollapseSmallEdges (*m);
+  
+  // gmshCollapseSmallEdges (*m);
 
   double t2 = Cpu();
-  CTX.mesh_timer[1] = t2 - t1;
-  Msg::Info("Mesh 2D complete (%g s)", CTX.mesh_timer[1]);
+  CTX::instance()->meshTimer[1] = t2 - t1;
+  Msg::Info("Mesh 2D complete (%g s)", CTX::instance()->meshTimer[1]);
   Msg::StatusBar(1, false, "Mesh");
 
   PrintMesh2dStatistics(m);
@@ -453,8 +441,8 @@ static void Mesh3D(GModel *m)
     MeshDelaunayVolume(connected[i]);
 
   double t2 = Cpu();
-  CTX.mesh_timer[2] = t2 - t1;
-  Msg::Info("Mesh 3D complete (%g s)", CTX.mesh_timer[2]);
+  CTX::instance()->meshTimer[2] = t2 - t1;
+  Msg::Info("Mesh 3D complete (%g s)", CTX::instance()->meshTimer[2]);
   Msg::StatusBar(1, false, "Mesh");
 }
 
@@ -487,12 +475,12 @@ void AdaptMesh(GModel *m)
   Msg::StatusBar(1, true, "Adapting 3D Mesh...");
   double t1 = Cpu();
 
-  if(CTX.threads_lock) {
+  if(CTX::instance()->lock) {
     Msg::Info("I'm busy! Ask me that later...");
     return;
   }
 
-  CTX.threads_lock = 1;
+  CTX::instance()->lock = 1;
 
   std::for_each(m->firstRegion(), m->lastRegion(), adaptMeshGRegion());
   std::for_each(m->firstRegion(), m->lastRegion(), adaptMeshGRegion());
@@ -512,11 +500,11 @@ void AdaptMesh(GModel *m)
 
 void GenerateMesh(GModel *m, int ask)
 {
-  if(CTX.threads_lock) {
+  if(CTX::instance()->lock) {
     Msg::Info("I'm busy! Ask me that later...");
     return;
   }
-  CTX.threads_lock = 1;
+  CTX::instance()->lock = 1;
 
   Msg::ResetErrorCounter();
 
@@ -552,22 +540,29 @@ void GenerateMesh(GModel *m, int ask)
 
   // Optimize quality of 3D tet mesh
   if(m->getMeshStatus() == 3){
-    for(int i = 0; i < std::max(CTX.mesh.optimize, CTX.mesh.optimize_netgen); i++){
-      if(CTX.mesh.optimize > i) OptimizeMesh(m);
-      if(CTX.mesh.optimize_netgen > i) OptimizeMeshNetgen(m);
+    for(int i = 0; i < std::max(CTX::instance()->mesh.optimize, 
+                                CTX::instance()->mesh.optimizeNetgen); i++){
+      if(CTX::instance()->mesh.optimize > i) OptimizeMesh(m);
+      if(CTX::instance()->mesh.optimizeNetgen > i) OptimizeMeshNetgen(m);
     }
   }
+
+  // Subdivide into quads or hexas
+  if(m->getMeshStatus() == 2 && CTX::instance()->mesh.algoSubdivide == 1) 
+    RefineMesh(m, CTX::instance()->mesh.secondOrderLinear, true);
+  else if(m->getMeshStatus() == 3 && CTX::instance()->mesh.algoSubdivide == 2) 
+    RefineMesh(m, CTX::instance()->mesh.secondOrderLinear, false, true);
   
   // Create high order elements
-  if(m->getMeshStatus() && CTX.mesh.order > 1) 
-    SetOrderN(m, CTX.mesh.order, CTX.mesh.second_order_linear, 
-              CTX.mesh.second_order_incomplete);
+  if(m->getMeshStatus() && CTX::instance()->mesh.order > 1) 
+    SetOrderN(m, CTX::instance()->mesh.order, CTX::instance()->mesh.secondOrderLinear, 
+              CTX::instance()->mesh.secondOrderIncomplete);
 
   Msg::Info("%d vertices %d elements",
 	    m->getNumMeshVertices(), m->getNumMeshElements());
 
   Msg::PrintErrorCounter("Mesh generation error summary");
 
-  CTX.threads_lock = 0;
-  CTX.mesh.changed = ENT_ALL;
+  CTX::instance()->lock = 0;
+  CTX::instance()->mesh.changed = ENT_ALL;
 }
