@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -19,8 +19,6 @@
 
 #define SQU(a)      ((a)*(a))
 
-extern Context_T CTX;
-
 static bool mappingIsInvertible(MTetrahedron *e)
 {
   if (e->getPolynomialOrder() == 1) return 1.0;
@@ -42,7 +40,7 @@ static bool mappingIsInvertible(MTetrahedron *e)
     if (det0 * detN <= 0.) return false;
   }
 
-  const Double_Matrix &points = e->getFunctionSpace()->points;
+  const gmshMatrix<double> &points = e->getFunctionSpace()->points;
 
   for (int i = 0; i < e->getNumPrimaryVertices(); i++) {
     const double u = points(i,0);
@@ -64,7 +62,7 @@ static double mylength(GEdge *ge, int i, double *u)
   return ge->length(u[i], u[i+1], 10);
 }
 
-static void myresid(int N, GEdge *ge, double *u, Double_Vector &r)
+static void myresid(int N, GEdge *ge, double *u, gmshVector<double> &r)
 {
   double L[100];
   for (int i = 0; i < N - 1; i++) L[i] = mylength(ge, i, u);
@@ -91,10 +89,10 @@ static bool computeEquidistantParameters(GEdge *ge, double u0, double uN, int N,
 
   // create the tangent matrix
   const int M = N - 2;
-  Double_Matrix J(M, M);
-  Double_Vector DU(M);
-  Double_Vector R(M);
-  Double_Vector Rp(M);
+  gmshMatrix<double> J(M, M);
+  gmshVector<double> DU(M);
+  gmshVector<double> R(M);
+  gmshVector<double> Rp(M);
   
   int iter = 1;
 
@@ -136,7 +134,7 @@ static double mylength(GFace *gf, int i, double *u, double *v)
   return gf->length(SPoint2(u[i], v[i]), SPoint2(u[i + 1], v[i + 1]), 10);
 }
 
-static void myresid(int N, GFace *gf, double *u, double *v, Double_Vector &r)
+static void myresid(int N, GFace *gf, double *u, double *v, gmshVector<double> &r)
 {
   double L[100];
   for (int i = 0; i < N - 1; i++) L[i] = mylength(gf, i, u, v);  
@@ -170,10 +168,10 @@ static bool computeEquidistantParameters(GFace *gf, double u0, double uN,
 
   // create the tangent matrix
   const int M = N - 2;
-  Double_Matrix J(M, M);
-  Double_Vector DU(M);
-  Double_Vector R(M);
-  Double_Vector Rp(M);
+  gmshMatrix<double> J(M, M);
+  gmshVector<double> DU(M);
+  gmshVector<double> R(M);
+  gmshVector<double> Rp(M);
   
   int iter = 1;
 
@@ -226,6 +224,10 @@ static void getEdgeVertices(GEdge *ge, MElement *ele, std::vector<MVertex*> &ve,
                             int nPts = 1, gmshHighOrderSmoother *displ2D = 0,
                             gmshHighOrderSmoother *displ3D = 0)
 {
+  if(ge->geomType() == GEntity::DiscreteCurve ||
+     ge->geomType() == GEntity::BoundaryLayerCurve)
+    linear = true;
+
   for(int i = 0; i < ele->getNumEdges(); i++){
     MEdge edge = ele->getEdge(i);
     std::pair<MVertex*, MVertex*> p(edge.getMinVertex(), edge.getMaxVertex());
@@ -235,58 +237,63 @@ static void getEdgeVertices(GEdge *ge, MElement *ele, std::vector<MVertex*> &ve,
       else
         ve.insert(ve.end(), edgeVertices[p].rbegin(), edgeVertices[p].rend());
     }
-    else{
-      MVertex *v0 = edge.getVertex(0), *v1 = edge.getVertex(1);            
-      double u0 = 0., u1 = 0.;
+    
+    else{  
+      
+      MVertex *v0 = edge.getVertex(0), *v1 = edge.getVertex(1);
+      std::vector<MVertex*> temp;
+        
+      double u0 = 0., u1 = 0., US[100];
       bool reparamOK = true;
-      if(!linear && ge->geomType() != GEntity::DiscreteCurve &&
-         ge->geomType() != GEntity::BoundaryLayerCurve){
+      if(!linear) {
         reparamOK &= reparamMeshVertexOnEdge(v0, ge, u0);
-        if (ge->periodic(0) && v1 == ge->getEndVertex()->mesh_vertices[0]){
-          Range<double> par = ge->parBounds(0);
-          u1 = par.high();
-        }         
+        if(ge->periodic(0) && v1 == ge->getEndVertex()->mesh_vertices[0])
+          u1 = ge->parBounds(0).high();
         else
           reparamOK &= reparamMeshVertexOnEdge(v1, ge, u1);
+        if(reparamOK){
+          double relax = 1.;
+          while (1){
+            if(computeEquidistantParameters(ge, u0, u1, nPts + 2, US, relax)) 
+                break;
+            relax /= 2.0;
+            if(relax < 1.e-2) 
+              break;
+          } 
+          if(relax < 1.e-2)
+            Msg::Warning
+              ("Failed to compute equidistant parameters (relax = %g) for edge %d-%d",
+               relax, v0->getNum(), v1->getNum());
+        }
       }
-      double US[100];
-      if(reparamOK && !linear && ge->geomType() != GEntity::DiscreteCurve){
-        double relax = 1.;
-        while (1){
-          if(computeEquidistantParameters(ge, u0, u1, nPts + 2, US, relax)) 
-            break;
-          relax /= 2.0;
-          if (relax < 1.e-2) 
-            break;
-        } 
-        if (relax < 1.e-2)
-          Msg::Warning("Failed to compute equidistant parameters (relax = %g)",
-                       relax);
-      }
-      std::vector<MVertex*> temp;      
       for(int j = 0; j < nPts; j++){
         const double t = (double)(j + 1)/(nPts + 1);
-        double uc = (1. - t) * u0 + t * u1;
+        
+        double uc = (1. - t) * u0 + t * u1; // can be wrong, that's ok
         MVertex *v;
-        if(!reparamOK || linear || ge->geomType() == GEntity::DiscreteCurve || 
-           uc < u0 || uc > u1){ // need to treat periodic curves properly!
+        if(linear || !reparamOK || uc < u0 || uc > u1){ 
+          Msg::Warning("We don't have a valid parameter on curve %d-%d",
+             v0->getNum(), v1->getNum());
+          // we don't have a (valid) parameter on the curve
           SPoint3 pc = edge.interpolate(t);
           v = new MVertex(pc.x(), pc.y(), pc.z(), ge);
-          v->setParameter(0, t);
         }
-        else {
+        else {          
           GPoint pc = ge->point(US[j + 1]);
-	  v = new MEdgeVertex(pc.x(), pc.y(), pc.z(), ge, US[j + 1]);
-	  if (displ2D){
-	    SPoint3 pc2 = edge.interpolate(t);          
-	    displ2D->add(v, SVector3(pc2.x(), pc2.y(), pc2.z()));
-	    displ3D->add(v, SVector3(pc2.x(), pc2.y(), pc2.z()));
-	  }
+          v = new MEdgeVertex(pc.x(), pc.y(), pc.z(), ge, US[j + 1]);
+            
+          if (displ2D){
+            SPoint3 pc2 = edge.interpolate(t);          
+            displ2D->add(v, SVector3(pc2.x(), pc2.y(), pc2.z()));
+            displ3D->add(v, SVector3(pc2.x(), pc2.y(), pc2.z()));
+          }
         }
         temp.push_back(v);
+        // this destroys the ordering of the mesh vertices on the edge
         ge->mesh_vertices.push_back(v);
         ve.push_back(v);
       }
+    
       if(edge.getVertex(0) == edge.getMinVertex())
         edgeVertices[p].insert(edgeVertices[p].end(), temp.begin(), temp.end());
       else
@@ -300,6 +307,10 @@ static void getEdgeVertices(GFace *gf, MElement *ele, std::vector<MVertex*> &ve,
                             int nPts = 1, gmshHighOrderSmoother *displ2D = 0,
                             gmshHighOrderSmoother *displ3D = 0)
 {
+  if(gf->geomType() == GEntity::DiscreteSurface ||
+     gf->geomType() == GEntity::BoundaryLayerSurface)
+    linear = true;
+
   for(int i = 0; i < ele->getNumEdges(); i++){
     MEdge edge = ele->getEdge(i);    
     std::pair<MVertex*, MVertex*> p(edge.getMinVertex(), edge.getMaxVertex());
@@ -312,27 +323,26 @@ static void getEdgeVertices(GFace *gf, MElement *ele, std::vector<MVertex*> &ve,
     else{
       MVertex *v0 = edge.getVertex(0), *v1 = edge.getVertex(1);
       SPoint2 p0, p1;
-      bool reparamOK = true;
-      if(!linear && 
-         gf->geomType() != GEntity::DiscreteSurface &&
-         gf->geomType() != GEntity::BoundaryLayerSurface){
-        reparamOK = reparamMeshEdgeOnFace(v0, v1, gf, p0, p1);
-      }
       double US[100], VS[100];
-      if(reparamOK && !linear && gf->geomType() != GEntity::DiscreteSurface){
-        computeEquidistantParameters(gf, p0[0], p1[0], p0[1], p1[1], nPts + 2, US, VS);
+      bool reparamOK = true;
+      if(!linear){
+        reparamOK = reparamMeshEdgeOnFace(v0, v1, gf, p0, p1);
+        if(reparamOK)
+          computeEquidistantParameters(gf, p0[0], p1[0], p0[1], p1[1], nPts + 2,
+                                       US, VS);
       }
       std::vector<MVertex*> temp;
       for(int j = 0; j < nPts; j++){
         const double t = (double)(j + 1) / (nPts + 1);
         MVertex *v;
-        if(!reparamOK || linear || gf->geomType() == GEntity::DiscreteSurface){
+        if(linear || !reparamOK){
+          // we don't have (valid) parameters on the surface
           SPoint3 pc = edge.interpolate(t);
           v = new MVertex(pc.x(), pc.y(), pc.z(), gf);
         }
         else{
-          GPoint pc = gf->point(US[j+1], VS[j+1]);
-	  v = new MFaceVertex(pc.x(), pc.y(), pc.z(), gf, US[j+1], VS[j+1]);
+          GPoint pc = gf->point(US[j + 1], VS[j + 1]);
+	  v = new MFaceVertex(pc.x(), pc.y(), pc.z(), gf, US[j + 1], VS[j + 1]);
 	  if (displ3D){
 	    SPoint3 pc2 = edge.interpolate(t);          
 	    displ3D->add(v, SVector3(pc2.x(), pc2.y(), pc2.z()));
@@ -351,8 +361,8 @@ static void getEdgeVertices(GFace *gf, MElement *ele, std::vector<MVertex*> &ve,
 }
 
 static void getEdgeVertices(GRegion *gr, MElement *ele, std::vector<MVertex*> &ve,
-                            std::set<MVertex*> &blocked, edgeContainer &edgeVertices, 
-                            bool linear, int nPts = 1, gmshHighOrderSmoother *displ2D = 0,
+                            edgeContainer &edgeVertices, bool linear,
+                            int nPts = 1, gmshHighOrderSmoother *displ2D = 0,
                             gmshHighOrderSmoother *displ3D = 0)
 {
   for(int i = 0; i < ele->getNumEdges(); i++){
@@ -363,9 +373,6 @@ static void getEdgeVertices(GRegion *gr, MElement *ele, std::vector<MVertex*> &v
         ve.insert(ve.end(), edgeVertices[p].begin(), edgeVertices[p].end());
       else
         ve.insert(ve.end(), edgeVertices[p].rbegin(), edgeVertices[p].rend());
-      blocked.insert(edgeVertices[p].begin(), edgeVertices[p].end());
-      blocked.insert(edge.getMinVertex());
-      blocked.insert(edge.getMaxVertex());
     }
     else{
       std::vector<MVertex*> temp;
@@ -390,7 +397,11 @@ static void getFaceVertices(GFace *gf, MElement *incomplete, MElement *ele,
                             bool linear, int nPts = 1, gmshHighOrderSmoother *displ2D = 0,
                             gmshHighOrderSmoother *displ3D = 0)
 {
-  Double_Matrix points;
+  if(gf->geomType() == GEntity::DiscreteSurface ||
+     gf->geomType() == GEntity::BoundaryLayerSurface)
+    linear = true;
+
+  gmshMatrix<double> points;
   int start = 0;
 
   switch (nPts){
@@ -421,19 +432,16 @@ static void getFaceVertices(GFace *gf, MElement *incomplete, MElement *ele,
       std::vector<MVertex*> &vtcs = faceVertices[face];
       SPoint2 pts[20];
       bool reparamOK = true;
-      if(!linear && 
-         gf->geomType() != GEntity::DiscreteSurface &&
-         gf->geomType() != GEntity::BoundaryLayerSurface){
-	for (int k = 0; k < incomplete->getNumVertices(); k++){
+      if(!linear){
+	for(int k = 0; k < incomplete->getNumVertices(); k++)
 	  reparamOK &= reparamMeshVertexOnFace(incomplete->getVertex(k), gf, pts[k]);
-	}
       }
       if(face.getNumVertices() == 3 && nPts > 1){ // tri face
         for(int k = start; k < points.size1(); k++){
           MVertex *v;
           const double t1 = points(k, 0);
           const double t2 = points(k, 1);
-          if(linear || gf->geomType() == GEntity::DiscreteSurface){
+          if(linear){
             SPoint3 pc = face.interpolate(t1, t2);
             v = new MVertex(pc.x(), pc.y(), pc.z(), gf);
           }
@@ -463,7 +471,7 @@ static void getFaceVertices(GFace *gf, MElement *incomplete, MElement *ele,
 	    else{
 	      v = new MVertex(X, Y, Z, gf);
 	    }
-	    if (displ3D){
+	    if(displ3D){
 	      SPoint3 pc2 = face.interpolate(t1, t2);
 	      displ3D->add(v, SVector3(pc2.x(), pc2.y(), pc2.z()));
 	    }	    
@@ -481,7 +489,7 @@ static void getFaceVertices(GFace *gf, MElement *incomplete, MElement *ele,
             // parameters are between -1 and 1
             double t1 = 2. * (double)(j + 1) / (nPts + 1) - 1.;
             double t2 = 2. * (double)(k + 1) / (nPts + 1) - 1.;
-            if(!reparamOK || linear || gf->geomType() == GEntity::DiscreteSurface){
+            if(linear || !reparamOK){
               SPoint3 pc = face.interpolate(t1, t2);
               v = new MVertex(pc.x(), pc.y(), pc.z(), gf);
             }
@@ -539,10 +547,10 @@ static void reorientTrianglePoints(std::vector<MVertex*> &vtcs, int orientation,
 // KH: check face orientation wrt element ... 
 
 static void getFaceVertices(GRegion *gr, MElement *ele, std::vector<MVertex*> &vf,
-                            std::set<MVertex*> &blocked, faceContainer &faceVertices,
-                            edgeContainer &edgeVertices, bool linear, int nPts = 1)
+                            faceContainer &faceVertices, edgeContainer &edgeVertices,
+                            bool linear, int nPts = 1)
 {
-  Double_Matrix points;
+  gmshMatrix<double> points;
   int start = 0;
   
   switch (nPts){
@@ -575,10 +583,6 @@ static void getFaceVertices(GRegion *gr, MElement *ele, std::vector<MVertex*> &v
           reorientTrianglePoints(vtcs, orientation, swap);
         else
           Msg::Error("Error in face lookup for recuperation of high order face nodes");
-        blocked.insert(vtcs.begin(), vtcs.end());
-        blocked.insert(face.getVertex(0));
-        blocked.insert(face.getVertex(1));
-        blocked.insert(face.getVertex(2));
       }
       else if(face.getNumVertices() == 4){ // quad face
         // TODO reorient if more than 1 face vertex
@@ -639,7 +643,7 @@ static void getFaceVertices(GRegion *gr, MElement *ele, std::vector<MVertex*> &v
 static void getRegionVertices(GRegion *gr, MElement *incomplete, MElement *ele, 
                               std::vector<MVertex*> &vr, bool linear, int nPts = 1)
 {
-  Double_Matrix points;
+  gmshMatrix<double> points;
   int start = 0;
 
   switch (nPts){
@@ -697,7 +701,7 @@ static void setHighOrder(GFace *gf, edgeContainer &edgeVertices,
   for(unsigned int i = 0; i < gf->triangles.size(); i++){
     MTriangle *t = gf->triangles[i];
     std::vector<MVertex*> ve, vf;
-    getEdgeVertices(gf, t, ve, edgeVertices, linear, nPts,displ2D,displ3D);
+    getEdgeVertices(gf, t, ve, edgeVertices, linear, nPts, displ2D, displ3D);
     if(nPts == 1){
       triangles2.push_back
         (new MTriangle6(t->getVertex(0), t->getVertex(1), t->getVertex(2),
@@ -727,7 +731,7 @@ static void setHighOrder(GFace *gf, edgeContainer &edgeVertices,
   for(unsigned int i = 0; i < gf->quadrangles.size(); i++){
     MQuadrangle *q = gf->quadrangles[i];
     std::vector<MVertex*> ve, vf;
-    getEdgeVertices(gf, q, ve, edgeVertices, linear, nPts,displ2D,displ3D);
+    getEdgeVertices(gf, q, ve, edgeVertices, linear, nPts, displ2D, displ3D);
     if(incomplete){
       quadrangles2.push_back
         (new MQuadrangle8(q->getVertex(0), q->getVertex(1), q->getVertex(2),
@@ -755,16 +759,15 @@ static void setHighOrder(GRegion *gr, edgeContainer &edgeVertices,
   std::vector<MTetrahedron*> tetrahedra2;
   for(unsigned int i = 0; i < gr->tetrahedra.size(); i++){
     MTetrahedron *t = gr->tetrahedra[i];
-    std::set<MVertex*> blocked;
     std::vector<MVertex*> ve, vf, vr;
-    getEdgeVertices(gr, t, ve, blocked, edgeVertices, linear, nPts, displ2D, displ3D);
+    getEdgeVertices(gr, t, ve, edgeVertices, linear, nPts, displ2D, displ3D);
     if(nPts == 1){
       tetrahedra2.push_back
 	(new MTetrahedron10(t->getVertex(0), t->getVertex(1), t->getVertex(2), 
 			    t->getVertex(3), ve[0], ve[1], ve[2], ve[3], ve[4], ve[5]));
     }
     else{
-      getFaceVertices(gr, t, vf, blocked, faceVertices, edgeVertices, linear, nPts);
+      getFaceVertices(gr, t, vf, faceVertices, edgeVertices, linear, nPts);
       ve.insert(ve.end(), vf.begin(), vf.end());     
       MTetrahedronN incpl(t->getVertex(0), t->getVertex(1), t->getVertex(2), t->getVertex(3),
                           ve, nPts + 1);
@@ -796,8 +799,7 @@ static void setHighOrder(GRegion *gr, edgeContainer &edgeVertices,
   for(unsigned int i = 0; i < gr->hexahedra.size(); i++){
     MHexahedron *h = gr->hexahedra[i];
     std::vector<MVertex*> ve, vf;
-    std::set<MVertex*> blocked;
-    getEdgeVertices(gr, h, ve, blocked, edgeVertices, linear, nPts,displ2D, displ3D);
+    getEdgeVertices(gr, h, ve, edgeVertices, linear, nPts, displ2D, displ3D);
     if(incomplete){
       hexahedra2.push_back
         (new MHexahedron20(h->getVertex(0), h->getVertex(1), h->getVertex(2), 
@@ -807,7 +809,7 @@ static void setHighOrder(GRegion *gr, edgeContainer &edgeVertices,
                            ve[11]));
     }
     else{
-      getFaceVertices(gr, h, vf, blocked, faceVertices, edgeVertices, linear, nPts);
+      getFaceVertices(gr, h, vf, faceVertices, edgeVertices, linear, nPts);
       SPoint3 pc = h->barycenter();
       MVertex *v = new MVertex(pc.x(), pc.y(), pc.z(), gr);
       gr->mesh_vertices.push_back(v);
@@ -826,8 +828,7 @@ static void setHighOrder(GRegion *gr, edgeContainer &edgeVertices,
   for(unsigned int i = 0; i < gr->prisms.size(); i++){
     MPrism *p = gr->prisms[i];
     std::vector<MVertex*> ve, vf;
-    std::set<MVertex*> blocked;
-    getEdgeVertices(gr, p, ve, blocked, edgeVertices, linear, nPts,displ2D, displ3D);
+    getEdgeVertices(gr, p, ve, edgeVertices, linear, nPts, displ2D, displ3D);
     if(incomplete){
       prisms2.push_back
         (new MPrism15(p->getVertex(0), p->getVertex(1), p->getVertex(2), 
@@ -835,7 +836,7 @@ static void setHighOrder(GRegion *gr, edgeContainer &edgeVertices,
                       ve[0], ve[1], ve[2], ve[3], ve[4], ve[5], ve[6], ve[7], ve[8]));
     }
     else{
-      getFaceVertices(gr, p, vf, blocked, faceVertices, edgeVertices, linear, nPts);
+      getFaceVertices(gr, p, vf, faceVertices, edgeVertices, linear, nPts);
       prisms2.push_back
         (new MPrism18(p->getVertex(0), p->getVertex(1), p->getVertex(2), 
                       p->getVertex(3), p->getVertex(4), p->getVertex(5), 
@@ -850,8 +851,7 @@ static void setHighOrder(GRegion *gr, edgeContainer &edgeVertices,
   for(unsigned int i = 0; i < gr->pyramids.size(); i++){
     MPyramid *p = gr->pyramids[i];
     std::vector<MVertex*> ve, vf;
-    std::set<MVertex*> blocked;
-    getEdgeVertices(gr, p, ve, blocked, edgeVertices, linear, nPts,displ2D, displ3D);
+    getEdgeVertices(gr, p, ve, edgeVertices, linear, nPts, displ2D, displ3D);
     if(incomplete){
       pyramids2.push_back
         (new MPyramid13(p->getVertex(0), p->getVertex(1), p->getVertex(2), 
@@ -859,7 +859,7 @@ static void setHighOrder(GRegion *gr, edgeContainer &edgeVertices,
                         ve[3], ve[4], ve[5], ve[6], ve[7]));
     }
     else{
-      getFaceVertices(gr, p, vf, blocked, faceVertices, edgeVertices, linear, nPts);
+      getFaceVertices(gr, p, vf, faceVertices, edgeVertices, linear, nPts);
       pyramids2.push_back
         (new MPyramid14(p->getVertex(0), p->getVertex(1), p->getVertex(2), 
                         p->getVertex(3), p->getVertex(4), ve[0], ve[1], ve[2], 
@@ -1010,11 +1010,6 @@ void SetOrderN(GModel *m, int order, bool linear, bool incomplete)
   //   edges (creating 8-node quads, 20-node hexas, etc., instead of
   //   9-node quads, 27-node hexas, etc.)
 
-#if !defined(HAVE_GSL)
-  Msg::Error("High order mesh generation requires the GSL");
-  return;
-#endif
-
   int nPts = order - 1;
 
   Msg::StatusBar(1, true, "Meshing order %d...", order);
@@ -1025,7 +1020,7 @@ void SetOrderN(GModel *m, int order, bool linear, bool incomplete)
 
   gmshHighOrderSmoother *displ2D = 0; 
   gmshHighOrderSmoother *displ3D = 0; 
-  if(CTX.mesh.smooth_internal_edges){
+  if(CTX::instance()->mesh.smoothInternalEdges){
     displ2D = new gmshHighOrderSmoother(2);
     displ3D = new gmshHighOrderSmoother(3);
   }
@@ -1033,12 +1028,18 @@ void SetOrderN(GModel *m, int order, bool linear, bool incomplete)
   // then create new second order vertices/elements
   edgeContainer edgeVertices;
   faceContainer faceVertices;
+  
+  Msg::StatusBar(1, true, "Meshing edges order %d...", order);
   for(GModel::eiter it = m->firstEdge(); it != m->lastEdge(); ++it)
     setHighOrder(*it, edgeVertices, linear, nPts, displ2D, displ3D);
+
+  Msg::StatusBar(1, true, "Meshing faces %d...", order);
   for(GModel::fiter it = m->firstFace(); it != m->lastFace(); ++it)
     setHighOrder(*it, edgeVertices, faceVertices, linear, incomplete, nPts,
                  displ2D, displ3D);
 
+  Msg::StatusBar(1, true, "Done meshing order %d", order);
+  
   // now we smooth mesh the internal vertices of the faces
   // we do that model face by model face
   std::vector<MElement*> bad;
@@ -1046,7 +1047,10 @@ void SetOrderN(GModel *m, int order, bool linear, bool incomplete)
   if (displ2D){
     checkHighOrderTriangles(m, bad, worst);
     for(GModel::fiter it = m->firstFace(); it != m->lastFace(); ++it)
-      if ((*it)->geomType() == GEntity::Plane) displ2D->smooth(*it);
+      if ((*it)->geomType() == GEntity::Plane) {
+	gmshSwapHighOrderTriangles(*it);
+	if (displ2D) displ2D->smooth(*it);
+      }
     // will have to smooth in the planar coordinates, using the metric
   }
 
