@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -7,7 +7,7 @@
 #include "GModel.h"
 #include "MElement.h"
 #include "ExtrudeParams.h"
-#include "Message.h"
+#include "GmshMessage.h"
 
 static void extrudeMesh(GVertex *from, GEdge *to)
 {
