@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -27,7 +27,58 @@
 #include "OS.h"
 #include "HighOrder.h"
 
-extern Context_T CTX;
+void computeEdgeLoops(const GFace *gf,
+		      std::vector<MVertex*> &all_mvertices,
+		      std::vector<int> &indices)
+{
+  std::list<GEdge*> edges = gf->edges();
+  std::list<int> ori = gf->orientations();
+  std::list<GEdge*>::iterator it = edges.begin();
+  std::list<int>::iterator ito = ori.begin();
+
+  indices.push_back(0);
+  GVertex *start = ((*ito) == 1) ? (*it)->getBeginVertex() : (*it)->getEndVertex();
+  GVertex *v_end = ((*ito) != 1) ? (*it)->getBeginVertex() : (*it)->getEndVertex();
+  all_mvertices.push_back(start->mesh_vertices[0]);
+  if (*ito == 1)
+    for (unsigned int i = 0; i < (*it)->mesh_vertices.size(); i++)
+      all_mvertices.push_back((*it)->mesh_vertices[i]);
+  else
+    for (int i = (*it)->mesh_vertices.size() - 1; i >= 0; i--)
+      all_mvertices.push_back((*it)->mesh_vertices[i]);
+  
+  GVertex *v_start = start;
+  while(1){
+    ++it;
+    ++ito;
+    if(v_end == start){
+      indices.push_back(all_mvertices.size());
+      if(it == edges.end ()) break;
+      start = ((*ito) == 1) ? (*it)->getBeginVertex() : (*it)->getEndVertex();
+      v_end = ((*ito) != 1) ? (*it)->getBeginVertex() : (*it)->getEndVertex();
+      v_start = start;
+    }
+    else{
+      if(it == edges.end ()){
+	Msg::Error("Something wrong in edge loop computation");
+	return;
+      }
+      v_start = ((*ito) == 1) ? (*it)->getBeginVertex() : (*it)->getEndVertex();
+      if(v_start != v_end){
+	Msg::Error("Something wrong in edge loop computation");
+	return;
+      }
+      v_end = ((*ito) != 1) ? (*it)->getBeginVertex() : (*it)->getEndVertex();
+    }
+    all_mvertices.push_back(v_start->mesh_vertices[0]);
+    if(*ito == 1)
+      for(unsigned int i = 0; i < (*it)->mesh_vertices.size(); i++)
+        all_mvertices.push_back((*it)->mesh_vertices[i]);
+    else
+      for (int i = (*it)->mesh_vertices.size()-1; i >= 0; i--)
+        all_mvertices.push_back((*it)->mesh_vertices[i]);
+  }
+}
 
 void fourthPoint(double *p1, double *p2, double *p3, double *p4)
 {
@@ -143,65 +194,12 @@ static void remeshUnrecoveredEdges(std::set<EdgeToRecover> &edgesNotRecovered,
 
 static bool AlgoDelaunay2D(GFace *gf)
 {
-  if(noseam(gf) && (CTX.mesh.algo2d == ALGO_2D_DELAUNAY || 
-		    CTX.mesh.algo2d == ALGO_2D_FRONTAL))
+  if(noseam(gf) && (CTX::instance()->mesh.algo2d == ALGO_2D_DELAUNAY || 
+		    CTX::instance()->mesh.algo2d == ALGO_2D_FRONTAL))
     return true;
   return false;
 }
 
-void computeEdgeLoops(const GFace *gf,
-		      std::vector<MVertex*> &all_mvertices,
-		      std::vector<int> &indices)
-{
-  std::list<GEdge*> edges = gf->edges();
-  std::list<int> ori = gf->orientations();
-  std::list<GEdge*>::iterator it = edges.begin();
-  std::list<int>::iterator ito = ori.begin();
-
-  indices.push_back(0);
-  GVertex *start = ((*ito) == 1) ? (*it)->getBeginVertex() : (*it)->getEndVertex();
-  GVertex *v_end = ((*ito) != 1) ? (*it)->getBeginVertex() : (*it)->getEndVertex();
-  all_mvertices.push_back(start->mesh_vertices[0]);
-  if (*ito == 1)
-    for (unsigned int i = 0; i < (*it)->mesh_vertices.size(); i++)
-      all_mvertices.push_back((*it)->mesh_vertices[i]);
-  else
-    for (int i = (*it)->mesh_vertices.size() - 1; i >= 0; i--)
-      all_mvertices.push_back((*it)->mesh_vertices[i]);
-  
-  GVertex *v_start = start;
-  while(1){
-    ++it;
-    ++ito;
-    if(v_end == start){
-      indices.push_back(all_mvertices.size());
-      if(it == edges.end ()) break;
-      start = ((*ito) == 1) ? (*it)->getBeginVertex() : (*it)->getEndVertex();
-      v_end = ((*ito) != 1) ? (*it)->getBeginVertex() : (*it)->getEndVertex();
-      v_start = start;
-    }
-    else{
-      if(it == edges.end ()){
-	Msg::Error("Something wrong in edge loop computation");
-	return;
-      }
-      v_start = ((*ito) == 1) ? (*it)->getBeginVertex() : (*it)->getEndVertex();
-      if(v_start != v_end){
-	Msg::Error("Something wrong in edge loop computation");
-	return;
-      }
-      v_end = ((*ito) != 1) ? (*it)->getBeginVertex() : (*it)->getEndVertex();
-    }
-    all_mvertices.push_back(v_start->mesh_vertices[0]);
-    if(*ito == 1)
-      for(unsigned int i = 0; i < (*it)->mesh_vertices.size(); i++)
-        all_mvertices.push_back((*it)->mesh_vertices[i]);
-    else
-      for (int i = (*it)->mesh_vertices.size()-1; i >= 0; i--)
-        all_mvertices.push_back((*it)->mesh_vertices[i]);
-  }
-}
-
 void computeElementShapes(GFace *gf, double &worst, double &avg, double &best,
                           int &nT, int &greaterThan)
 {
@@ -230,6 +228,51 @@ static bool recover_medge(BDS_Mesh *m, GEdge *ge, std::set<EdgeToRecover> *e2r,
     g = m->get_geom(ge->tag(),1);
   }
   
+  for (unsigned int i = 0; i < ge->lines.size(); i++){
+    MVertex *vstart = ge->lines[i]->getVertex(0);
+    MVertex *vend   = ge->lines[i]->getVertex(1);
+    if (pass_ == 1) e2r->insert(EdgeToRecover(vstart->getIndex(), vend->getIndex(), ge));
+    else{
+      BDS_Edge *e = m->recover_edge(vstart->getIndex(), vend->getIndex(), e2r, not_recovered);
+      if (e) e->g = g;
+      else {
+        // Msg::Error("Unable to recover an edge %g %g && %g %g (%d/%d)",
+        //            vstart->x(), vstart->y(), vend->x(), vend->y(), i, 
+        //            ge->mesh_vertices.size());
+        // return false;
+      }
+    }
+  }
+
+  if (pass_ == 2 && ge->getBeginVertex()){
+    MVertex *vstart = *(ge->getBeginVertex()->mesh_vertices.begin());
+    MVertex *vend = *(ge->getEndVertex()->mesh_vertices.begin());    
+    BDS_Point *pstart = m->find_point(vstart->getIndex());
+    BDS_Point *pend = m->find_point(vend->getIndex());
+    if(!pstart->g){
+      m->add_geom (vstart->getIndex(), 0);
+      BDS_GeomEntity *g0 = m->get_geom(vstart->getIndex(), 0);
+      pstart->g = g0;
+    }
+    if(!pend->g){
+      m->add_geom (vend->getIndex(), 0);
+      BDS_GeomEntity *g0 = m->get_geom(vend->getIndex(), 0);
+      pend->g = g0;
+    }
+  }
+
+  return true;
+}
+
+static bool recover_medge_old(BDS_Mesh *m, GEdge *ge, std::set<EdgeToRecover> *e2r, 
+			      std::set<EdgeToRecover> *not_recovered, int pass_)
+{
+  BDS_GeomEntity *g = 0;
+  if (pass_ == 2){
+    m->add_geom (ge->tag(), 1);
+    g = m->get_geom(ge->tag(),1);
+  }
+  
   if(ge->mesh_vertices.size() == 0){
     MVertex *vstart = *(ge->getBeginVertex()->mesh_vertices.begin());
     MVertex *vend = *(ge->getEndVertex()->mesh_vertices.begin());
@@ -321,7 +364,7 @@ static bool recover_medge(BDS_Mesh *m, GEdge *ge, std::set<EdgeToRecover> *e2r,
 // Builds An initial triangular mesh that respects the boundaries of
 // the domain, including embedded points and surfaces
 
-static bool gmsh2DMeshGenerator(GFace *gf, int RECUR_ITER, bool debug = true)
+static bool gmsh2DMeshGenerator(GFace *gf, int RECUR_ITER, bool repairSelfIntersecting1dMesh, bool debug = true)
 {
   BDS_GeomEntity CLASS_F (1, 2);
   typedef std::set<MVertex*> v_container;
@@ -342,12 +385,6 @@ static bool gmsh2DMeshGenerator(GFace *gf, int RECUR_ITER, bool debug = true)
 	all_vertices.insert((*it)->lines[i]->getVertex(0));
 	all_vertices.insert((*it)->lines[i]->getVertex(1));
       }      
-      //      all_vertices.insert((*it)->mesh_vertices.begin(), 
-      //                          (*it)->mesh_vertices.end());
-      //      all_vertices.insert((*it)->getBeginVertex()->mesh_vertices.begin(),
-      //                          (*it)->getBeginVertex()->mesh_vertices.end());
-      //      all_vertices.insert((*it)->getEndVertex()->mesh_vertices.begin(),
-      //                          (*it)->getEndVertex()->mesh_vertices.end());
     }
     ++it;
   }
@@ -393,6 +430,7 @@ static bool gmsh2DMeshGenerator(GFace *gf, int RECUR_ITER, bool debug = true)
     reparamMeshVertexOnFace(here, gf, param);
     U_[count] = param[0];
     V_[count] = param[1];
+    //    printf("coucou : %g %g -> %g %g\n",here->x(),here->y(),param.x(),param.y());
     (*itv)->setIndex(count);
     numbered_vertices[(*itv)->getIndex()] = *itv;
     bbox += SPoint3(param.x(), param.y(), 0);
@@ -422,8 +460,8 @@ static bool gmsh2DMeshGenerator(GFace *gf, int RECUR_ITER, bool debug = true)
     int j = 0;
     while(itv != all_vertices.end()){
       MVertex *here = *itv;
-      double XX = CTX.mesh.rand_factor * LC2D * (double)rand() / (double)RAND_MAX;
-      double YY = CTX.mesh.rand_factor * LC2D * (double)rand() / (double)RAND_MAX;
+      double XX = CTX::instance()->mesh.randFactor * LC2D * (double)rand() / (double)RAND_MAX;
+      double YY = CTX::instance()->mesh.randFactor * LC2D * (double)rand() / (double)RAND_MAX;
       doc.points[j].where.h = U_[j] + XX;
       doc.points[j].where.v = V_[j] + YY;
       doc.points[j].adjacent = NULL;
@@ -535,20 +573,30 @@ static bool gmsh2DMeshGenerator(GFace *gf, int RECUR_ITER, bool debug = true)
       }
       
       std::list<GFace *> facesToRemesh;
-      remeshUnrecoveredEdges(edgesNotRecovered, facesToRemesh);
-      std::set<EdgeToRecover>::iterator itr = edgesNotRecovered.begin();
-      for (; itr != edgesNotRecovered.end(); ++itr){
-	int p1 = itr->p1;
-	int p2 = itr->p2;
-	int tag = itr->ge->tag();
-	Msg::Warning("MEdge %d %d in GEdge %d",p1,p2,tag);
+      if (repairSelfIntersecting1dMesh) 
+	remeshUnrecoveredEdges(edgesNotRecovered, facesToRemesh);
+      else{
+	std::set<EdgeToRecover>::iterator itr = edgesNotRecovered.begin();
+	int *_error = new int[3*edgesNotRecovered.size()];
+	int I = 0;
+	for (; itr != edgesNotRecovered.end(); ++itr){
+	  int p1 = itr->p1;
+	  int p2 = itr->p2;
+	  int tag = itr->ge->tag();
+	  _error[3*I+0] = p1;
+	  _error[3*I+1] = p2;
+	  _error[3*I+2] = tag;
+	  I++;
+	}
+	throw _error;
       }
+
       // delete the mesh
       delete m;
       delete [] U_;
       delete [] V_;
       if (RECUR_ITER < 10 && facesToRemesh.size() == 0)
-	return gmsh2DMeshGenerator(gf, RECUR_ITER+1, debug);
+	return gmsh2DMeshGenerator(gf, RECUR_ITER+1,   repairSelfIntersecting1dMesh, debug);
       return false;
     }
     if(RECUR_ITER > 0)
@@ -657,13 +705,10 @@ static bool gmsh2DMeshGenerator(GFace *gf, int RECUR_ITER, bool debug = true)
 
   // start mesh generation
   if(!AlgoDelaunay2D(gf)){
-    gmshRefineMeshBDS (gf,*m, CTX.mesh.refine_steps, true);
+    gmshRefineMeshBDS (gf,*m, CTX::instance()->mesh.refineSteps, true);
     gmshOptimizeMeshBDS(gf, *m, 2);
-    gmshRefineMeshBDS (gf,*m, CTX.mesh.refine_steps, false);
+    gmshRefineMeshBDS (gf,*m, CTX::instance()->mesh.refineSteps, false);
     gmshOptimizeMeshBDS(gf, *m, 2);
-    if (gf->meshAttributes.recombine){
-      m->recombineIntoQuads (gf->meshAttributes.recombineAngle,gf);
-    }
   }
 
   computeMeshSizeFieldAccuracy(gf, *m, gf->meshStatistics.efficiency_index,
@@ -711,8 +756,12 @@ static bool gmsh2DMeshGenerator(GFace *gf, int RECUR_ITER, bool debug = true)
   // BDS mesh is passed in order not to recompute local coordinates of
   // vertices
   if(AlgoDelaunay2D(gf)){
-    gmshBowyerWatson(gf);
-    for (int i=0;i<CTX.mesh.nb_smoothing;i++)laplaceSmoothing(gf);
+    if (CTX::instance()->mesh.algo2d == ALGO_2D_FRONTAL)
+      gmshBowyerWatsonFrontal(gf);
+    else
+      gmshBowyerWatson(gf);
+    for (int i = 0; i < CTX::instance()->mesh.nbSmoothing; i++) 
+      laplaceSmoothing(gf);
   }
   else if (debug){
     char name[256];
@@ -721,12 +770,15 @@ static bool gmsh2DMeshGenerator(GFace *gf, int RECUR_ITER, bool debug = true)
     sprintf(name, "param%d.pos", gf->tag());
     outputScalarField(m->triangles, name,1);
   }
-  
+
   // delete the mesh
   delete m;
   delete [] U_;
   delete [] V_;
 
+  if (gf->meshAttributes.recombine)
+    gmshRecombineIntoQuads(gf);
+
   computeElementShapes(gf, gf->meshStatistics.worst_element_shape,
                        gf->meshStatistics.average_element_shape,
                        gf->meshStatistics.best_element_shape,
@@ -1041,8 +1093,10 @@ static bool gmsh2DMeshGeneratorPeriodic(GFace *gf, bool debug = true)
         BDS_Point *pp = edgeLoop_BDS[j];
         const double U = pp->u;
         const double V = pp->v;
-        double XX = CTX.mesh.rand_factor * LC2D * (double)rand() / (double)RAND_MAX;
-        double YY = CTX.mesh.rand_factor * LC2D * (double)rand() / (double)RAND_MAX;
+        double XX = CTX::instance()->mesh.randFactor * LC2D * 
+          (double)rand() / (double)RAND_MAX;
+        double YY = CTX::instance()->mesh.randFactor * LC2D * 
+          (double)rand() / (double)RAND_MAX;
         doc.points[count].where.h = U + XX;
         doc.points[count].where.v = V + YY;
         doc.points[count].adjacent = NULL;
@@ -1187,13 +1241,10 @@ static bool gmsh2DMeshGeneratorPeriodic(GFace *gf, bool debug = true)
 
   // start mesh generation
   if (!AlgoDelaunay2D(gf)){
-    gmshRefineMeshBDS(gf, *m, CTX.mesh.refine_steps, true);
+    gmshRefineMeshBDS(gf, *m, CTX::instance()->mesh.refineSteps, true);
     gmshOptimizeMeshBDS(gf, *m, 2);
-    gmshRefineMeshBDS (gf, *m, -CTX.mesh.refine_steps, false);
+    gmshRefineMeshBDS (gf, *m, -CTX::instance()->mesh.refineSteps, false);
     gmshOptimizeMeshBDS(gf, *m, 2, &recover_map);
-    if (gf->meshAttributes.recombine){
-      m->recombineIntoQuads (gf->meshAttributes.recombineAngle, gf);
-    }
     // compute mesh statistics
     computeMeshSizeFieldAccuracy(gf, *m, gf->meshStatistics.efficiency_index,
                                  gf->meshStatistics.longest_edge_length,
@@ -1252,13 +1303,20 @@ static bool gmsh2DMeshGeneratorPeriodic(GFace *gf, bool debug = true)
   }
   
   if (AlgoDelaunay2D(gf)){
-    gmshBowyerWatson(gf);
-    laplaceSmoothing(gf);
+    if (CTX::instance()->mesh.algo2d == ALGO_2D_FRONTAL)
+      gmshBowyerWatsonFrontal(gf);
+    else
+      gmshBowyerWatson(gf);
+    for (int i = 0; i < CTX::instance()->mesh.nbSmoothing; i++) 
+      laplaceSmoothing(gf);
   }
-
+  
   // delete the mesh  
   delete m;
 
+  if (gf->meshAttributes.recombine)
+    gmshRecombineIntoQuads(gf);
+
   computeElementShapes(gf, gf->meshStatistics.worst_element_shape,
                        gf->meshStatistics.average_element_shape,
                        gf->meshStatistics.best_element_shape,
@@ -1295,7 +1353,7 @@ void meshGFace::operator() (GFace *gf)
   if(gf->geomType() == GEntity::BoundaryLayerSurface) return;
   if(gf->geomType() == GEntity::ProjectionFace) return;
   if(gf->meshAttributes.Method == MESH_NONE) return;
-  if(CTX.mesh.mesh_only_visible && !gf->getVisibility()) return;
+  if(CTX::instance()->mesh.meshOnlyVisible && !gf->getVisibility()) return;
 
   // destroy the mesh if it exists
   deMeshGFace dem;
@@ -1306,8 +1364,8 @@ void meshGFace::operator() (GFace *gf)
 
   const char *algo = "Unknown";
   if(AlgoDelaunay2D(gf))
-    algo = (CTX.mesh.algo2d == ALGO_2D_FRONTAL) ? "Frontal" : "Delaunay";
-  else if(CTX.mesh.algo2d == ALGO_2D_MESHADAPT)
+    algo = (CTX::instance()->mesh.algo2d == ALGO_2D_FRONTAL) ? "Frontal" : "Delaunay";
+  else if(CTX::instance()->mesh.algo2d == ALGO_2D_MESHADAPT)
     algo = "MeshAdapt";
   else 
     algo = "MeshAdapt+Delaunay";
@@ -1318,13 +1376,11 @@ void meshGFace::operator() (GFace *gf)
   // compute loops on the fly (indices indicate start and end points
   // of a loop; loops are not yet oriented)
   Msg::Debug("Computing edge loops");
-  //  std::vector<MVertex*> points;
-  //  std::vector<int> indices;
-  //  computeEdgeLoops(gf, points, indices);
 
   Msg::Debug("Generating the mesh");
   if(noseam(gf) || gf->getNativeType() == GEntity::GmshModel || gf->edgeLoops.empty()){
-    gmsh2DMeshGenerator(gf, 0, debugSurface >= 0 || debugSurface == -100);
+    //gmsh2DMeshGeneratorPeriodic(gf, debugSurface >= 0 || debugSurface == -100);
+    gmsh2DMeshGenerator(gf, 0, repairSelfIntersecting1dMesh, debugSurface >= 0 || debugSurface == -100);
   }
   else{
     if(!gmsh2DMeshGeneratorPeriodic(gf, debugSurface >= 0 || debugSurface == -100))
@@ -1362,31 +1418,52 @@ void orientMeshGFace::operator()(GFace *gf)
 
   if(gf->geomType() == GEntity::ProjectionFace) return;
 
-  // surface orientions in OCC are not consistent with the orientation
-  // of the bounding edges, so just leave them unchanged:
-  if(gf->getNativeType() == GEntity::OpenCascadeModel) return;
-
   // in old versions we did not reorient transfinite surface meshes;
   // we could add the following to provide backward compatibility:
   // if(gf->meshAttributes.Method == MESH_TRANSFINITE) return;
 
-  // orients the mesh to match the orientation of the first edge
-  std::list<GEdge*> edges = gf->edges();
-  std::list<int> ori = gf->orientations();
-  if(edges.empty() || ori.empty()) return;
-  GEdge *ge = *edges.begin();
-  GVertex *beg = ge->getBeginVertex();
-  GVertex *end = ge->getEndVertex();
-  if(!beg || beg->mesh_vertices.empty() || !end || end->mesh_vertices.empty()) return;
-  MVertex *v1 = beg->mesh_vertices[0];
-  MVertex *v2 = ge->mesh_vertices.empty() ? end->mesh_vertices[0] : ge->mesh_vertices[0];
-  int sign = *ori.begin();
-  MEdge ref(sign > 0 ? v1 : v2, sign > 0 ? v2 : v1);
-  if(shouldRevert(ref, gf->triangles) || shouldRevert(ref, gf->quadrangles)){
-    Msg::Debug("Reverting orientation of mesh in face %d", gf->tag());
-    for(unsigned int i = 0; i < gf->triangles.size(); i++)
-      gf->triangles[i]->revert();
-    for(unsigned int i = 0; i < gf->quadrangles.size(); i++)
-      gf->quadrangles[i]->revert();
+  if(gf->getNativeType() == GEntity::OpenCascadeModel){
+    // surface orientions in OCC do not seem to be consistent with the
+    // orientation of the bounding edges, so we compare the normals
+    // pointwise in an element
+    for(int i = 0; i < gf->getNumMeshElements(); i++){
+      MElement *e = gf->getMeshElement(i);
+      for(int j = 0; j < e->getNumVertices(); j++){
+        MVertex *v = e->getVertex(j);
+        SPoint2 param;
+        if(reparamMeshVertexOnFace(v, gf, param)){
+          SVector3 ne = e->getFace(0).normal();
+          SVector3 nf = gf->normal(param); 
+          if(dot(ne, nf) < 0){
+            Msg::Debug("Reverting orientation of mesh in face %d", gf->tag());
+            for(int k = 0; k < gf->getNumMeshElements(); k++)
+              gf->getMeshElement(k)->revert();
+          }
+          return;
+        }
+      }
+    }
+    Msg::Warning("Could not orient mesh in face %d", gf->tag());
+  }
+  else{
+    // orient the mesh to match the orientation of the first edge
+    std::list<GEdge*> edges = gf->edges();
+    std::list<int> ori = gf->orientations();
+    if(edges.empty() || ori.empty()) return;
+    GEdge *ge = *edges.begin();
+    GVertex *beg = ge->getBeginVertex();
+    GVertex *end = ge->getEndVertex();
+    if(!beg || beg->mesh_vertices.empty() || !end || end->mesh_vertices.empty()) return;
+    MVertex *v1 = beg->mesh_vertices[0];
+    MVertex *v2 = ge->mesh_vertices.empty() ? end->mesh_vertices[0] : ge->mesh_vertices[0];
+    int sign = *ori.begin();
+    MEdge ref(sign > 0 ? v1 : v2, sign > 0 ? v2 : v1);
+    if(shouldRevert(ref, gf->triangles) || shouldRevert(ref, gf->quadrangles)){
+      Msg::Debug("Reverting orientation of mesh in face %d", gf->tag());
+      for(unsigned int i = 0; i < gf->triangles.size(); i++)
+        gf->triangles[i]->revert();
+      for(unsigned int i = 0; i < gf->quadrangles.size(); i++)
+        gf->quadrangles[i]->revert();
+    }
   }
 }
