@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -11,7 +11,9 @@
 #include "MVertex.h"
 #include "MElement.h"
 #include "BackgroundMesh.h"
+#include "Numeric.h"
 #include "GmshMessage.h"
+#include "Generator.h"
 
 static void setLcsInit(MTriangle *t, std::map<MVertex*, double> &vSizes)
 {
@@ -25,7 +27,7 @@ static void setLcsInit(MTriangle *t, std::map<MVertex*, double> &vSizes)
   }
 }
 
-static void setLcs(MTriangle *t, std::map<MVertex*,double> &vSizes)
+static void setLcs(MTriangle *t, std::map<MVertex*, double> &vSizes)
 {
   for (int i = 0; i < 3; i++){
     for (int j = i + 1; j < 3; j++){
@@ -43,25 +45,12 @@ static void setLcs(MTriangle *t, std::map<MVertex*,double> &vSizes)
   }
 }
 
-static void setLcs(MLine *t, std::map<MVertex*,double> &vSizes)
-{
-  MVertex *vi = t->getVertex(0);
-  MVertex *vj = t->getVertex(1);
-  double dx = vi->x()-vj->x();
-  double dy = vi->y()-vj->y();
-  double dz = vi->z()-vj->z();
-  double l = sqrt(dx * dx + dy * dy + dz * dz);
-  std::map<MVertex*,double>::iterator iti = vSizes.find(vi);	  
-  std::map<MVertex*,double>::iterator itj = vSizes.find(vj);	  
-  if (iti->second < 0 || iti->second < l) iti->second = l;
-  if (itj->second < 0 || itj->second < l) itj->second = l;
-}
-
-void buidMeshGenerationDataStructures(GFace *gf, std::set<MTri3*, compareTri3Ptr> &AllTris,
-                                      std::vector<double> &vSizes,
-                                      std::vector<double> &vSizesBGM,
-                                      std::vector<double> &Us,
-                                      std::vector<double> &Vs)
+void buildMeshGenerationDataStructures(GFace *gf, 
+                                       std::set<MTri3*, compareTri3Ptr> &AllTris,
+                                       std::vector<double> &vSizes,
+                                       std::vector<double> &vSizesBGM,
+                                       std::vector<double> &Us,
+                                       std::vector<double> &Vs)
 {
   std::map<MVertex*, double> vSizesMap;
   std::list<GEdge*> edges = gf->edges();
@@ -69,15 +58,6 @@ void buidMeshGenerationDataStructures(GFace *gf, std::set<MTri3*, compareTri3Ptr
   for (unsigned int i = 0;i < gf->triangles.size(); i++)
     setLcsInit(gf->triangles[i], vSizesMap);
 
-//   std::list<GEdge*>::iterator it = edges.begin();
-//   while (it != edges.end()){
-//     GEdge *ge = *it ;
-//     for (int i=0;i<ge->lines.size();i++){
-//       setLcs(ge->lines[i], vSizesMap);      
-//     }
-//     ++it;
-//   }
-
   for (unsigned int i = 0;i < gf->triangles.size(); i++)
     setLcs(gf->triangles[i], vSizesMap);
 
@@ -87,7 +67,7 @@ void buidMeshGenerationDataStructures(GFace *gf, std::set<MTri3*, compareTri3Ptr
     std::list<GVertex*>::iterator itvx = emb_vertx.begin();
     while(itvx != emb_vertx.end()){
       MVertex *v = *((*itvx)->mesh_vertices.begin());
-      vSizesMap[v] = std::min(vSizesMap[v],(*itvx)->prescribedMeshSizeAtVertex());      
+      vSizesMap[v] = std::min(vSizesMap[v], (*itvx)->prescribedMeshSizeAtVertex());      
       ++itvx;
     }
   }
@@ -104,19 +84,20 @@ void buidMeshGenerationDataStructures(GFace *gf, std::set<MTri3*, compareTri3Ptr
     Vs.push_back(param[1]);
   }
   for(unsigned int i = 0; i < gf->triangles.size(); i++){
-    double lc = 0.3333333333 * (vSizes [gf->triangles[i]->getVertex(0)->getNum()] +
-                                vSizes [gf->triangles[i]->getVertex(1)->getNum()] +
-                                vSizes [gf->triangles[i]->getVertex(2)->getNum()]);
+    double lc = 0.3333333333 * (vSizes[gf->triangles[i]->getVertex(0)->getNum()] +
+                                vSizes[gf->triangles[i]->getVertex(1)->getNum()] +
+                                vSizes[gf->triangles[i]->getVertex(2)->getNum()]);
     AllTris.insert(new MTri3(gf->triangles[i], lc));
   }
   gf->triangles.clear();
-  connectTriangles(AllTris );
+  connectTriangles(AllTris);
 }
 
-void transferDataStructure(GFace *gf, std::set<MTri3*, compareTri3Ptr> &AllTris)
+void transferDataStructure(GFace *gf, std::set<MTri3*, compareTri3Ptr> &AllTris,
+                           std::vector<double> &Us, std::vector<double> &Vs)
 {
   while (1) {
-    if (AllTris.begin() == AllTris.end() ) break;
+    if (AllTris.begin() == AllTris.end()) break;
     MTri3 *worst = *AllTris.begin();
     if (worst->isDeleted())
       delete worst->tri();
@@ -125,18 +106,40 @@ void transferDataStructure(GFace *gf, std::set<MTri3*, compareTri3Ptr> &AllTris)
     delete worst;
     AllTris.erase(AllTris.begin());      
   }
+
+  // make sure all the triangles are oriented in the same way in
+  // parameter space (it would be nicer to change the actual algorithm
+  // to ensure that we create correctly-oriented triangles in the
+  // first place)
+  if(gf->triangles.size() > 1){
+    double n1[3], n2[3];
+    MTriangle *t = gf->triangles[0];
+    MVertex *v0 = t->getVertex(0), *v1 = t->getVertex(1), *v2 = t->getVertex(2);
+    normal3points(Us[v0->getNum()], Vs[v0->getNum()], 0.,
+                  Us[v1->getNum()], Vs[v1->getNum()], 0.,
+                  Us[v2->getNum()], Vs[v2->getNum()], 0., n1);
+    for(unsigned int j = 1; j < gf->triangles.size(); j++){
+      t = gf->triangles[j];
+      v0 = t->getVertex(0); v1 = t->getVertex(1); v2 = t->getVertex(2);
+      normal3points(Us[v0->getNum()], Vs[v0->getNum()], 0.,
+                    Us[v1->getNum()], Vs[v1->getNum()], 0.,
+                    Us[v2->getNum()], Vs[v2->getNum()], 0., n2);
+      double pp; prosca(n1, n2, &pp);
+      if(pp < 0) t->revert();
+    }
+  }
 }
 
-void buildVertexToTriangle(std::vector<MTriangle*> &triangles, v2t_cont &adj)
+template <class T>
+void buildVertexToElement(std::vector<T*> &eles, v2t_cont &adj)
 {
-  adj.clear();
-  for (unsigned int i = 0; i < triangles.size(); i++){
-    MTriangle *t = triangles[i];
-    for (unsigned int j = 0; j < 3; j++){
+  for (unsigned int i = 0; i < eles.size(); i++){
+    T *t = eles[i];
+    for (int j = 0; j < t->getNumVertices(); j++){
       MVertex *v = t->getVertex(j);
       v2t_cont :: iterator it = adj.find(v);
       if (it == adj.end()){
-        std::vector<MTriangle*> one;
+        std::vector<MElement*> one;
         one.push_back(t);
         adj[v] = one;
       }
@@ -147,36 +150,47 @@ void buildVertexToTriangle(std::vector<MTriangle*> &triangles, v2t_cont &adj)
   }
 }
 
-void buildEdgeToTriangle(GFace *gf, e2t_cont &adj)
-{
-  buildEdgeToTriangle(gf->triangles, adj);
-}
-
-void buildEdgeToTriangle(std::vector<MTriangle*> &triangles, e2t_cont &adj)
+void buildVertexToTriangle(std::vector<MTriangle*> &eles, v2t_cont &adj)
 {
   adj.clear();
-  for (unsigned int i = 0; i < triangles.size(); i++){
-    MTriangle *t = triangles[i];
-    for (unsigned int j = 0; j < 3; j++){
-      MVertex *v1 = t->getVertex(j);
-      MVertex *v2 = t->getVertex((j + 1) % 3);
-      MEdge e(v1, v2);
+  buildVertexToElement(eles,adj);
+}
+
+template <class T>
+void buildEdgeToElement(std::vector<T*> &elements, e2t_cont &adj)
+{
+  for (unsigned int i = 0; i < elements.size(); i++){
+    T *t = elements[i];
+    for (int j = 0; j < t->getNumEdges(); j++){
+      MEdge e = t->getEdge(j);
       e2t_cont::iterator it = adj.find(e);
       if (it == adj.end()){
-        std::pair<MTriangle*, MTriangle*> one = std::make_pair(t, (MTriangle*)0);
+        std::pair<MElement*, MElement*> one = std::make_pair(t, (MElement*)0);
         adj[e] = one;
       }
-      else
-        {
-          it->second.second = t;
-        }
+      else{
+        it->second.second = t;
+      }
     }
   }
 }
 
-void parametricCoordinates(MTriangle *t, GFace *gf, double u[3], double v[3])
+void buildEdgeToElement(GFace *gf, e2t_cont &adj)
 {
-  for (unsigned int j = 0; j < 3; j++){
+  adj.clear();
+  buildEdgeToElement(gf->triangles, adj);
+  buildEdgeToElement(gf->quadrangles, adj);
+}
+
+void buildEdgeToTriangle(std::vector<MTriangle*> &tris, e2t_cont &adj)
+{
+  adj.clear();
+  buildEdgeToElement(tris, adj);
+}
+
+void parametricCoordinates(MElement *t, GFace *gf, double u[4], double v[4])
+{
+  for (int j = 0; j < t->getNumVertices(); j++){
     MVertex *ver = t->getVertex(j);
     SPoint2 param;
     reparamMeshVertexOnFace(ver, gf, param);
@@ -188,7 +202,8 @@ void parametricCoordinates(MTriangle *t, GFace *gf, double u[3], double v[3])
 void laplaceSmoothing(GFace *gf)
 {
   v2t_cont adj;
-  buildVertexToTriangle(gf->triangles, adj);
+  buildVertexToElement(gf->triangles, adj);
+  buildVertexToElement(gf->quadrangles, adj);
 
   for (int i = 0; i < 5; i++){
     v2t_cont :: iterator it = adj.begin();
@@ -200,30 +215,35 @@ void laplaceSmoothing(GFace *gf)
         double initu,initv;
         ver->getParameter(0, initu);
         ver->getParameter(1, initv);
-        const std::vector<MTriangle*> &lt = it->second;
-        double fact = lt.size() ? 1. / (3. * lt.size()) : 0;
+        const std::vector<MElement*> &lt = it->second;
         double cu = 0, cv = 0;
-        double pu[3], pv[3];
+        double pu[4], pv[4];
+	double fact  = 0.0;
         for (unsigned int i = 0; i < lt.size(); i++){
           parametricCoordinates(lt[i], gf, pu, pv);
-          cu += fact * (pu[0] + pu[1] + pu[2]);
-          cv += fact * (pv[0] + pv[1] + pv[2]);
+          cu += (pu[0] + pu[1] + pu[2]);
+          cv += (pv[0] + pv[1] + pv[2]);
+	  if (lt[i]->getNumVertices() == 4){
+	    cu += pu[3];
+	    cv += pv[3];
+	  }	    
+	  fact += lt[i]->getNumVertices();
           // have to test validity !
         }
-        ver->setParameter(0, cu);
-        ver->setParameter(1, cv);
-        GPoint pt = gf->point(SPoint2(cu, cv));
-        ver->x() = pt.x();
-        ver->y() = pt.y();
-        ver->z() = pt.z();
+	if (fact != 0.0){
+	  ver->setParameter(0, cu / fact);
+	  ver->setParameter(1, cv / fact);
+	  GPoint pt = gf->point(SPoint2(cu / fact, cv / fact));
+	  ver->x() = pt.x();
+	  ver->y() = pt.y();
+	  ver->z() = pt.z();
+	}
       }
       ++it;
     }  
   }
 }
 
-extern void fourthPoint(double *p1, double *p2, double *p3, double *p4);
-
 double surfaceTriangleUV(MVertex *v1, MVertex *v2, MVertex *v3,           
                          const std::vector<double> &Us,
                          const std::vector<double> &Vs)
@@ -416,7 +436,7 @@ bool gmshEdgeSplit(const double lMax, MTri3 *t1, GFace *gf, int iLocalEdge,
   if (al <= lMax) return false;
 
   MVertex *v3 = t1->tri()->getVertex((iLocalEdge + 1) % 3);
-  MVertex *v4 = 0 ;
+  MVertex *v4 = 0;
   for (int i = 0; i < 3; i++)
     if (t2->tri()->getVertex(i) != v1 && t2->tri()->getVertex(i) != v2)
       v4 = t2->tri()->getVertex(i);
@@ -615,7 +635,7 @@ bool gmshVertexCollapse(const double lMin, MTri3 *t1, GFace *gf,
   MVertex *v;
   std::vector<MTri3*> cavity;
   std::vector<MTri3*> outside;
-  std::vector<MVertex*> ring ;
+  std::vector<MVertex*> ring;
 
   if (!gmshBuildVertexCavity(t1, iLocalVertex, &v, cavity, outside, ring)) return false;
   
@@ -675,7 +695,7 @@ int edgeSwapPass (GFace *gf, std::set<MTri3*, compareTri3Ptr> &allTris,
                   const std::vector<double> &Us, const std::vector<double> &Vs,
                   const std::vector<double> &vSizes, const std::vector<double> &vSizesBGM)
 {
-  typedef std::set<MTri3*, compareTri3Ptr> CONTAINER ;
+  typedef std::set<MTri3*, compareTri3Ptr> CONTAINER;
 
   int nbSwapTot = 0;
   std::set<swapquad> configs;
@@ -711,7 +731,7 @@ int edgeSplitPass(double maxLC, GFace *gf, std::set<MTri3*,compareTri3Ptr> &allT
                   std::vector<double> &Us, std::vector<double> &Vs,
                   std::vector<double> &vSizes, std::vector<double> &vSizesBGM)
 {
-  typedef std::set<MTri3*, compareTri3Ptr> CONTAINER ;
+  typedef std::set<MTri3*, compareTri3Ptr> CONTAINER;
   std::vector<MTri3*> newTris;
 
   int nbSplit = 0;
@@ -743,7 +763,7 @@ int edgeCollapsePass(double minLC, GFace *gf, std::set<MTri3*,compareTri3Ptr> &a
                      std::vector<double> &Us, std::vector<double> &Vs,
                      std::vector<double> &vSizes, std::vector<double> &vSizesBGM)
 {
-  typedef std::set<MTri3*,compareTri3Ptr> CONTAINER ;
+  typedef std::set<MTri3*,compareTri3Ptr> CONTAINER;
   std::vector<MTri3*> newTris;
 
   int nbCollapse = 0;
@@ -772,4 +792,127 @@ int edgeCollapsePass(double minLC, GFace *gf, std::set<MTri3*,compareTri3Ptr> &a
   return nbCollapse;
 }
 
+extern double angle3Points(MVertex *p1, MVertex *p2, MVertex *p3);
 
+struct recombine_triangle
+{
+  MElement *t1, *t2;
+  double angle;
+  MVertex *n1, *n2, *n3, *n4;
+  recombine_triangle(const MEdge &me, MElement *_t1, MElement *_t2)
+    : t1(_t1), t2(_t2)
+  {
+    n1 = me.getVertex(0);
+    n2 = me.getVertex(1);
+    
+    if (t1->getVertex(0) != n1 && t1->getVertex(0) != n2) n3 = t1->getVertex(0);
+    else if (t1->getVertex(1) != n1 && t1->getVertex(1) != n2) n3 = t1->getVertex(1);
+    else if (t1->getVertex(2) != n1 && t1->getVertex(2) != n2) n3 = t1->getVertex(2);
+    if (t2->getVertex(0) != n1 && t2->getVertex(0) != n2) n4 = t2->getVertex(0);
+    else if (t2->getVertex(1) != n1 && t2->getVertex(1) != n2) n4 = t2->getVertex(1);
+    else if (t2->getVertex(2) != n1 && t2->getVertex(2) != n2) n4 = t2->getVertex(2);
+
+    double a1 = 180 * angle3Points(n1, n4, n2) / M_PI;
+    double a2 = 180 * angle3Points(n4, n2, n3) / M_PI;
+    double a3 = 180 * angle3Points(n2, n3, n1) / M_PI;
+    double a4 = 180 * angle3Points(n3, n1, n4) / M_PI;
+    angle = fabs(90. - a1);
+    angle = std::max(fabs(90. - a2),angle);
+    angle = std::max(fabs(90. - a3),angle);
+    angle = std::max(fabs(90. - a4),angle);    
+  }
+  bool operator < (const recombine_triangle &other) const
+  {
+    return angle < other.angle;
+  }
+};
+
+static void _gmshRecombineIntoQuads(GFace *gf)
+{
+  std::set<MVertex*> emb_edgeverts;
+  {
+    std::list<GEdge*> emb_edges = gf->embeddedEdges();
+    std::list<GEdge*>::iterator ite = emb_edges.begin();
+    while(ite != emb_edges.end()){
+      if(!(*ite)->isMeshDegenerated()){
+        emb_edgeverts.insert((*ite)->mesh_vertices.begin(),
+                             (*ite)->mesh_vertices.end() );
+        emb_edgeverts.insert((*ite)->getBeginVertex()->mesh_vertices.begin(),
+                             (*ite)->getBeginVertex()->mesh_vertices.end());
+        emb_edgeverts.insert((*ite)->getEndVertex()->mesh_vertices.begin(),
+                             (*ite)->getEndVertex()->mesh_vertices.end());
+      }
+      ++ite;
+    }
+  }
+
+  e2t_cont adj;
+  buildEdgeToElement(gf->triangles, adj);
+
+  std::set<recombine_triangle> pairs;
+  for (e2t_cont::iterator it = adj.begin(); it!= adj.end(); ++it){
+    if (it->second.second && 
+        it->second.first->getNumVertices() == 3 &&  
+	it->second.second->getNumVertices() == 3 &&
+        (emb_edgeverts.find(it->first.getVertex(0)) == emb_edgeverts.end() ||
+         emb_edgeverts.find(it->first.getVertex(1)) == emb_edgeverts.end()))
+      pairs.insert(recombine_triangle(it->first,
+				      it->second.first,
+				      it->second.second));
+  }
+
+  std::set<MElement*> touched;
+  std::set<recombine_triangle>::iterator itp = pairs.begin();
+  while(itp != pairs.end()){
+    // recombine if difference between max quad angle and right
+    // angle is smaller than tol
+    if(itp->angle < gf->meshAttributes.recombineAngle){
+      MElement *t1 = itp->t1;
+      MElement *t2 = itp->t2;
+      if (touched.find(t1) == touched.end() &&
+	  touched.find(t2) == touched.end()){
+	touched.insert(t1);
+	touched.insert(t2);
+
+        int orientation = 0;
+        for(int i = 0; i < 3; i++) {
+          if(t1->getVertex(i) == itp->n1) {
+            if(t1->getVertex((i + 1) % 3) == itp->n2)
+              orientation = 1;
+            else
+              orientation = -1;
+            break;
+          }
+        }
+        MQuadrangle *q;
+        if(orientation < 0)
+          q = new MQuadrangle(itp->n1, itp->n3, itp->n2, itp->n4);
+        else
+          q = new MQuadrangle(itp->n1, itp->n4, itp->n2, itp->n3);
+	gf->quadrangles.push_back(q);
+      }
+    }
+    ++itp;
+  }
+
+  std::vector<MTriangle*> triangles2;
+  for (unsigned int i = 0; i < gf->triangles.size(); i++){
+    if (touched.find(gf->triangles[i]) == touched.end()){
+      triangles2.push_back(gf->triangles[i]);
+    }
+    else {
+      delete gf->triangles[i];
+    }    
+  } 
+  gf->triangles = triangles2;
+}
+
+void gmshRecombineIntoQuads(GFace *gf)
+{
+  _gmshRecombineIntoQuads(gf);
+  laplaceSmoothing(gf);
+  _gmshRecombineIntoQuads(gf);
+  laplaceSmoothing(gf);
+  _gmshRecombineIntoQuads(gf);
+  laplaceSmoothing(gf);
+}
