@@ -1,10 +1,12 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
 
 #include <stdlib.h>
 #include <vector>
+#include "GmshConfig.h"
+#include "GmshMessage.h"
 #include "meshGRegion.h"
 #include "meshGRegionDelaunayInsertion.h"
 #include "GModel.h"
@@ -13,11 +15,8 @@
 #include "GEdge.h"
 #include "gmshRegion.h"
 #include "BDS.h"
-#include "GmshMessage.h"
 #include "Context.h"
 
-extern Context_T CTX;
-
 void getAllBoundingVertices(GRegion *gr, std::set<MVertex*> &allBoundingVertices)
 {
   std::list<GFace*> faces = gr->faces();
@@ -191,7 +190,8 @@ void TransferTetgenMesh(GRegion *gr, tetgenio &in, tetgenio &out,
                      v[j]->onWhat()->mesh_vertices.end(),
                      v[j]));
         MVertex *v1b;
-	if (CTX.mesh.order > 1 && CTX.mesh.second_order_experimental){
+	if (CTX::instance()->mesh.order > 1 && 
+            CTX::instance()->mesh.secondOrderExperimental){
 	  // PARAMETRIC COORDINATES SHOULD BE SET for the vertex !!!!!!!!!!!!!
 	  // This is not 100 % safe yet, so we reserve that operation for high order
 	  // meshes.
@@ -200,10 +200,11 @@ void TransferTetgenMesh(GRegion *gr, tetgenio &in, tetgenio &out,
 
 	  // To be safe, we should ensure that this mesh motion does not lead to an invalid mesh !!!!
 	  //v1b = new MVertex(gp.x(),gp.y(),gp.z(),gf);
-	  if (gp.g()){
+	  if (gp.g() ){
+	    //v1b = new MFaceVertex(v[j]->x(), v[j]->y(), v[j]->z(),gf,gp.u(),gp.v());
 	    v1b = new MFaceVertex(gp.x(),gp.y(),gp.z(),gf,gp.u(),gp.v());
-	    Msg::Info("The point has been projected back to the surface (%g %g %g) -> (%g %g %g)",
-		       v[j]->x(), v[j]->y(), v[j]->z(),gp.x(),gp.y(),gp.z());
+	    //	    Msg::Info("The point has been projected back to the surface (%g %g %g) -> (%g %g %g)",
+	    //	      v[j]->x(), v[j]->y(), v[j]->z(),gp.x(),gp.y(),gp.z());
 	  }
 	  else{
 	    v1b = new MVertex(v[j]->x(), v[j]->y(), v[j]->z(),gf);	  
@@ -216,7 +217,7 @@ void TransferTetgenMesh(GRegion *gr, tetgenio &in, tetgenio &out,
 
         gf->mesh_vertices.push_back(v1b);
         numberedV[out.trifacelist[i * 3 + j] - 1] = v1b;
-        delete v[j];
+	delete v[j];
         v[j] = v1b;
       }
     }
@@ -559,7 +560,7 @@ void meshGRegion::operator() (GRegion *gr)
 
   if(gr->geomType() == GEntity::DiscreteVolume) return;
   if(gr->meshAttributes.Method == MESH_NONE) return;
-  if(CTX.mesh.mesh_only_visible && !gr->getVisibility()) return;
+  if(CTX::instance()->mesh.meshOnlyVisible && !gr->getVisibility()) return;
 
   ExtrudeParams *ep = gr->meshAttributes.extrude;
 
@@ -581,10 +582,10 @@ void meshGRegion::operator() (GRegion *gr)
     }
   }
 
-  if(CTX.mesh.algo3d == ALGO_3D_TETGEN_DELAUNAY){
+  if(CTX::instance()->mesh.algo3d == ALGO_3D_TETGEN_DELAUNAY){
     delaunay.push_back(gr);
   }
-  else if(CTX.mesh.algo3d == ALGO_3D_NETGEN ){
+  else if(CTX::instance()->mesh.algo3d == ALGO_3D_NETGEN ){
 #if !defined(HAVE_NETGEN)
     Msg::Error("Netgen is not compiled in this version of Gmsh");
 #else
@@ -593,7 +594,7 @@ void meshGRegion::operator() (GRegion *gr)
     meshNormalsPointOutOfTheRegion(gr);
     std::vector<MVertex*> numberedV;
     Ng_Mesh *ngmesh = buildNetgenStructure(gr, false, numberedV);
-    NgAddOn_GenerateVolumeMesh(ngmesh, CTX.lc); // does not optimize
+    NgAddOn_GenerateVolumeMesh(ngmesh, CTX::instance()->lc); // does not optimize
     TransferVolumeMesh(gr, ngmesh, numberedV);
     Ng_DeleteMesh(ngmesh);
     Ng_Exit();
@@ -623,7 +624,7 @@ void optimizeMeshGRegionNetgen::operator() (GRegion *gr)
   deMeshGRegion dem;
   dem(gr);
   // optimize mesh
-  NgAddOn_OptimizeVolumeMesh(ngmesh, CTX.lc);
+  NgAddOn_OptimizeVolumeMesh(ngmesh, CTX::instance()->lc);
   TransferVolumeMesh(gr, ngmesh, numberedV);
   Ng_DeleteMesh(ngmesh);
   Ng_Exit();
