@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -12,8 +12,8 @@
 #include "MElement.h"
 #include "GmshMatrix.h"
 
-class gmshElasticityTerm : public gmshNodalFemTerm {
-  double _E,_nu;
+class gmshElasticityTerm : public gmshNodalFemTerm<double> {
+  double _E, _nu;
   int _iField;
  protected:
   virtual int sizeOfR(MElement *e) const { return 3 * e->getNumVertices(); }
@@ -27,8 +27,8 @@ class gmshElasticityTerm : public gmshNodalFemTerm {
   }
  public:
   gmshElasticityTerm(GModel *gm, double E, double nu, int iField = 1) : 
-    gmshNodalFemTerm(gm), _E(E), _nu(nu), _iField(iField){}
-  void elementMatrix(MElement *e, Double_Matrix & m) const;
+    gmshNodalFemTerm<double>(gm), _E(E), _nu(nu), _iField(iField){}
+  void elementMatrix(MElement *e, gmshMatrix<double> &m) const;
 };
 
 #endif
