@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -7,13 +7,15 @@
 #define _GMSH_LAPLACE_H_
 
 #include "gmshTermOfFormulation.h"
+#include "gmshFunction.h"
 #include "Gmsh.h"
 #include "GModel.h"
 #include "MElement.h"
 #include "GmshMatrix.h"
 
-class gmshLaplaceTerm : public gmshNodalFemTerm {
-  const double _diffusivity;
+class gmshLaplaceTerm : public gmshNodalFemTerm<double> {
+ private:
+  const gmshFunction<double> *_diffusivity;
   const int _iField ;
  protected:
   virtual int sizeOfR(MElement *e) const { return e->getNumVertices(); }
@@ -25,11 +27,9 @@ class gmshLaplaceTerm : public gmshNodalFemTerm {
     *iFieldR = _iField;
   }
  public:
-  gmshLaplaceTerm(GModel *gm, double diffusivity = 1.0, int iField = 0) : 
-    gmshNodalFemTerm(gm), _diffusivity(diffusivity), _iField(iField){}
-  virtual void elementMatrix(MElement *e, Double_Matrix &m) const;
-  double getDiffusivity () const { return _diffusivity; }
+  gmshLaplaceTerm(GModel *gm, gmshFunction<double> *diffusivity, int iField = 0) : 
+    gmshNodalFemTerm<double>(gm), _diffusivity(diffusivity), _iField(iField){}
+  virtual void elementMatrix(MElement *e, gmshMatrix<double> &m) const;
 };
 
-
 #endif
