@@ -1,20 +1,20 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
 
 #include <vector>
+#include "GmshConfig.h"
 #include "Annotate.h"
 #include "Context.h"
 
 #if defined(HAVE_FLTK)
-#include "GmshUI.h"
-#include "GUI.h"
+#include <FL/gl.h>
+#include "drawContext.h"
 #include "Draw.h"
+#include "GUI.h"
 #endif
 
-extern Context_T CTX;
-
 StringXNumber AnnotateOptions_Number[] = {
   {GMSH_FULLRC, "X", GMSH_AnnotatePlugin::callbackX, 50.},
   {GMSH_FULLRC, "Y", GMSH_AnnotatePlugin::callbackY, 30.},
@@ -42,26 +42,27 @@ static double getStyle()
 {
   int fontsize = (int)AnnotateOptions_Number[4].def, font = 0, align = 0;
 #if defined(HAVE_FLTK)
-  font = GetFontIndex(AnnotateOptions_String[1].def);
-  align = GetFontAlign(AnnotateOptions_String[2].def);
+  font = GetFontIndex(AnnotateOptions_String[1].def.c_str());
+  align = GetFontAlign(AnnotateOptions_String[2].def.c_str());
 #endif
   return (double)((align<<16)|(font<<8)|(fontsize));
 }
 
-void GMSH_AnnotatePlugin::draw()
+void GMSH_AnnotatePlugin::draw(void *context)
 {
 #if defined(HAVE_FLTK)
   double X = AnnotateOptions_Number[0].def;
   double Y = AnnotateOptions_Number[1].def;
   double Z = AnnotateOptions_Number[2].def;
   double style = getStyle();
+  drawContext *ctx = (drawContext*)context;
 
-  glColor4ubv((GLubyte *) & CTX.color.fg);
+  glColor4ubv((GLubyte *) & CTX::instance()->color.fg);
   if(AnnotateOptions_Number[3].def){ // 3D
     glRasterPos3d(X, Y, Z);
-    Draw_String(AnnotateOptions_String[0].def, style);
+    ctx->drawString(AnnotateOptions_String[0].def, style);
     // draw 10-pixel marker
-    double d = 10 * CTX.pixel_equiv_x / CTX.s[0];
+    double d = 10 * ctx->pixel_equiv_x / ctx->s[0];
     glBegin(GL_LINES);
     glVertex3d(X-d,Y,Z); glVertex3d(X+d,Y,Z);
     glVertex3d(X,Y-d,Z); glVertex3d(X,Y+d,Z);
@@ -75,13 +76,13 @@ void GMSH_AnnotatePlugin::draw()
 
     glMatrixMode(GL_PROJECTION);
     glLoadIdentity();
-    glOrtho((double)CTX.viewport[0], (double)CTX.viewport[2],
-            (double)CTX.viewport[1], (double)CTX.viewport[3], -1., 1.);
+    glOrtho((double)ctx->viewport[0], (double)ctx->viewport[2],
+            (double)ctx->viewport[1], (double)ctx->viewport[3], -1., 1.);
     glMatrixMode(GL_MODELVIEW);
     glLoadIdentity();
-    Fix2DCoordinates(&X, &Y);
+    ctx->fix2dCoordinates(&X, &Y);
     glRasterPos2d(X, Y);
-    Draw_String(AnnotateOptions_String[0].def, style);
+    ctx->drawString(AnnotateOptions_String[0].def, style);
     // draw 10-pixel marker
     glBegin(GL_LINES);
     glVertex2d(X-10,Y); glVertex2d(X+10,Y);
@@ -112,14 +113,14 @@ double GMSH_AnnotatePlugin::callback(int num, int action, double value, double *
   return 0.;
 }
 
-const char *GMSH_AnnotatePlugin::callbackStr(int num, int action, const char *value,
-                                             const char **opt)
+std::string GMSH_AnnotatePlugin::callbackStr(int num, int action, std::string value,
+                                             std::string &opt)
 {
-  *opt = value;
+  opt = value;
 #if defined(HAVE_FLTK)
   DrawPlugin(draw);
 #endif
-  return NULL;
+  return opt;
 }
 
 double GMSH_AnnotatePlugin::callbackX(int num, int action, double value)
@@ -127,9 +128,9 @@ double GMSH_AnnotatePlugin::callbackX(int num, int action, double value)
   // not perfect: the change will only take place if we reopen the dialog...
   int dim3 = (int)AnnotateOptions_Number[3].def;
   return callback(num, action, value, &AnnotateOptions_Number[0].def,
-                  dim3 ? CTX.lc/200. : 0.5, 
-                  dim3 ? -CTX.lc : -100., 
-                  dim3 ? CTX.lc : 100000.);
+                  dim3 ? CTX::instance()->lc/200. : 0.5, 
+                  dim3 ? -CTX::instance()->lc : -100., 
+                  dim3 ? CTX::instance()->lc : 100000.);
 }
 
 double GMSH_AnnotatePlugin::callbackY(int num, int action, double value)
@@ -137,9 +138,9 @@ double GMSH_AnnotatePlugin::callbackY(int num, int action, double value)
   // not perfect: the change will only take place if we reopen the dialog...
   int dim3 = (int)AnnotateOptions_Number[3].def;
   return callback(num, action, value, &AnnotateOptions_Number[1].def,
-                  dim3 ? CTX.lc/200. : 0.5, 
-                  dim3 ? -CTX.lc : -100., 
-                  dim3 ? CTX.lc : 100000.);
+                  dim3 ? CTX::instance()->lc/200. : 0.5, 
+                  dim3 ? -CTX::instance()->lc : -100., 
+                  dim3 ? CTX::instance()->lc : 100000.);
 }
 
 double GMSH_AnnotatePlugin::callbackZ(int num, int action, double value)
@@ -147,9 +148,9 @@ double GMSH_AnnotatePlugin::callbackZ(int num, int action, double value)
   // not perfect: the change will only take place if we reopen the dialog...
   int dim3 = (int)AnnotateOptions_Number[3].def;
   return callback(num, action, value, &AnnotateOptions_Number[2].def,
-                  dim3 ? CTX.lc/200. : 0.5, 
-                  dim3 ? -CTX.lc : -100., 
-                  dim3 ? CTX.lc : 100000.);
+                  dim3 ? CTX::instance()->lc/200. : 0.5, 
+                  dim3 ? -CTX::instance()->lc : -100., 
+                  dim3 ? CTX::instance()->lc : 100000.);
 }
 
 double GMSH_AnnotatePlugin::callback3D(int num, int action, double value)
@@ -164,19 +165,19 @@ double GMSH_AnnotatePlugin::callbackFontSize(int num, int action, double value)
                   1, 5, 100);
 }
 
-const char *GMSH_AnnotatePlugin::callbackText(int num, int action, const char *value)
+std::string GMSH_AnnotatePlugin::callbackText(int num, int action, std::string value)
 {
-  return callbackStr(num, action, value, &AnnotateOptions_String[0].def);
+  return callbackStr(num, action, value, AnnotateOptions_String[0].def);
 }
 
-const char *GMSH_AnnotatePlugin::callbackFont(int num, int action, const char *value)
+std::string GMSH_AnnotatePlugin::callbackFont(int num, int action, std::string value)
 {
-  return callbackStr(num, action, value, &AnnotateOptions_String[1].def);
+  return callbackStr(num, action, value, AnnotateOptions_String[1].def);
 }
 
-const char *GMSH_AnnotatePlugin::callbackAlign(int num, int action, const char *value)
+std::string GMSH_AnnotatePlugin::callbackAlign(int num, int action, std::string value)
 {
-  return callbackStr(num, action, value, &AnnotateOptions_String[2].def);
+  return callbackStr(num, action, value, AnnotateOptions_String[2].def);
 }
 
 void GMSH_AnnotatePlugin::getName(char *name) const
@@ -187,8 +188,8 @@ void GMSH_AnnotatePlugin::getName(char *name) const
 void GMSH_AnnotatePlugin::getInfos(char *author, char *copyright,
                                    char *help_text) const
 {
-  strcpy(author, "C. Geuzaine");
-  strcpy(copyright, "DGR (www.multiphysics.com)");
+  strcpy(author, "C. Geuzaine, J.-F. Remacle");
+  strcpy(copyright, "C. Geuzaine, J.-F. Remacle");
   strcpy(help_text,
          "Plugin(Annotate) adds the text string `Text',\n"
          "in font `Font' and size `FontSize', in the view\n"
@@ -235,7 +236,7 @@ PView *GMSH_AnnotatePlugin::execute(PView *v)
   double Z = AnnotateOptions_Number[2].def;
   int dim3 = (int)AnnotateOptions_Number[3].def;
   int iView = (int)AnnotateOptions_Number[5].def;
-  const char *text = AnnotateOptions_String[0].def;
+  const char *text = AnnotateOptions_String[0].def.c_str();
   double style = getStyle();
 
   PView *v1 = getView(iView, v);
