@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -6,6 +6,7 @@
 #ifndef _ANNOTATE_H_
 #define _ANNOTATE_H_
 
+#include <string>
 #include "Plugin.h"
 
 extern "C"
@@ -13,12 +14,13 @@ extern "C"
   GMSH_Plugin *GMSH_RegisterAnnotatePlugin();
 }
 
-class GMSH_AnnotatePlugin : public GMSH_Post_Plugin
+class GMSH_AnnotatePlugin : public GMSH_PostPlugin
 {
 private:
   static double callback(int num, int action, double value, double *opt,
                          double step, double min, double max);
-  static const char *callbackStr(int num, int action, const char *value, const char **opt);
+  static std::string callbackStr(int num, int action, std::string value,
+                                 std::string &opt);
 public:
   GMSH_AnnotatePlugin(){}
   void getName(char *name) const;
@@ -35,10 +37,10 @@ public:
   static double callbackZ(int, int, double);
   static double callback3D(int, int, double);
   static double callbackFontSize(int, int, double);
-  static const char *callbackText(int, int, const char *);
-  static const char *callbackFont(int, int, const char *);
-  static const char *callbackAlign(int, int, const char *);
-  static void draw();
+  static std::string callbackText(int, int, std::string);
+  static std::string callbackFont(int, int, std::string);
+  static std::string callbackAlign(int, int, std::string);
+  static void draw(void *context);
 };
 
 #endif
