@@ -1,15 +1,17 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
 
 #include <math.h>
+#include "GmshConfig.h"
 #include "OctreePost.h"
 #include "CutParametric.h"
 #include "Context.h"
 
 #if defined(HAVE_FLTK)
-#include "GmshUI.h"
+#include <FL/gl.h>
+#include "drawContext.h"
 #include "Draw.h"
 #endif
 
@@ -17,8 +19,6 @@
 #include "matheval.h"
 #endif
 
-extern Context_T CTX;
-
 StringXNumber CutParametricOptions_Number[] = {
   {GMSH_FULLRC, "MinU", GMSH_CutParametricPlugin::callbackMinU, 0.},
   {GMSH_FULLRC, "MaxU", GMSH_CutParametricPlugin::callbackMaxU, 2*3.1416},
@@ -64,9 +64,9 @@ int GMSH_CutParametricPlugin::fillXYZ()
   Msg::Error("MathEval is not compiled in this version of Gmsh");
   return 0;
 #else
-  const char *exprx = CutParametricOptions_String[0].def;
-  const char *expry = CutParametricOptions_String[1].def;
-  const char *exprz = CutParametricOptions_String[2].def;
+  const char *exprx = CutParametricOptions_String[0].def.c_str();
+  const char *expry = CutParametricOptions_String[1].def.c_str();
+  const char *exprz = CutParametricOptions_String[2].def.c_str();
   int nbU = (int)CutParametricOptions_Number[2].def;
 
   x.resize(nbU);
@@ -101,14 +101,14 @@ int GMSH_CutParametricPlugin::fillXYZ()
 #endif
 }
 
-void GMSH_CutParametricPlugin::draw()
+void GMSH_CutParametricPlugin::draw(void *context)
 {
 #if defined(HAVE_FLTK)
   if(recompute){
     fillXYZ();
     recompute = 0;
   }
-  glColor4ubv((GLubyte *) & CTX.color.fg);
+  glColor4ubv((GLubyte *) & CTX::instance()->color.fg);
   if(CutParametricOptions_Number[3].def && x.size() > 1){
     glBegin(GL_LINES);
     for(unsigned int i = 1; i < x.size(); ++i){
@@ -118,8 +118,9 @@ void GMSH_CutParametricPlugin::draw()
     glEnd();
   }
   else{
+    drawContext *ctx = (drawContext*)context;
     for(unsigned int i = 0; i < x.size(); ++i)
-      Draw_Sphere(CTX.point_size, x[i], y[i], z[i], 1);
+      ctx->drawSphere(CTX::instance()->pointSize, x[i], y[i], z[i], 1);
   }
 #endif
 }
@@ -141,15 +142,15 @@ double GMSH_CutParametricPlugin::callback(int num, int action, double value, dou
   return 0.;
 }
 
-const char *GMSH_CutParametricPlugin::callbackStr(int num, int action, const char *value,
-                                                  const char **opt)
+std::string GMSH_CutParametricPlugin::callbackStr(int num, int action, std::string value,
+                                                  std::string &opt)
 {
-  *opt = value;
+  opt = value;
 #if defined(HAVE_FLTK)
   recompute = 1;
   DrawPlugin(draw);
 #endif
-  return NULL;
+  return opt;
 }
 
 double GMSH_CutParametricPlugin::callbackMinU(int num, int action, double value)
@@ -176,19 +177,19 @@ double GMSH_CutParametricPlugin::callbackConnect(int num, int action, double val
                   1, 0, 1);
 }
 
-const char *GMSH_CutParametricPlugin::callbackX(int num, int action, const char *value)
+std::string GMSH_CutParametricPlugin::callbackX(int num, int action, std::string value)
 {
-  return callbackStr(num, action, value, &CutParametricOptions_String[0].def);
+  return callbackStr(num, action, value, CutParametricOptions_String[0].def);
 }
 
-const char *GMSH_CutParametricPlugin::callbackY(int num, int action, const char *value)
+std::string GMSH_CutParametricPlugin::callbackY(int num, int action, std::string value)
 {
-  return callbackStr(num, action, value, &CutParametricOptions_String[1].def);
+  return callbackStr(num, action, value, CutParametricOptions_String[1].def);
 }
 
-const char *GMSH_CutParametricPlugin::callbackZ(int num, int action, const char *value)
+std::string GMSH_CutParametricPlugin::callbackZ(int num, int action, std::string value)
 {
-  return callbackStr(num, action, value, &CutParametricOptions_String[2].def);
+  return callbackStr(num, action, value, CutParametricOptions_String[2].def);
 }
 
 void GMSH_CutParametricPlugin::getName(char *name) const
@@ -199,8 +200,8 @@ void GMSH_CutParametricPlugin::getName(char *name) const
 void GMSH_CutParametricPlugin::getInfos(char *author, char *copyright,
                                    char *help_text) const
 {
-  strcpy(author, "C. Geuzaine");
-  strcpy(copyright, "DGR (www.multiphysics.com)");
+  strcpy(author, "C. Geuzaine, J.-F. Remacle");
+  strcpy(copyright, "C. Geuzaine, J.-F. Remacle");
   strcpy(help_text,
          "Plugin(CutParametric) cuts the view `iView' with\n"
          "the parametric function (`X'(u), `Y'(u), `Z'(u)),\n"
