@@ -1,17 +1,21 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
 
+#include "GmshConfig.h"
 #include "CutPlane.h"
 #include "Context.h"
 
 #if defined(HAVE_FLTK)
-#include "GmshUI.h"
+#include "drawContext.h"
 #include "Draw.h"
 #endif
 
-extern Context_T CTX;
+#if defined(WIN32)
+#undef min
+#undef max
+#endif
 
 int GMSH_CutPlanePlugin::iview = 0;
 
@@ -34,21 +38,22 @@ extern "C"
   }
 }
 
-void GMSH_CutPlanePlugin::draw()
+void GMSH_CutPlanePlugin::draw(void *context)
 {
 #if defined(HAVE_FLTK)
   int num = (int)CutPlaneOptions_Number[7].def;
+  drawContext *ctx = (drawContext*)context;
   if(num < 0) num = iview;
   if(num >= 0 && num < (int)PView::list.size()){
-    glColor4ubv((GLubyte *) & CTX.color.fg);
-    glLineWidth(CTX.line_width);
+    glColor4ubv((GLubyte *) & CTX::instance()->color.fg);
+    glLineWidth(CTX::instance()->lineWidth);
     SBoundingBox3d bb = PView::list[num]->getData()->getBoundingBox();
-    Draw_PlaneInBoundingBox(bb.min().x(), bb.min().y(), bb.min().z(), 
-                            bb.max().x(), bb.max().y(), bb.max().z(), 
-                            CutPlaneOptions_Number[0].def,
-                            CutPlaneOptions_Number[1].def,
-                            CutPlaneOptions_Number[2].def,
-                            CutPlaneOptions_Number[3].def);
+    ctx->drawPlaneInBoundingBox(bb.min().x(), bb.min().y(), bb.min().z(), 
+                                bb.max().x(), bb.max().y(), bb.max().z(), 
+                                CutPlaneOptions_Number[0].def,
+                                CutPlaneOptions_Number[1].def,
+                                CutPlaneOptions_Number[2].def,
+                                CutPlaneOptions_Number[3].def);
   }
 #endif
 }
@@ -91,7 +96,7 @@ double GMSH_CutPlanePlugin::callbackC(int num, int action, double value)
 double GMSH_CutPlanePlugin::callbackD(int num, int action, double value)
 {
   return callback(num, action, value, &CutPlaneOptions_Number[3].def,
-                  CTX.lc/200., -CTX.lc, CTX.lc);
+                  CTX::instance()->lc/200., -CTX::instance()->lc, CTX::instance()->lc);
 }
 
 double GMSH_CutPlanePlugin::callbackVol(int num, int action, double value)
@@ -155,7 +160,7 @@ double GMSH_CutPlanePlugin::levelset(double x, double y, double z, double val) c
     CutPlaneOptions_Number[2].def * z + CutPlaneOptions_Number[3].def;
 }
 
-bool GMSH_CutPlanePlugin::geometricalFilter(Double_Matrix *node_positions) const
+bool GMSH_CutPlanePlugin::geometricalFilter(gmshMatrix<double> *node_positions) const
 {
   const double l0 = levelset((*node_positions)(0, 0),
                              (*node_positions)(0, 1),
