@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -27,7 +27,7 @@ public:
   int getNbOptions() const;
   StringXNumber *getOption(int iopt);  
   PView *execute(PView *);
-  virtual bool geometricalFilter(Double_Matrix *) const;
+  virtual bool geometricalFilter(gmshMatrix<double> *) const;
 
   static double callbackA(int, int, double);
   static double callbackB(int, int, double);
@@ -36,7 +36,7 @@ public:
   static double callbackVol(int, int, double);
   static double callbackRecur(int, int, double);
   static double callbackTarget(int, int, double);
-  static void draw();
+  static void draw(void *context);
 };
 
 #endif
