@@ -1,19 +1,18 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
 
 #include <string.h>
+#include "GmshConfig.h"
 #include "CutSphere.h"
 #include "Context.h"
 
 #if defined(HAVE_FLTK)
-#include "GmshUI.h"
+#include "drawContext.h"
 #include "Draw.h"
 #endif
 
-extern Context_T CTX;
-
 StringXNumber CutSphereOptions_Number[] = {
   {GMSH_FULLRC, "Xc", GMSH_CutSpherePlugin::callbackX, 0.},
   {GMSH_FULLRC, "Yc", GMSH_CutSpherePlugin::callbackY, 0.},
@@ -32,26 +31,19 @@ extern "C"
   }
 }
 
-void GMSH_CutSpherePlugin::draw()
+void GMSH_CutSpherePlugin::draw(void *context)
 {
 #if defined(HAVE_FLTK)
-  static GLUquadricObj *qua;
-  static int first = 1;
-  if(first) {
-    first = 0;
-    qua = gluNewQuadric();
-  }
   GLint mode[2];
   glGetIntegerv(GL_POLYGON_MODE, mode);
   glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
-  glPushMatrix();
-  glColor4ubv((GLubyte *) & CTX.color.fg);
-  glLineWidth(CTX.line_width);
-  glTranslated(CutSphereOptions_Number[0].def,
-               CutSphereOptions_Number[1].def,
-               CutSphereOptions_Number[2].def);
-  gluSphere(qua, CutSphereOptions_Number[3].def, 40, 40);
-  glPopMatrix();
+  glColor4ubv((GLubyte *) & CTX::instance()->color.fg);
+  glLineWidth(CTX::instance()->lineWidth);
+  drawContext *ctx = (drawContext*)context;
+  ctx->drawSphere(CutSphereOptions_Number[3].def,
+                  CutSphereOptions_Number[0].def,
+                  CutSphereOptions_Number[1].def,
+                  CutSphereOptions_Number[2].def, 40, 40, 1);
   glPolygonMode(GL_FRONT_AND_BACK, mode[1]);
 #endif
 }
@@ -75,25 +67,25 @@ double GMSH_CutSpherePlugin::callback(int num, int action, double value, double
 double GMSH_CutSpherePlugin::callbackX(int num, int action, double value)
 {
   return callback(num, action, value, &CutSphereOptions_Number[0].def,
-                  CTX.lc/100., -2*CTX.lc, 2*CTX.lc);
+                  CTX::instance()->lc/100., -2*CTX::instance()->lc, 2*CTX::instance()->lc);
 }
 
 double GMSH_CutSpherePlugin::callbackY(int num, int action, double value)
 {
   return callback(num, action, value, &CutSphereOptions_Number[1].def,
-                  CTX.lc/100., -2*CTX.lc, 2*CTX.lc);
+                  CTX::instance()->lc/100., -2*CTX::instance()->lc, 2*CTX::instance()->lc);
 }
 
 double GMSH_CutSpherePlugin::callbackZ(int num, int action, double value)
 {
   return callback(num, action, value, &CutSphereOptions_Number[2].def,
-                  CTX.lc/100., -2*CTX.lc, 2*CTX.lc);
+                  CTX::instance()->lc/100., -2*CTX::instance()->lc, 2*CTX::instance()->lc);
 }
 
 double GMSH_CutSpherePlugin::callbackR(int num, int action, double value)
 {
   return callback(num, action, value, &CutSphereOptions_Number[3].def,
-                  CTX.lc/100., 0., 2*CTX.lc);
+                  CTX::instance()->lc/100., 0., 2*CTX::instance()->lc);
 }
 
 double GMSH_CutSpherePlugin::callbackVol(int num, int action, double value)
