@@ -1,8 +1,9 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
 
+#include "GmshConfig.h"
 #include "Evaluate.h"
 
 #if defined(HAVE_MATH_EVAL)
@@ -162,6 +163,7 @@ void GMSH_EvaluatePlugin::evaluate(PView *v1, List_T *list1, int nbElm1,
         double tmp[9];
         val2 = tmp;
         if(_octree->searchScalar(x[j], y[j], z[j], val2, timeStep2)){
+        //if(_octree->searchScalarWithTol(x[j], y[j], z[j], val2, timeStep2, 0, 0.1)){
           w[0] = val2[0];
         }
         else if(_octree->searchVector(x[j], y[j], z[j], val2, timeStep2)){
@@ -201,7 +203,7 @@ PView *GMSH_EvaluatePlugin::execute(PView *v)
   int externalView = (int)EvaluateOptions_Number[2].def;
   int externalTimeStep = (int)EvaluateOptions_Number[3].def;
   int iView = (int)EvaluateOptions_Number[4].def;
-  const char *expr = EvaluateOptions_String[0].def;
+  const char *expr = EvaluateOptions_String[0].def.c_str();
 
   PView *v1 = getView(iView, v);
   if(!v1) return v;
