@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -14,7 +14,7 @@ extern "C"
   GMSH_Plugin *GMSH_RegisterEvaluatePlugin();
 }
 
-class GMSH_EvaluatePlugin : public GMSH_Post_Plugin
+class GMSH_EvaluatePlugin : public GMSH_PostPlugin
 {
 private:
   OctreePost *_octree;
