@@ -1,8 +1,9 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
 
+#include "GmshConfig.h"
 #include "Extract.h"
 
 #if defined(HAVE_MATH_EVAL)
@@ -205,15 +206,15 @@ PView *GMSH_ExtractPlugin::execute(PView *v)
 {
   int step = (int)ExtractOptions_Number[0].def;
   int iView = (int)ExtractOptions_Number[1].def;
-  const char *expr[9] = { ExtractOptions_String[0].def, 
-                          ExtractOptions_String[1].def,
-                          ExtractOptions_String[2].def,
-                          ExtractOptions_String[3].def,
-                          ExtractOptions_String[4].def,
-                          ExtractOptions_String[5].def,
-                          ExtractOptions_String[6].def,
-                          ExtractOptions_String[7].def,
-                          ExtractOptions_String[8].def };
+  const char *expr[9] = { ExtractOptions_String[0].def.c_str(), 
+                          ExtractOptions_String[1].def.c_str(),
+                          ExtractOptions_String[2].def.c_str(),
+                          ExtractOptions_String[3].def.c_str(),
+                          ExtractOptions_String[4].def.c_str(),
+                          ExtractOptions_String[5].def.c_str(),
+                          ExtractOptions_String[6].def.c_str(),
+                          ExtractOptions_String[7].def.c_str(),
+                          ExtractOptions_String[8].def.c_str() };
 
   PView *v1 = getView(iView, v);
   if(!v1) return v;
@@ -221,7 +222,7 @@ PView *GMSH_ExtractPlugin::execute(PView *v)
   PViewDataList *data1 = getDataList(v1);
   if(!data1) return v;
 
-  PView *v2 = new PView(true);
+  PView *v2 = new PView(true, data1->getNumElements());
 
   PViewDataList *data2 = getDataList(v2);
   if(!data2) return v;
