@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -31,8 +31,8 @@ void GMSH_ExtractEdgesPlugin::getName(char *name) const
 
 void GMSH_ExtractEdgesPlugin::getInfos(char *author, char *copyright, char *help_text) const
 {
-  strcpy(author, "C. Geuzaine (geuz@geuz.org)");
-  strcpy(copyright, "DGR (www.multiphysics.com)");
+  strcpy(author, "C. Geuzaine, J.-F. Remacle");
+  strcpy(copyright, "C. Geuzaine, J.-F. Remacle");
   strcpy(help_text,
          "Plugin(ExtractEdges) extracts the geometry edges\n"
          "from the surface view `iView', using `Angle' as\n"
@@ -74,7 +74,7 @@ PView *GMSH_ExtractEdgesPlugin::execute(PView *v)
   if(!data2) return v;
 
   BDS_Mesh bds;
-  //bds.import_view(v1, CTX.lc * 1.e-12);
+  //bds.import_view(v1, CTX::instance()->lc * 1.e-12);
   //bds.classify(angle * M_PI / 180.);
 
   Msg::Error("BDS->classify(angle, edge_prolongation) must be reinterfaced");
@@ -95,7 +95,6 @@ PView *GMSH_ExtractEdgesPlugin::execute(PView *v)
     ++it;
   }
 
-
   data2->setName(data1->getName() + "_ExtractEdges");
   data2->setFileName(data1->getName() + "_ExtractEdges.pos");
   data2->finalize();
