@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -32,8 +32,8 @@ void GMSH_IntegratePlugin::getName(char *name) const
 void GMSH_IntegratePlugin::getInfos(char *author, char *copyright,
                                     char *help_text) const
 {
-  strcpy(author, "C. Geuzaine");
-  strcpy(copyright, "DGR (www.multiphysics.com)");
+  strcpy(author, "C. Geuzaine, J.-F. Remacle");
+  strcpy(copyright, "C. Geuzaine, J.-F. Remacle");
   strcpy(help_text,
          "Plugin(Integrate) integrates scalar fields over\n"
          "all the elements in the view `iView', as well\n"
@@ -111,7 +111,7 @@ PView *GMSH_IntegratePlugin::execute(PView * v)
     List_Add(data2->SP, &res);
   }
   data2->NbSP = 1;
-  v2->getOptions()->IntervalsType = PViewOptions::Numeric;
+  v2->getOptions()->intervalsType = PViewOptions::Numeric;
   
   for(int i = 0; i < data1->getNumTimeSteps(); i++){
     double time = data1->getTime(i);
