@@ -1,11 +1,11 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
 
 #include "Plugin.h"
 
-PView *GMSH_Post_Plugin::getView(int index, PView *view)
+PView *GMSH_PostPlugin::getView(int index, PView *view)
 {
   if(index < 0)
     index = view ? view->getIndex() : 0;
@@ -19,7 +19,7 @@ PView *GMSH_Post_Plugin::getView(int index, PView *view)
   }
 }
 
-PViewDataList *GMSH_Post_Plugin::getDataList(PView *view)
+PViewDataList *GMSH_PostPlugin::getDataList(PView *view)
 {
   if(!view) return 0;
 
