@@ -1,18 +1,23 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
 
+#include "GmshConfig.h"
 #include "Probe.h"
 #include "Context.h"
 #include "OctreePost.h"
 
 #if defined(HAVE_FLTK)
-#include "GmshUI.h"
+#include <FL/gl.h>
+#include "drawContext.h"
 #include "Draw.h"
 #endif
 
-extern Context_T CTX;
+#if defined(WIN32)
+#undef min
+#undef max
+#endif
 
 int GMSH_ProbePlugin::iview = 0;
 
@@ -31,7 +36,7 @@ extern "C"
   }
 }
 
-void GMSH_ProbePlugin::draw()
+void GMSH_ProbePlugin::draw(void *context)
 {
 #if defined(HAVE_FLTK)
   int num = (int)ProbeOptions_Number[3].def;
@@ -40,8 +45,9 @@ void GMSH_ProbePlugin::draw()
     double x = ProbeOptions_Number[0].def;
     double y = ProbeOptions_Number[1].def;
     double z = ProbeOptions_Number[2].def;
-    glColor4ubv((GLubyte *) & CTX.color.fg);
-    glLineWidth(CTX.line_width);
+    drawContext *ctx = (drawContext*)context;
+    glColor4ubv((GLubyte *) & CTX::instance()->color.fg);
+    glLineWidth(CTX::instance()->lineWidth);
     SBoundingBox3d bb = PView::list[num]->getData()->getBoundingBox();
     if(x >= bb.min().x() && x <= bb.max().x() &&
        y >= bb.min().y() && y <= bb.max().y() &&
@@ -55,14 +61,14 @@ void GMSH_ProbePlugin::draw()
     }
     else{
       // draw 10-pixel marker
-      double d = 10 * CTX.pixel_equiv_x / CTX.s[0];
+      double d = 10 * ctx->pixel_equiv_x / ctx->s[0];
       glBegin(GL_LINES);
       glVertex3d(x - d, y, z); glVertex3d(x + d, y, z);
       glVertex3d(x, y - d, z); glVertex3d(x, y + d, z);
       glVertex3d(x, y, z - d); glVertex3d(x, y, z + d);
       glEnd();
     }
-    Draw_Sphere(CTX.point_size, x, y, z, 1);
+    ctx->drawSphere(CTX::instance()->pointSize, x, y, z, 1);
   }
 #endif
 }
@@ -71,9 +77,9 @@ double GMSH_ProbePlugin::callback(int num, int action, double value, double *opt
 {
   if(action > 0) iview = num;
   switch(action){ // configure the input field
-  case 1: return CTX.lc / 100.;
-  case 2: return -2 * CTX.lc;
-  case 3: return 2 * CTX.lc;
+  case 1: return CTX::instance()->lc / 100.;
+  case 2: return -2 * CTX::instance()->lc;
+  case 3: return 2 * CTX::instance()->lc;
   default: break;
   }
   *opt = value;
