@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -13,7 +13,7 @@ extern "C"
   GMSH_Plugin *GMSH_RegisterProbePlugin();
 }
 
-class GMSH_ProbePlugin : public GMSH_Post_Plugin
+class GMSH_ProbePlugin : public GMSH_PostPlugin
 {
   double levelset(double x, double y, double z, double val) const;
   static double callback(int num, int action, double value, double *opt);
@@ -30,7 +30,7 @@ public:
   static double callbackX(int, int, double);
   static double callbackY(int, int, double);
   static double callbackZ(int, int, double);
-  static void draw();
+  static void draw(void *context);
 };
 
 #endif
