@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -13,7 +13,7 @@ extern "C"
   GMSH_Plugin *GMSH_RegisterStreamLinesPlugin ();
 }
 
-class GMSH_StreamLinesPlugin : public GMSH_Post_Plugin
+class GMSH_StreamLinesPlugin : public GMSH_PostPlugin
 {
   static double callback(int num, int action, double value, double *opt,
                          double step, double min, double max);
@@ -41,7 +41,7 @@ public:
   static double callbackZ2(int, int, double);
   static double callbackU(int, int, double);
   static double callbackV(int, int, double);
-  static void draw();
+  static void draw(void *context);
 };
 
 #endif
