@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -13,7 +13,7 @@ extern "C"
   GMSH_Plugin *GMSH_RegisterTransformPlugin();
 }
 
-class GMSH_TransformPlugin : public GMSH_Post_Plugin
+class GMSH_TransformPlugin : public GMSH_PostPlugin
 {
 public:
   GMSH_TransformPlugin(){}
