@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -12,8 +12,6 @@
 #include "Triangulate.h"
 #include "Context.h"
 
-extern Context_T CTX;
-
 StringXNumber TriangulateOptions_Number[] = {
   {GMSH_FULLRC, "iView", NULL, -1.}
 };
@@ -98,8 +96,8 @@ static void Triangulate(int nbIn, List_T *inList, int *nbOut, List_T *outList,
   // build a point record structure for the divide and conquer algorithm
   DocRecord doc(points.size());
   for(unsigned int i = 0; i < points.size(); i++){
-    double XX = CTX.mesh.rand_factor * lc * (double)rand() / (double)RAND_MAX;
-    double YY = CTX.mesh.rand_factor * lc * (double)rand() / (double)RAND_MAX;
+    double XX = CTX::instance()->mesh.randFactor * lc * (double)rand() / (double)RAND_MAX;
+    double YY = CTX::instance()->mesh.randFactor * lc * (double)rand() / (double)RAND_MAX;
     doc.points[i].where.h = points[i]->x() + XX;
     doc.points[i].where.v = points[i]->y() + YY;
     doc.points[i].adjacent = NULL;
@@ -139,7 +137,7 @@ PView *GMSH_TriangulatePlugin::execute(PView *v)
   PViewDataList *data1 = getDataList(v1);
   if(!data1) return v;
 
-  PView *v2 = new PView(true);
+  PView *v2 = new PView(true, data1->getNumElements());
 
   PViewDataList *data2 = getDataList(v2);
   if(!data2) return v;
