@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -10,7 +10,7 @@
 #include "PViewDataList.h"
 #include "PViewDataGModel.h"
 #include "Numeric.h"
-#include "Message.h"
+#include "GmshMessage.h"
 #include "shapeFunctions.h"
 #include "GModel.h"
 #include "MElement.h"
@@ -26,7 +26,6 @@ static void minmax(int n, double *X, double *Y, double *Z,
   max[0] = X[0];
   max[1] = Y[0];
   max[2] = Z[0];
-
   for(int i = 1; i < n; i++) {
     min[0] = (X[i] < min[0]) ? X[i] : min[0];
     min[1] = (Y[i] < min[1]) ? Y[i] : min[1];
@@ -222,12 +221,13 @@ OctreePost::OctreePost(PView *v)
 {
   _theViewDataGModel = dynamic_cast<PViewDataGModel*>(_theView->getData());
 
-  if(_theViewDataGModel) return; // the octree is available in the model
+  if(_theViewDataGModel) return; // the octree is already available in the model
 
-  _theViewDataList = dynamic_cast<PViewDataList*>(_theView->getData());
+  // use adaptive data if available
+  _theViewDataList = dynamic_cast<PViewDataList*>(_theView->getData(true));
 
   if(_theViewDataList){
-    SBoundingBox3d bb = v->getData()->getBoundingBox();
+    SBoundingBox3d bb = _theViewDataList->getBoundingBox();
     double min[3] = {bb.min().x(), bb.min().y(), bb.min().z()};
     double size[3] = {bb.max().x() - bb.min().x(),
 		      bb.max().y() - bb.min().y(),
@@ -369,7 +369,7 @@ bool OctreePost::_getValue(void *in, int nbComp, double P[3], int timestep,
         }
       }
       for(int comp = 0; comp < nbComp; comp++){
-        double val = e->interpolate(nodeval, U[0], U[1], U[2], nbComp);
+        double val = e->interpolate(&nodeval[comp], U[0], U[1], U[2], nbComp);
         if(timestep < 0)
           values[nbComp * step + comp] = val;
         else
@@ -413,6 +413,22 @@ bool OctreePost::searchScalar(double x, double y, double z, double *values,
   return false;
 }
 
+bool OctreePost::searchScalarWithTol(double x, double y, double z, double *values, 
+                                     int step, double *size, double tol)
+{
+  bool a = searchScalar(x, y, z, values, step, size);
+  if(!a && tol != 0.){
+    double oldtol1 = element::getTolerance();
+    double oldtol2 = MElement::getTolerance();
+    element::setTolerance(tol);
+    MElement::setTolerance(tol);
+    a = searchScalar(x, y, z, values, step, size);
+    element::setTolerance(oldtol1);
+    MElement::setTolerance(oldtol2);
+  }    
+  return a;
+}
+
 bool OctreePost::searchVector(double x, double y, double z, double *values, 
                               int step, double *size)
 {
