@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -11,8 +11,6 @@
 #include "Context.h"
 #include "adaptiveData.h"
 
-extern Context_T CTX;
-
 bool PViewDataList::readPOS(FILE *fp, double version, int format, int size)
 {
   char name[256];
@@ -318,17 +316,16 @@ bool PViewDataList::writePOS(std::string fileName, bool binary, bool parsed, boo
     fprintf(fp, "$EndPostFormat\n");
   }
 
-  char str[256];
-  strcpy(str, getName().c_str());
-  for(int i = 0; i < (int)strlen(str); i++)
+  std::string str = getName();
+  for(unsigned int i = 0; i < str.size(); i++)
     if(str[i] == ' ') str[i] = '^';
 
   if(!parsed){
     fprintf(fp, "$View /* %s */\n", getName().c_str());
-    if (strlen(str) == 0)
+    if (str.empty())
       fprintf(fp, "noname ");
     else
-      fprintf(fp, "%s ", str);
+      fprintf(fp, "%s ", str.c_str());
     fprintf(fp, "%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d "
             "%d %d %d %d %d %d %d %d %d %d %d %d\n",
             List_Nbr(Time),
@@ -399,7 +396,7 @@ class pVertexLessThan{
  public:
   bool operator()(const pVertex ent1, const pVertex ent2) const
   {
-    double tol = CTX.lc * 1.e-10 ;
+    double tol = CTX::instance()->lc * 1.e-10 ;
     if(ent1.X - ent2.X  >  tol) return true;
     if(ent1.X - ent2.X  < -tol) return false;
     if(ent1.Y - ent2.Y  >  tol) return true;
