@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2008 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2009 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <gmsh@geuz.org>.
@@ -8,22 +8,20 @@
 
 #include <list>
 #include <set>
-#include "ListUtils.h"
+#include <vector>
 #include "GmshMatrix.h"
 
 class PViewData;
 class PViewDataList;
-class GMSH_Post_Plugin;
+class GMSH_PostPlugin;
 
 class adaptivePoint {
  public:
   double x, y, z, X, Y, Z;
   double val, valx, valy, valz;
-  double shapeFunctions[128];
-  static std::set<adaptivePoint> all;
  public:
-  static adaptivePoint *create(double x, double y, double z, 
-			       Double_Matrix *coeffs, Double_Matrix *eexps);
+  static adaptivePoint *add(double x, double y, double z, 
+                            std::set<adaptivePoint> &allPoints);
   bool operator < (const adaptivePoint &other) const
   {
     if(other.x < x) return true;
@@ -41,6 +39,7 @@ class adaptiveLine {
   adaptivePoint *p[2];
   adaptiveLine *e[2];
   static std::list<adaptiveLine*> all;
+  static std::set<adaptivePoint> allPoints;
   static int numNodes, numEdges;
  public:
   adaptiveLine(adaptivePoint *p1, adaptivePoint *p2)
@@ -54,14 +53,13 @@ class adaptiveLine {
   {
     return (p[0]->val + p[1]->val) / 2.;
   }
-  inline static void GSF(double u, double v, double w, double sf[])
+  inline static void GSF(double u, double v, double w, gmshVector<double> &sf)
   {
-    sf[0] = (1 - u) / 2.;
-    sf[1] = (1 + u) / 2.;
+    sf(0) = (1 - u) / 2.;
+    sf(1) = (1 + u) / 2.;
   }
-  static void create(int maxlevel, Double_Matrix *coeffs, Double_Matrix *eexps);
-  static void recurCreate(adaptiveLine *e, int maxlevel, int level, 
-			  Double_Matrix *coeffs, Double_Matrix *eexps);
+  static void create(int maxlevel);
+  static void recurCreate(adaptiveLine *e, int maxlevel, int level);
   static void error(double AVG, double tol);
   static void recurError(adaptiveLine *e, double AVG, double tol);
 };
@@ -72,6 +70,7 @@ class adaptiveTriangle {
   adaptivePoint *p[3];
   adaptiveTriangle *e[4];
   static std::list<adaptiveTriangle*> all;
+  static std::set<adaptivePoint> allPoints;
   static int numNodes, numEdges;
  public:
   adaptiveTriangle(adaptivePoint *p1, adaptivePoint *p2, adaptivePoint *p3)
@@ -86,15 +85,14 @@ class adaptiveTriangle {
   {
     return (p[0]->val + p[1]->val + p[2]->val) / 3.;
   }
-  inline static void GSF(double u, double v, double w, double sf[])
+  inline static void GSF(double u, double v, double w, gmshVector<double> &sf)
   {
-    sf[0] = 1. - u - v;
-    sf[1] = u;
-    sf[2] = v;
+    sf(0) = 1. - u - v;
+    sf(1) = u;
+    sf(2) = v;
   }
-  static void create(int maxlevel, Double_Matrix *coeffs, Double_Matrix *eexps);
-  static void recurCreate(adaptiveTriangle *t, int maxlevel, int level,
-			  Double_Matrix *coeffs, Double_Matrix *eexps);
+  static void create(int maxlevel);
+  static void recurCreate(adaptiveTriangle *t, int maxlevel, int level);
   static void error(double AVG, double tol);
   static void recurError(adaptiveTriangle *t, double AVG, double tol);
 };
@@ -105,6 +103,7 @@ class adaptiveQuadrangle {
   adaptivePoint *p[4];
   adaptiveQuadrangle *e[4];
   static std::list<adaptiveQuadrangle*> all;
+  static std::set<adaptivePoint> allPoints;
   static int numNodes, numEdges;
  public:
   adaptiveQuadrangle(adaptivePoint *p1, adaptivePoint *p2, 
@@ -121,16 +120,15 @@ class adaptiveQuadrangle {
   {
     return (p[0]->val + p[1]->val + p[2]->val + p[3]->val) / 4.;
   }
-  inline static void GSF(double u, double v, double w, double sf[])
+  inline static void GSF(double u, double v, double w, gmshVector<double> &sf)
   {
-    sf[0] = 0.25 * (1. - u) * (1. - v);
-    sf[1] = 0.25 * (1. + u) * (1. - v);
-    sf[2] = 0.25 * (1. + u) * (1. + v);
-    sf[3] = 0.25 * (1. - u) * (1. + v);
+    sf(0) = 0.25 * (1. - u) * (1. - v);
+    sf(1) = 0.25 * (1. + u) * (1. - v);
+    sf(2) = 0.25 * (1. + u) * (1. + v);
+    sf(3) = 0.25 * (1. - u) * (1. + v);
   }
-  static void create(int maxlevel, Double_Matrix *coeffs, Double_Matrix *eexps);
-  static void recurCreate(adaptiveQuadrangle *q, int maxlevel, int level,
-			  Double_Matrix *coeffs, Double_Matrix *eexps);
+  static void create(int maxlevel);
+  static void recurCreate(adaptiveQuadrangle *q, int maxlevel, int level);
   static void error(double AVG, double tol);
   static void recurError(adaptiveQuadrangle *q, double AVG, double tol);
 };
@@ -141,6 +139,7 @@ class adaptivePrism {
   adaptivePoint *p[6];
   adaptivePrism *e[12];
   static std::list<adaptivePrism*> all;
+  static std::set<adaptivePoint> allPoints;
   static int numNodes, numEdges;
  public:
   adaptivePrism(adaptivePoint *p1, adaptivePoint *p2, 
@@ -162,18 +161,17 @@ class adaptivePrism {
   {
     return (p[0]->val + p[1]->val + p[2]->val + p[3]->val + p[4]->val + p[5]->val) / 6.;
   }
-  inline static void GSF(double u, double v, double w, double sf[])
+  inline static void GSF(double u, double v, double w, gmshVector<double> &sf)
   {
-    sf[0] = (1. - u - v) * (1 - w) / 2;
-    sf[1] = u * (1-w)/2;
-    sf[2] = v*(1-w)/2;
-    sf[3] = (1. - u - v)*(1+w)/2;
-    sf[4] = u*(1+w)/2;
-    sf[5] = v*(1+w)/2;
+    sf(0) = (1. - u - v) * (1 - w) / 2;
+    sf(1) = u * (1-w)/2;
+    sf(2) = v*(1-w)/2;
+    sf(3) = (1. - u - v)*(1+w)/2;
+    sf(4) = u*(1+w)/2;
+    sf(5) = v*(1+w)/2;
   }
-  static void create(int maxlevel, Double_Matrix *coeffs, Double_Matrix *eexps);
-  static void recurCreate(adaptivePrism *p, int maxlevel, int level, 
-			  Double_Matrix *coeffs, Double_Matrix *eexps);
+  static void create(int maxlevel);
+  static void recurCreate(adaptivePrism *p, int maxlevel, int level);
   static void error(double AVG, double tol);
   static void recurError(adaptivePrism *p, double AVG, double tol);
 };
@@ -184,6 +182,7 @@ class adaptiveTetrahedron {
   adaptivePoint *p[4];
   adaptiveTetrahedron *e[8];
   static std::list<adaptiveTetrahedron*> all;
+  static std::set<adaptivePoint> allPoints;
   static int numNodes, numEdges;
  public:
   adaptiveTetrahedron(adaptivePoint *p1, adaptivePoint *p2, 
@@ -201,16 +200,15 @@ class adaptiveTetrahedron {
   {
     return (p[0]->val + p[1]->val + p[2]->val + p[3]->val) / 4.;
   }
-  inline static void GSF(double u, double v, double w, double sf[])
+  inline static void GSF(double u, double v, double w, gmshVector<double> &sf)
   {
-    sf[0] = 1. - u - v - w;
-    sf[1] = u;
-    sf[2] = v;
-    sf[3] = w;
+    sf(0) = 1. - u - v - w;
+    sf(1) = u;
+    sf(2) = v;
+    sf(3) = w;
   }
-  static void create(int maxlevel, Double_Matrix *coeffs, Double_Matrix *eexps);
-  static void recurCreate(adaptiveTetrahedron *t, int maxlevel, int level, 
-			  Double_Matrix *coeffs, Double_Matrix *eexps);
+  static void create(int maxlevel);
+  static void recurCreate(adaptiveTetrahedron *t, int maxlevel, int level);
   static void error(double AVG, double tol);
   static void recurError(adaptiveTetrahedron *t, double AVG, double tol);
 };
@@ -221,6 +219,7 @@ class adaptiveHexahedron {
   adaptivePoint *p[8];
   adaptiveHexahedron *e[8];
   static std::list<adaptiveHexahedron*> all;
+  static std::set<adaptivePoint> allPoints;
   static int numNodes, numEdges;
  public:
   adaptiveHexahedron(adaptivePoint *p1, adaptivePoint *p2, adaptivePoint *p3, 
@@ -244,51 +243,73 @@ class adaptiveHexahedron {
     return (p[0]->val + p[1]->val + p[2]->val+ p[3]->val +
             p[4]->val + p[5]->val + p[6]->val+ p[7]->val) / 8.;
   }
-  inline static void GSF(double u, double v, double w, double sf[])
+  inline static void GSF(double u, double v, double w, gmshVector<double> &sf)
   {
-    sf[0] = 0.125 * (1 - u) * (1 - v) * (1 - w);
-    sf[1] = 0.125 * (1 + u) * (1 - v) * (1 - w);
-    sf[2] = 0.125 * (1 + u) * (1 + v) * (1 - w);
-    sf[3] = 0.125 * (1 - u) * (1 + v) * (1 - w);
-    sf[4] = 0.125 * (1 - u) * (1 - v) * (1 + w);
-    sf[5] = 0.125 * (1 + u) * (1 - v) * (1 + w);
-    sf[6] = 0.125 * (1 + u) * (1 + v) * (1 + w);
-    sf[7] = 0.125 * (1 - u) * (1 + v) * (1 + w);
+    sf(0) = 0.125 * (1 - u) * (1 - v) * (1 - w);
+    sf(1) = 0.125 * (1 + u) * (1 - v) * (1 - w);
+    sf(2) = 0.125 * (1 + u) * (1 + v) * (1 - w);
+    sf(3) = 0.125 * (1 - u) * (1 + v) * (1 - w);
+    sf(4) = 0.125 * (1 - u) * (1 - v) * (1 + w);
+    sf(5) = 0.125 * (1 + u) * (1 - v) * (1 + w);
+    sf(6) = 0.125 * (1 + u) * (1 + v) * (1 + w);
+    sf(7) = 0.125 * (1 - u) * (1 + v) * (1 + w);
   }
-  static void create(int maxlevel, Double_Matrix *coeffs, Double_Matrix *eexps);
-  static void recurCreate(adaptiveHexahedron *h, int maxlevel, int level,
-			  Double_Matrix *coeffs, Double_Matrix *eexps);
+  static void create(int maxlevel);
+  static void recurCreate(adaptiveHexahedron *h, int maxlevel, int level);
   static void error(double AVG, double tol);
   static void recurError(adaptiveHexahedron *h, double AVG, double tol);
 };
 
+class PCoords { 
+ public:
+  double c[3];
+  PCoords(double x, double y, double z)
+  {
+    c[0] = x; c[1] = y; c[2] = z;
+  }
+};
+
+class PValues{
+ public:
+  double v[3];
+  PValues(double vx)
+  {
+    v[0] = vx;
+  }
+  PValues(double vx, double vy, double vz)
+  {
+    v[0] = vx; v[1] = vy; v[2] = vz;
+  }
+};
+
 template <class T>
 class adaptiveElements {
  private:
-  int _resolutionLevel;
-  double _tolerance, _minVal, _maxVal;
-  List_T *_listEle;
-  int *_numEle;
-  Double_Matrix *_coefs, *_eexps;
-  Double_Matrix *_coefsGeom, *_eexpsGeom;
-  Double_Matrix *_posX, *_posY, *_posZ;
-  Double_Matrix *_val, *_valX, *_valY, *_valZ;
-  Double_Matrix *_interpolate, *_geometry;
-  void _changeResolution(int level, GMSH_Post_Plugin *plug, int *done);
-  int _zoomElement(int ielem, int level, GMSH_Post_Plugin *plug);
+  gmshMatrix<double> *_coeffsVal, *_eexpsVal, *_interpolVal;
+  gmshMatrix<double> *_coeffsGeom, *_eexpsGeom, *_interpolGeom;
  public:
-  adaptiveElements(List_T *listEle, int *numEle,
-		   List_T *coeffs, List_T *eexps, 
-		   List_T *coeffsGeom=0, List_T *eexpsGeom=0);
+  adaptiveElements(std::vector<gmshMatrix<double>*> &interpolationMatrices);
   ~adaptiveElements();
-  void initWithLowResolution(PViewData *data, int step);
-  void changeResolution(int level, double tol, GMSH_Post_Plugin *plug=0);
-  // The number of nodes is supposed to be fixed in an adaptive view
-  inline int getNumNodes () const {return _coefsGeom ? _coefsGeom->size1() : T::numNodes;}
+  // create the _interpolVal and _interpolGeom matrices at the given
+  // refinement level
+  void init(int level);
+  // process the element data in coords/values and return the refined
+  // elements in coords/values
+  void adapt(double tol, int numComp,
+             std::vector<PCoords> &coords, std::vector<PValues> &values,
+             double &minVal, double &maxVal, GMSH_PostPlugin *plug=0,
+             bool onlyComputeMinMax=false);
+  // adapt all the T-type elements in the input view and add the
+  // refined elements in the output view (we will remove this when we
+  // switch to true on-the-fly local refinement in drawPost())
+  void addInView(double tol, int step, PViewData *in, PViewDataList *out, 
+                 GMSH_PostPlugin *plug=0);
 };
 
 class adaptiveData {
  private:
+  int _step, _level;
+  double _tol;
   PViewData *_inData;
   PViewDataList *_outData;
   adaptiveElements<adaptiveLine> *_lines;
@@ -298,11 +319,11 @@ class adaptiveData {
   adaptiveElements<adaptiveHexahedron> *_hexahedra;
   adaptiveElements<adaptivePrism> *_prisms;
  public:
+  static double timerInit, timerAdapt;
   adaptiveData(PViewData *data);
   ~adaptiveData();
   PViewData *getData(){ return (PViewData*)_outData; }
-  void initWithLowResolution(int step);
-  void changeResolution(int level, double tol, GMSH_Post_Plugin *plug=0);
+  void changeResolution(int step, int level, double tol, GMSH_PostPlugin *plug=0);
 };
 
 #endif
